@@ -25,6 +25,7 @@
 #include <glib.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-window-icon.h>
 #include <bonobo/bonobo-ui-container.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <gal/e-table/e-table-scrolled.h>
@@ -97,6 +98,8 @@ enum {
 	ARG_EDITABLE
 };
 
+static GSList *all_contact_list_editors = NULL;
+
 GtkType
 e_contact_list_editor_get_type (void)
 {
@@ -207,6 +210,7 @@ e_contact_list_editor_init (EContactListEditor *editor)
 	GladeXML *gui;
 	GtkWidget *bonobo_win;
 	BonoboUIContainer *container;
+	char *icon_path;
 
 	editor->card = NULL;
 	editor->changed = FALSE;
@@ -294,6 +298,11 @@ e_contact_list_editor_init (EContactListEditor *editor)
 
 	gtk_signal_connect (GTK_OBJECT (editor->app), "delete_event",
 			    GTK_SIGNAL_FUNC (app_delete_event_cb), editor);
+
+	/* set the icon */
+	icon_path = g_concat_dir_and_file (EVOLUTION_ICONSDIR, "contact-list-16.png");
+	gnome_window_icon_set_from_file (GTK_WINDOW (editor->app), icon_path);
+	g_free (icon_path);
 }
 
 static void
@@ -501,9 +510,9 @@ BonoboUIVerb verbs [] = {
 	BONOBO_UI_UNSAFE_VERB ("ContactListEditorSave", file_save_cb),
 	BONOBO_UI_UNSAFE_VERB ("ContactListEditorSaveClose", tb_save_and_close_cb),
 	BONOBO_UI_UNSAFE_VERB ("ContactListEditorDelete", delete_cb),
-	BONOBO_UI_UNSAFE_VERB ("ContactEditorSaveAs", file_save_as_cb),
-	BONOBO_UI_UNSAFE_VERB ("ContactEditorSendAs", file_send_as_cb),
-	BONOBO_UI_UNSAFE_VERB ("ContactEditorSendTo", file_send_to_cb),
+	BONOBO_UI_UNSAFE_VERB ("ContactListEditorSaveAs", file_save_as_cb),
+	BONOBO_UI_UNSAFE_VERB ("ContactListEditorSendAs", file_send_as_cb),
+	BONOBO_UI_UNSAFE_VERB ("ContactListEditorSendTo", file_send_to_cb),
 	BONOBO_UI_UNSAFE_VERB ("ContactListEditorClose", file_close_cb),
 	BONOBO_UI_VERB_END
 };
@@ -538,6 +547,14 @@ create_ui (EContactListEditor *ce)
 	e_pixmaps_update (ce->uic, pixmaps);
 }
 
+static void
+contact_list_editor_destroy_notify (void *data)
+{
+	EContactListEditor *ce = E_CONTACT_LIST_EDITOR (data);
+
+	all_contact_list_editors = g_slist_remove (all_contact_list_editors, ce);
+}
+
 EContactListEditor *
 e_contact_list_editor_new (EBook *book,
 			   ECard *list_card,
@@ -548,6 +565,9 @@ e_contact_list_editor_new (EBook *book,
 
 	ce = E_CONTACT_LIST_EDITOR (gtk_type_new (E_CONTACT_LIST_EDITOR_TYPE));
 
+	all_contact_list_editors = g_slist_prepend (all_contact_list_editors, ce);
+	gtk_object_weakref (GTK_OBJECT (ce), contact_list_editor_destroy_notify, ce);
+
 	gtk_object_set (GTK_OBJECT (ce),
 			"book", book,
 			"card", list_card,
@@ -978,3 +998,27 @@ fill_in_info(EContactListEditor *editor)
 		}
 	}
 }
+
+
+gboolean
+e_contact_list_editor_request_close_all (void)
+{
+	GSList *p;
+	GSList *pnext;
+	gboolean retval;
+
+	retval = TRUE;
+	for (p = all_contact_list_editors; p != NULL; p = pnext) {
+		pnext = p->next;
+
+		e_contact_list_editor_raise (E_CONTACT_LIST_EDITOR (p->data));
+		if (! prompt_to_save_changes (E_CONTACT_LIST_EDITOR (p->data))) {
+			retval = FALSE;
+			break;
+		}
+
+		close_dialog (E_CONTACT_LIST_EDITOR (p->data));
+	}
+
+	return retval;
+}
