@@ -9,10 +9,9 @@
  *          Federico Mena-Quintero <federico@ximian.com>
  *          Seth Alves <alves@hungry.com>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -46,6 +45,7 @@
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-i18n.h>
 #include <bonobo/bonobo-ui-util.h>
+#include <bonobo/bonobo-exception.h>
 #include <cal-util/timeutil.h>
 #include "shell/Evolution.h"
 #include "calendar-commands.h"
@@ -60,29 +60,13 @@
 /* A list of all of the calendars started */
 static GList *all_calendars = NULL;
 
-/* We have one global preferences dialog. */
-static CalPrefsDialog *preferences_dialog = NULL;
-
-/* Callback for the new appointment command */
-static void
-new_appointment_cb (BonoboUIComponent *uic, gpointer data, const char *path)
-{
-	GnomeCalendar *gcal;
-
-	gcal = GNOME_CALENDAR (data);
-	gnome_calendar_new_appointment (gcal);
-}
-
-static void
-new_event_cb (BonoboUIComponent *uic, gpointer data, const char *path)
-{
-	GnomeCalendar *gcal;
-	time_t dtstart, dtend;
-
-	gcal = GNOME_CALENDAR (data);
-	gnome_calendar_get_current_time_range (gcal, &dtstart, &dtend);
-	gnome_calendar_new_appointment_for (gcal, dtstart, dtend, TRUE);
-}
+/* Focusing information for the calendar view.  We have to keep track of this
+ * ourselves because with Bonobo controls, we may get unpaired focus_out events.
+ */
+typedef struct {
+	guint calendar_focused : 1;
+	guint taskpad_focused : 1;
+} FocusData;
 
 /* Prints the calendar at its current view and time range */
 static void
@@ -263,16 +247,7 @@ show_month_view_clicked (BonoboUIComponent *uic, gpointer data, const char *path
 
 
 static void
-settings_cmd (BonoboUIComponent *uic, gpointer data, const char *path)
-{
-	if (!preferences_dialog)
-		preferences_dialog = cal_prefs_dialog_new (CAL_PREFS_DIALOG_PAGE_CALENDAR);
-	else
-		cal_prefs_dialog_show (preferences_dialog, CAL_PREFS_DIALOG_PAGE_CALENDAR);
-}
-
-static void
-cut_event_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
+cut_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
 {
 	GnomeCalendar *gcal;
 
@@ -283,7 +258,7 @@ cut_event_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
 }
 
 static void
-copy_event_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
+copy_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
 {
 	GnomeCalendar *gcal;
 
@@ -295,7 +270,7 @@ copy_event_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
 }
 
 static void
-paste_event_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
+paste_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
 {
 	GnomeCalendar *gcal;
 
@@ -306,22 +281,47 @@ paste_event_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
 	set_normal_cursor (gcal);
 }
 
+static void
+delete_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
+{
+	GnomeCalendar *gcal;
+
+	gcal = GNOME_CALENDAR (data);
+
+	set_clock_cursor (gcal);
+	gnome_calendar_delete_selection (gcal);
+	set_normal_cursor (gcal);
+}
+
 static void
 publish_freebusy_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
 {
 	GnomeCalendar *gcal;
 	CalClient *client;
-	CalClientGetStatus status;
-	CalComponent *comp;
-	time_t start, end;
-	
+	GList *comp_list;
+	icaltimezone *utc;
+	time_t start = time (NULL), end;
+
 	gcal = GNOME_CALENDAR (data);
-	gnome_calendar_get_current_time_range (gcal, &start, &end);
+
+	utc = icaltimezone_get_utc_timezone ();
+	start = time_day_begin_with_zone (start, utc);
+	end = time_add_week_with_zone (start, 6, utc);
 
 	client = gnome_calendar_get_cal_client (gcal);
-	status = cal_client_get_free_busy (client, start, end, &comp);
-	if (status == CAL_CLIENT_GET_SUCCESS)
-		itip_send_comp (CAL_COMPONENT_METHOD_PUBLISH, comp);
+	comp_list = cal_client_get_free_busy (client, NULL, start, end);
+	if (comp_list) {
+		GList *l;
+
+		for (l = comp_list; l; l = l->next) {
+			CalComponent *comp = CAL_COMPONENT (l->data);
+			itip_send_comp (CAL_COMPONENT_METHOD_PUBLISH, comp, client, NULL);
+
+			gtk_object_unref (GTK_OBJECT (comp));
+		}
+
+ 		g_list_free (comp_list);
+	}
 }
 
 /* Does a queryInterface on the control's parent control frame for the ShellView interface */
@@ -340,7 +340,7 @@ get_shell_view_interface (BonoboControl *control)
 	shell_view = Bonobo_Unknown_queryInterface (control_frame,
 						    "IDL:GNOME/Evolution/ShellView:1.0",
 						    &ev);
-	if (ev._major != CORBA_NO_EXCEPTION) {
+	if (BONOBO_EX (&ev)) {
 		g_message ("get_shell_view_interface(): "
 			   "Could not queryInterface() on the control frame");
 		shell_view = CORBA_OBJECT_NIL;
@@ -356,11 +356,9 @@ get_shell_view_interface (BonoboControl *control)
 
 /* Displays the currently displayed time range in the folder bar label on the
    shell view, according to which view we are showing. */
-static void
-clear_folder_bar_label (GnomeCalendar *gcal, BonoboControl *control)
+void
+calendar_set_folder_bar_label (GnomeCalendar *gcal, BonoboControl *control)
 {
-	GNOME_Evolution_ShellView shell_view;
-	CORBA_Environment ev;
 	icaltimezone *zone;
 	struct icaltimetype start_tt, end_tt;
 	time_t start_time, end_time;
@@ -368,8 +366,7 @@ clear_folder_bar_label (GnomeCalendar *gcal, BonoboControl *control)
 	char buffer[512], end_buffer[256];
 	GnomeCalendarViewType view;
 
-	/* FIXME: This isn't the same as the currently visible time range. */
-	gnome_calendar_get_selected_time_range (gcal, &start_time, &end_time);
+	gnome_calendar_get_visible_time_range (gcal, &start_time, &end_time);
 	zone = gnome_calendar_get_timezone (gcal);
 
 	start_tt = icaltime_from_timet_with_zone (start_time, FALSE, zone);
@@ -383,7 +380,8 @@ clear_folder_bar_label (GnomeCalendar *gcal, BonoboControl *control)
 	start_tm.tm_wday = time_day_of_week (start_tt.day, start_tt.month - 1,
 					     start_tt.year);
 
-	end_tt = icaltime_from_timet_with_zone (end_time, FALSE, zone);
+	/* Take one off end_time so we don't get an extra day. */
+	end_tt = icaltime_from_timet_with_zone (end_time - 1, FALSE, zone);
 	end_tm.tm_year = end_tt.year - 1900;
 	end_tm.tm_mon = end_tt.month - 1;
 	end_tm.tm_mday = end_tt.day;
@@ -398,12 +396,14 @@ clear_folder_bar_label (GnomeCalendar *gcal, BonoboControl *control)
 
 	switch (view) {
 	case GNOME_CAL_DAY_VIEW:
-		strftime (buffer, sizeof (buffer),
-			  _("%A %d %B %Y"), &start_tm);
-		break;
 	case GNOME_CAL_WORK_WEEK_VIEW:
 	case GNOME_CAL_WEEK_VIEW:
-		if (start_tm.tm_year == end_tm.tm_year) {
+		if (start_tm.tm_year == end_tm.tm_year
+		    && start_tm.tm_mon == end_tm.tm_mon
+		    && start_tm.tm_mday == end_tm.tm_mday) {
+			strftime (buffer, sizeof (buffer),
+				  _("%A %d %B %Y"), &start_tm);
+		} else if (start_tm.tm_year == end_tm.tm_year) {
 			strftime (buffer, sizeof (buffer),
 				  _("%a %d %b"), &start_tm);
 			strftime (end_buffer, sizeof (end_buffer),
@@ -421,17 +421,26 @@ clear_folder_bar_label (GnomeCalendar *gcal, BonoboControl *control)
 		break;
 	case GNOME_CAL_MONTH_VIEW:
 		if (start_tm.tm_year == end_tm.tm_year) {
-			strftime (buffer, sizeof (buffer),
-				  _("%B"), &start_tm);
-			strftime (end_buffer, sizeof (end_buffer),
-				  _("%B %Y"), &end_tm);
-			strcat (buffer, " - ");
-			strcat (buffer, end_buffer);
+			if (start_tm.tm_mon == end_tm.tm_mon) {
+				strftime (buffer, sizeof (buffer),
+					  "%d", &start_tm);
+				strftime (end_buffer, sizeof (end_buffer),
+					  _("%d %B %Y"), &end_tm);
+				strcat (buffer, " - ");
+				strcat (buffer, end_buffer);
+			} else {
+				strftime (buffer, sizeof (buffer),
+					  _("%d %B"), &start_tm);
+				strftime (end_buffer, sizeof (end_buffer),
+					  _("%d %B %Y"), &end_tm);
+				strcat (buffer, " - ");
+				strcat (buffer, end_buffer);
+			}
 		} else {
 			strftime (buffer, sizeof (buffer),
-				  _("%B %Y"), &start_tm);
+				  _("%d %B %Y"), &start_tm);
 			strftime (end_buffer, sizeof (end_buffer),
-				  _("%B %Y"), &end_tm);
+				  _("%d %B %Y"), &end_tm);
 			strcat (buffer, " - ");
 			strcat (buffer, end_buffer);
 		}
@@ -440,31 +449,213 @@ clear_folder_bar_label (GnomeCalendar *gcal, BonoboControl *control)
 		g_assert_not_reached ();
 	}
 
+	control_util_set_folder_bar_label (control, buffer);
+}
+
+void
+control_util_set_folder_bar_label (BonoboControl *control, char *label)
+{
+	GNOME_Evolution_ShellView shell_view;
+	CORBA_Environment ev;
+
 	shell_view = get_shell_view_interface (control);
 	if (shell_view == CORBA_OBJECT_NIL)
 		return;
 
 	CORBA_exception_init (&ev);
-#if 0
-	GNOME_Evolution_ShellView_setFolderBarLabel (shell_view, buffer, &ev);
-#endif
-	GNOME_Evolution_ShellView_setFolderBarLabel (shell_view, "", &ev);
-	if (ev._major != CORBA_NO_EXCEPTION)
-		g_message ("clear_folder_bar_label(): Could not set the folder bar label");
+	GNOME_Evolution_ShellView_setFolderBarLabel (shell_view, label, &ev);
+
+	if (BONOBO_EX (&ev))
+		g_message ("control_util_set_folder_bar_label(): Could not set the folder bar label");
 
 	CORBA_exception_free (&ev);
 }
 
+void
+control_util_show_settings (GnomeCalendar *gcal)
+{
+	BonoboControl *control;
+	GNOME_Evolution_ShellView shell_view;
+	CORBA_Environment ev;
+
+	control = gtk_object_get_data (GTK_OBJECT (gcal), "control");
+	if (control == NULL)
+		return;
+
+	shell_view = get_shell_view_interface (control);
+	if (shell_view == CORBA_OBJECT_NIL)
+		return;
+
+	CORBA_exception_init (&ev);
+	
+	GNOME_Evolution_ShellView_showSettings (shell_view, &ev);
+	
+	if (BONOBO_EX (&ev))
+		g_message ("control_util_show_settings(): Could not show settings");
+
+	CORBA_exception_free (&ev);
+}
+
+/* Sensitizes the UI Component menu/toolbar calendar commands based on the
+ * number of selected events. (This will always be 0 or 1 currently.)  If enable
+ * is FALSE, all will be disabled.  Otherwise, the currently-selected number of
+ * events will be used.
+ */
+static void
+sensitize_calendar_commands (GnomeCalendar *gcal, BonoboControl *control, gboolean enable)
+{
+	BonoboUIComponent *uic;
+	int n_selected;
+	
+	uic = bonobo_control_get_ui_component (control);
+	g_assert (uic != NULL);
+
+	n_selected = enable ? gnome_calendar_get_num_events_selected (gcal) : 0;
+
+	bonobo_ui_component_set_prop (uic, "/commands/Cut", "sensitive",
+				      n_selected == 0 ? "0" : "1",
+				      NULL);
+	bonobo_ui_component_set_prop (uic, "/commands/Copy", "sensitive",
+				      n_selected == 0 ? "0" : "1",
+				      NULL);
+	bonobo_ui_component_set_prop (uic, "/commands/Paste", "sensitive",
+				      enable ? "1" : "0",
+				      NULL);
+	bonobo_ui_component_set_prop (uic, "/commands/Delete", "sensitive",
+				      n_selected == 0 ? "0" : "1",
+				      NULL);
+}
+
+/* Sensitizes the UI Component menu/toolbar tasks commands based on the number
+ * of selected tasks.  If enable is FALSE, all will be disabled.  Otherwise, the
+ * currently-selected number of tasks will be used.
+ */
+static void
+sensitize_taskpad_commands (GnomeCalendar *gcal, BonoboControl *control, gboolean enable)
+{
+	BonoboUIComponent *uic;
+	int n_selected;
+	
+	uic = bonobo_control_get_ui_component (control);
+	g_assert (uic != NULL);
+
+	n_selected = enable ? gnome_calendar_get_num_tasks_selected (gcal) : 0;
+
+	bonobo_ui_component_set_prop (uic, "/commands/Cut", "sensitive",
+				      n_selected == 0 ? "0" : "1",
+				      NULL);
+	bonobo_ui_component_set_prop (uic, "/commands/Copy", "sensitive",
+				      n_selected == 0 ? "0" : "1",
+				      NULL);
+	bonobo_ui_component_set_prop (uic, "/commands/Paste", "sensitive",
+				      enable ? "1" : "0",
+				      NULL);
+	bonobo_ui_component_set_prop (uic, "/commands/Delete", "sensitive",
+				      n_selected == 0 ? "0" : "1",
+				      NULL);
+}
+
+/* Callback used when the dates shown by the GnomeCalendar are changed.
+   We want to update the dates in the folder bar. */
+static void
+gcal_calendar_dates_change_cb (GnomeCalendar *gcal, gpointer data)
+{
+	BonoboControl *control;
+
+	control = BONOBO_CONTROL (data);
+
+	calendar_set_folder_bar_label (gcal, control);
+}
+
+/* Callback used when the selection in the calendar views changes */
+static void
+gcal_calendar_selection_changed_cb (GnomeCalendar *gcal, gpointer data)
+{
+	BonoboControl *control;
+
+	control = BONOBO_CONTROL (data);
+
+	sensitize_calendar_commands (gcal, control, TRUE);
+}
+
+/* Callback used when the selection in the taskpad changes */
+static void
+gcal_taskpad_selection_changed_cb (GnomeCalendar *gcal, gpointer data)
+{
+	BonoboControl *control;
+
+	control = BONOBO_CONTROL (data);
+
+	sensitize_taskpad_commands (gcal, control, TRUE);
+}
+
+/* Callback used when the focus changes for a calendar view */
+static void
+gcal_calendar_focus_change_cb (GnomeCalendar *gcal, gboolean in, gpointer data)
+{
+	BonoboControl *control;
+	FocusData *focus;
+
+	control = BONOBO_CONTROL (data);
+
+	focus = gtk_object_get_data (GTK_OBJECT (control), "focus_data");
+	g_assert (focus != NULL);
+
+	if (in) {
+		gtk_signal_connect (GTK_OBJECT (gcal), "calendar_selection_changed",
+				    GTK_SIGNAL_FUNC (gcal_calendar_selection_changed_cb), control);
+		sensitize_calendar_commands (gcal, control, TRUE);
+		focus->calendar_focused = TRUE;
+	} else if (focus->calendar_focused) {
+		gtk_signal_disconnect_by_func (GTK_OBJECT (gcal),
+					       GTK_SIGNAL_FUNC (gcal_calendar_selection_changed_cb),
+					       control);
+		sensitize_calendar_commands (gcal, control, FALSE);
+		focus->calendar_focused = FALSE;
+	}
+}
+
+/* Callback used when the taskpad focus changes */
+static void
+gcal_taskpad_focus_change_cb (GnomeCalendar *gcal, gboolean in, gpointer data)
+{
+	BonoboControl *control;
+	FocusData *focus;
+
+	control = BONOBO_CONTROL (data);
+
+	focus = gtk_object_get_data (GTK_OBJECT (control), "focus_data");
+	g_assert (focus != NULL);
+
+	if (in) {
+		gtk_signal_connect (GTK_OBJECT (gcal), "taskpad_selection_changed",
+				    GTK_SIGNAL_FUNC (gcal_taskpad_selection_changed_cb), control);
+		sensitize_taskpad_commands (gcal, control, TRUE);
+		focus->taskpad_focused = TRUE;
+	} else if (focus->taskpad_focused) {
+		/* With Bonobo controls, we may get unpaired focus_out events.
+		 * That is why we have to keep track of this ourselves instead
+		 * of blindly assumming that we are getting this event because
+		 * the taskpad was in fact focused.
+		 */
+		gtk_signal_disconnect_by_func (GTK_OBJECT (gcal),
+					       GTK_SIGNAL_FUNC (gcal_taskpad_selection_changed_cb),
+					       control);
+		sensitize_taskpad_commands (gcal, control, FALSE);
+		focus->taskpad_focused = FALSE;
+	}
+
+}
+
+
 static BonoboUIVerb verbs [] = {
 	BONOBO_UI_VERB ("CalendarPrint", file_print_cb),
 	BONOBO_UI_VERB ("CalendarPrintPreview", file_print_preview_cb),
-	BONOBO_UI_VERB ("CalendarNewAppointment", new_appointment_cb),
-	BONOBO_UI_VERB ("CalendarNewEvent", new_event_cb),
-	BONOBO_UI_VERB ("CalendarSettings", settings_cmd),
 
-	BONOBO_UI_VERB ("CutEvent", cut_event_cmd),
-	BONOBO_UI_VERB ("CopyEvent", copy_event_cmd),
-	BONOBO_UI_VERB ("PasteEvent", paste_event_cmd),
+	BONOBO_UI_VERB ("Cut", cut_cmd),
+	BONOBO_UI_VERB ("Copy", copy_cmd),
+	BONOBO_UI_VERB ("Paste", paste_cmd),
+	BONOBO_UI_VERB ("Delete", delete_cmd),
 
 	BONOBO_UI_VERB ("CalendarPrev", previous_clicked),
 	BONOBO_UI_VERB ("CalendarToday", today_clicked),
@@ -483,18 +674,25 @@ static BonoboUIVerb verbs [] = {
 
 static EPixmap pixmaps [] =
 {
-	E_PIXMAP ("/menu/File/New/NewFirstItem/NewAppointment",	"new_appointment.xpm"),
-	E_PIXMAP ("/menu/File/Print/Print",			"print.xpm"),
-	E_PIXMAP ("/menu/File/Print/PrintPreview",		"print-preview.xpm"),
-	E_PIXMAP ("/menu/Actions/Component/NewAppointment",	"new_appointment.xpm"),
-	E_PIXMAP ("/menu/Tools/Component/CalendarSettings",	"configure_16_calendar.xpm"),
+	E_PIXMAP ("/menu/EditPlaceholder/Edit/Cut",			      "16_cut.png"),
+	E_PIXMAP ("/menu/EditPlaceholder/Edit/Copy",			      "16_copy.png"),
+	E_PIXMAP ("/menu/EditPlaceholder/Edit/Paste",			      "16_paste.png"),
+	E_PIXMAP ("/menu/EditPlaceholder/Edit/Delete",			      "evolution-trash-mini.png"),
+	E_PIXMAP ("/menu/File/Print/Print",				      "print.xpm"),
+	E_PIXMAP ("/menu/File/Print/PrintPreview",			      "print-preview.xpm"),
+	E_PIXMAP ("/menu/View/ViewBegin/Goto",				      "goto-16.png"),
 
-	E_PIXMAP ("/Toolbar/New",				"buttons/new_appointment.png"),
-	E_PIXMAP ("/Toolbar/Print",				"buttons/print.png"),
-	E_PIXMAP ("/Toolbar/DayView",				"buttons/dayview.xpm"),
-	E_PIXMAP ("/Toolbar/WorkWeekView",			"buttons/workweekview.xpm"),
-	E_PIXMAP ("/Toolbar/WeekView",				"buttons/weekview.xpm"),
-	E_PIXMAP ("/Toolbar/MonthView",				"buttons/monthview.xpm"),
+	E_PIXMAP ("/Toolbar/New",					      "buttons/new_appointment.png"),
+	E_PIXMAP ("/Toolbar/NewTask",					      "buttons/new_task.png"),
+	E_PIXMAP ("/Toolbar/Print",					      "buttons/print.png"),
+	E_PIXMAP ("/Toolbar/Delete",					      "buttons/delete-message.png"),
+	E_PIXMAP ("/Toolbar/Prev",					      "buttons/arrow-left-24.png"),
+	E_PIXMAP ("/Toolbar/Next",					      "buttons/arrow-right-24.png"),
+	E_PIXMAP ("/Toolbar/Goto",					      "buttons/goto-24.png"),
+	E_PIXMAP ("/Toolbar/DayView",					      "buttons/dayview.xpm"),
+	E_PIXMAP ("/Toolbar/WorkWeekView",				      "buttons/workweekview.xpm"),
+	E_PIXMAP ("/Toolbar/WeekView",					      "buttons/weekview.xpm"),
+	E_PIXMAP ("/Toolbar/MonthView",					      "buttons/monthview.xpm"),
 
 	E_PIXMAP_END
 };
@@ -505,6 +703,7 @@ calendar_control_activate (BonoboControl *control,
 {
 	Bonobo_UIContainer remote_uih;
 	BonoboUIComponent *uic;
+	FocusData *focus;
 
 	uic = bonobo_control_get_ui_component (control);
 	g_assert (uic != NULL);
@@ -513,21 +712,7 @@ calendar_control_activate (BonoboControl *control,
 	bonobo_ui_component_set_container (uic, remote_uih);
 	bonobo_object_release_unref (remote_uih, NULL);
 
-#if 0
-	/* FIXME: Need to update this to use new Bonobo ui stuff somehow.
-	   Also need radio buttons really. */
-
-	/* Note that these indices should correspond with the button indices
-	   in the gnome_toolbar_view_buttons UIINFO struct. */
-	gnome_calendar_set_view_buttons (cal,
-					 gnome_toolbar_view_buttons[0].widget,
-					 gnome_toolbar_view_buttons[1].widget,
-					 gnome_toolbar_view_buttons[2].widget,
-					 gnome_toolbar_view_buttons[3].widget);
-
-	/* This makes the appropriate radio button in the toolbar active. */
-	gnome_calendar_update_view_buttons (cal);
-#endif
+	gnome_calendar_set_ui_component (gcal, uic);
 
 	bonobo_ui_component_add_verb_list_with_data (uic, verbs, gcal);
 
@@ -541,23 +726,56 @@ calendar_control_activate (BonoboControl *control,
 
 	gnome_calendar_setup_view_menus (gcal, uic);
 
+	gtk_signal_connect (GTK_OBJECT (gcal), "dates_shown_changed",
+			    GTK_SIGNAL_FUNC (gcal_calendar_dates_change_cb),
+			    control);
+	gtk_signal_connect (GTK_OBJECT (gcal), "calendar_focus_change",
+			    GTK_SIGNAL_FUNC (gcal_calendar_focus_change_cb), control);
+	gtk_signal_connect (GTK_OBJECT (gcal), "taskpad_focus_change",
+			    GTK_SIGNAL_FUNC (gcal_taskpad_focus_change_cb), control);
+
+	sensitize_calendar_commands (gcal, control, FALSE);
+	sensitize_taskpad_commands (gcal, control, FALSE);
+
 	bonobo_ui_component_thaw (uic, NULL);
 
 	/* Show the dialog for setting the timezone if the user hasn't chosen
-	   a default timezone already. */
+	   a default timezone already. This is done in the startup wizard now,
+	   so we don't do it here. */
+#if 0
 	calendar_config_check_timezone_set ();
+#endif
 
-	clear_folder_bar_label (gcal, control);
+	calendar_set_folder_bar_label (gcal, control);
+
+	focus = g_new (FocusData, 1);
+	focus->calendar_focused = FALSE;
+	focus->taskpad_focused = FALSE;
+
+	gtk_object_set_data (GTK_OBJECT (control), "focus_data", focus);
 }
 
 void
 calendar_control_deactivate (BonoboControl *control, GnomeCalendar *gcal)
 {
-	BonoboUIComponent *uic = bonobo_control_get_ui_component (control);
+	FocusData *focus;
+	BonoboUIComponent *uic;
+
+	uic = bonobo_control_get_ui_component (control);
 	g_assert (uic != NULL);
 
+	gnome_calendar_set_ui_component (gcal, uic);
+
+	focus = gtk_object_get_data (GTK_OBJECT (control), "focus_data");
+	g_assert (focus != NULL);
+
+	gtk_object_remove_data (GTK_OBJECT (control), "focus_data");
+	g_free (focus);
+
 	gnome_calendar_discard_view_menus (gcal);
 
+	gtk_signal_disconnect_by_data (GTK_OBJECT (gcal), control);
+
 	bonobo_ui_component_rm (uic, "/", NULL);
  	bonobo_ui_component_unset_container (uic);
 }
@@ -569,13 +787,6 @@ on_calendar_destroyed (GnomeCalendar *gcal)
 	all_calendars = g_list_remove (all_calendars, gcal);
 }
 
-static void
-on_calendar_dates_shown_changed (GnomeCalendar *gcal)
-{
-	g_print ("In on_calendar_dates_shown_changed\n");
-}
-
-
 GnomeCalendar *
 new_calendar (void)
 {
@@ -590,9 +801,6 @@ new_calendar (void)
 
 	gtk_signal_connect (GTK_OBJECT (gcal), "destroy",
 			    GTK_SIGNAL_FUNC (on_calendar_destroyed), NULL);
-	gtk_signal_connect (GTK_OBJECT (gcal), "dates_shown_changed",
-			    GTK_SIGNAL_FUNC (on_calendar_dates_shown_changed),
-			    NULL);
 
 	all_calendars = g_list_prepend (all_calendars, gcal);
 
