@@ -635,21 +635,27 @@ is_complete (CalComponent *comp)
 	return retval;
 }
 
-/* Returns whether a component is overdue.  Sigh, this is very similar to
- * get_color() below.
- */
-static gboolean
-is_overdue (CalendarModel *model, CalComponent *comp)
+typedef enum {
+	CALENDAR_MODEL_DUE_NEVER,
+	CALENDAR_MODEL_DUE_FUTURE,
+	CALENDAR_MODEL_DUE_TODAY,
+	CALENDAR_MODEL_DUE_OVERDUE,
+	CALENDAR_MODEL_DUE_COMPLETE
+} CalendarModelDueStatus;
+
+
+static CalendarModelDueStatus
+get_due_status (CalendarModel *model, CalComponent *comp)
 {
 	CalComponentDateTime dt;
-	gboolean retval;
+	CalendarModelDueStatus retval;
 
 	cal_component_get_due (comp, &dt);
 
 	/* First, do we have a due date? */
 
 	if (!dt.value)
-		retval = FALSE;
+		retval = CALENDAR_MODEL_DUE_NEVER;
 	else {
 		struct icaltimetype now_tt;
 		CalClientGetStatus status;
@@ -658,22 +664,39 @@ is_overdue (CalendarModel *model, CalComponent *comp)
 		/* Second, is it already completed? */
 
 		if (is_complete (comp)) {
-			retval = FALSE;
+			retval = CALENDAR_MODEL_DUE_COMPLETE;
 			goto out;
 		}
 
 		/* Third, are we overdue as of right now? */
 
-		/* Get the current time in the same timezone as the DUE date.*/
-		/* FIXME: TIMEZONES: Handle error. */
-		status = cal_client_get_timezone (model->priv->client, dt.tzid,
-						  &zone);
-		now_tt = icaltime_current_time_with_zone (zone);
+		if (dt.value->is_date) {
+			int cmp;
+			
+			now_tt = icaltime_today ();
+			cmp = icaltime_compare_date_only (*dt.value, now_tt);
+			
+			if (cmp < 0)
+				retval = CALENDAR_MODEL_DUE_OVERDUE;
+			else if (cmp == 0)
+				retval = CALENDAR_MODEL_DUE_TODAY;
+			else
+				retval = CALENDAR_MODEL_DUE_FUTURE;
+		} else {
+			/* Get the current time in the same timezone as the DUE date.*/
+			/* FIXME: TIMEZONES: Handle error. */
+			status = cal_client_get_timezone (model->priv->client, dt.tzid,
+							  &zone);
+			now_tt = icaltime_current_time_with_zone (zone);
 
-		if (icaltime_compare (*dt.value, now_tt) <= 0)
-			retval = TRUE;
-		else
-			retval = FALSE;
+			if (icaltime_compare (*dt.value, now_tt) <= 0) 
+				retval = CALENDAR_MODEL_DUE_OVERDUE;
+			else
+				if (icaltime_compare_date_only (*dt.value, now_tt) == 0)
+					retval = CALENDAR_MODEL_DUE_TODAY;
+				else
+					retval = CALENDAR_MODEL_DUE_FUTURE;
+		}
 	}
 
  out:
@@ -683,60 +706,39 @@ is_overdue (CalendarModel *model, CalComponent *comp)
 	return retval;
 }
 
+/* Returns whether a component is overdue. */
+static gboolean
+is_overdue (CalendarModel *model, CalComponent *comp)
+{
+	switch (get_due_status (model, comp)) {
+	case CALENDAR_MODEL_DUE_NEVER:
+	case CALENDAR_MODEL_DUE_FUTURE:
+	case CALENDAR_MODEL_DUE_COMPLETE:
+		return FALSE;
+	case CALENDAR_MODEL_DUE_TODAY:
+	case CALENDAR_MODEL_DUE_OVERDUE:
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 /* Computes the color to be used to display a component */
 static const char *
 get_color (CalendarModel *model, CalComponent *comp)
 {
-	CalComponentDateTime dt;
-	const char *retval;
-
-	cal_component_get_due (comp, &dt);
-
-	/* First, do we have a due date? */
-
-	if (!dt.value)
-		retval = NULL;
-	else {
-		struct icaltimetype now_tt;
-		CalClientGetStatus status;
-		icaltimezone *zone;
-
-		/* Second, is it already completed? */
-
-		if (is_complete (comp)) {
-			retval = NULL;
-			goto out;
-		}
-
-		/* Third, is it due today? */
-
-		/* Get the current time in the same timezone as the DUE date.*/
-		/* FIXME: TIMEZONES: Handle error. */
-		status = cal_client_get_timezone (model->priv->client, dt.tzid,
-						  &zone);
-		now_tt = icaltime_current_time_with_zone (zone);
-
-		if (icaltime_compare_date_only (*dt.value, now_tt) == 0) {
-			retval = calendar_config_get_tasks_due_today_color ();
-			goto out;
-		}
-
-		/* Fourth, are we overdue as of right now?  We use <= in the
-		 * comparison below so that the table entries change color
-		 * immediately.
-		 */
-
-		if (icaltime_compare (*dt.value, now_tt) <= 0)
-			retval = calendar_config_get_tasks_overdue_color ();
-		else
-			retval = NULL;
+	switch (get_due_status (model, comp)) {
+	case CALENDAR_MODEL_DUE_NEVER:
+	case CALENDAR_MODEL_DUE_FUTURE:
+	case CALENDAR_MODEL_DUE_COMPLETE:
+		return NULL;
+	case CALENDAR_MODEL_DUE_TODAY:
+		return calendar_config_get_tasks_due_today_color ();
+	case CALENDAR_MODEL_DUE_OVERDUE:
+		return calendar_config_get_tasks_overdue_color ();
 	}
 
- out:
-
-	cal_component_free_datetime (&dt);
-
-	return retval;
+	return NULL;
 }
 
 static void *
@@ -839,27 +841,14 @@ calendar_model_value_at (ETableModel *etm, int col, int row)
 	case CAL_COMPONENT_FIELD_ICON:
 	{
 		ItipAddress *ia;		
-		CalComponentOrganizer organizer;		
 		GSList *attendees = NULL, *sl;		
 		gint retval = 0;
 
 		if (cal_component_has_recurrences (comp))
 			return GINT_TO_POINTER (1);
-		
-		cal_component_get_organizer (comp, &organizer);
-		if (organizer.value != NULL) {
-			GList *l;
-			const char *text = itip_strip_mailto (organizer.value);
-			
-			for (l = priv->addresses; l != NULL; l = l->next) {
-				ia = l->data;
-				
-				if (!strcmp (text, ia->address)) {
-					retval = 3;
-					goto cleanup;
-				}
-			}
-		}		
+
+		if (itip_organizer_is_user (comp))
+			return GINT_TO_POINTER (3);
 		
 		cal_component_get_attendee_list (comp, &attendees);
 		for (sl = attendees; sl != NULL; sl = sl->next) {
@@ -1001,6 +990,7 @@ set_classification (CalComponent *comp,
 static void
 set_completed (CalendarModel *model, CalComponent *comp, const void *value)
 {
+	CalendarModelPrivate *priv = model->priv;
 	ECellDateEditValue *dv = (ECellDateEditValue*) value;
 
 	if (!dv) {
@@ -1008,10 +998,17 @@ set_completed (CalendarModel *model, CalComponent *comp, const void *value)
 	} else {
 		time_t t;
 
-		/* We assume that COMPLETED is entered in the current timezone,
-		   even though it gets stored in UTC. */
-		t = icaltime_as_timet_with_zone (dv->tt, dv->zone);
-
+		if (dv->tt.is_date) {
+			/* If its a date, it will be floating, 
+			   but completed needs a date time value */
+			dv->tt.is_date = FALSE;
+			t = icaltime_as_timet_with_zone (dv->tt, priv->zone);
+		} else {
+			/* We assume that COMPLETED is entered in the current timezone,
+			   even though it gets stored in UTC. */
+			t = icaltime_as_timet_with_zone (dv->tt, dv->zone);
+		}
+		
 		ensure_task_complete (comp, t);
 	}
 }
@@ -1099,8 +1096,11 @@ set_percent (CalComponent *comp, const void *value)
 
 		if (percent == 100)
 			ensure_task_complete (comp, -1);
-		else
+		else {
 			ensure_task_not_complete (comp);
+			if (percent > 0)
+				cal_component_set_status (comp, ICAL_STATUS_INPROCESS);
+		}
 	}
 }
 
@@ -1207,6 +1207,10 @@ set_status (CalComponent *comp, const char *value)
 		percent = 0;
 		cal_component_set_percent (comp, &percent);
 		cal_component_set_completed (comp, NULL);
+	} else if (status == ICAL_STATUS_INPROCESS) {	
+		ensure_task_not_complete (comp);	
+		percent = 50;
+		cal_component_set_percent (comp, &percent);
 	} else if (status == ICAL_STATUS_COMPLETED) {
 		ensure_task_complete (comp, -1);
 	}
@@ -1320,7 +1324,7 @@ calendar_model_set_value_at (ETableModel *etm, int col, int row, const void *val
 		return;
 	}
 
-	if (!cal_client_update_object (priv->client, comp))
+	if (cal_client_update_object (priv->client, comp) != CAL_CLIENT_RESULT_SUCCESS)
 		g_message ("calendar_model_set_value_at(): Could not update the object!");
 }
 
@@ -1399,7 +1403,7 @@ calendar_model_append_row (ETableModel *etm, ETableModel *source, gint row)
 	set_complete (comp, e_table_model_value_at(source, CAL_COMPONENT_FIELD_COMPLETE, row));
 	set_status (comp, e_table_model_value_at(source, CAL_COMPONENT_FIELD_STATUS, row));
 
-	if (!cal_client_update_object (priv->client, comp)) {
+	if (cal_client_update_object (priv->client, comp) != CAL_CLIENT_RESULT_SUCCESS) {
 		/* FIXME: Show error dialog. */
 		g_message ("calendar_model_append_row(): Could not add new object!");
 	}
@@ -1891,7 +1895,7 @@ query_query_done_cb (CalQuery *query, CalQueryDoneStatus status, const char *err
 	calendar_model_set_status_message (model, NULL);
 
 	if (status != CAL_QUERY_DONE_SUCCESS)
-		fprintf (stderr, "query done: %s\n", error_str);
+		g_warning ("query done: %s\n", error_str);
 }
 
 /* Callback used when an evaluation error occurs when running a query */
@@ -1906,7 +1910,7 @@ query_eval_error_cb (CalQuery *query, const char *error_str, gpointer data)
 
 	calendar_model_set_status_message (model, NULL);
 
-	fprintf (stderr, "eval error: %s\n", error_str);
+	g_warning ("eval error: %s\n", error_str);
 }
 
 /* Builds a complete query sexp for the calendar model by adding the predicates
@@ -1987,6 +1991,7 @@ update_query (CalendarModel *model)
 
 	if (!priv->query) {
 		g_message ("update_query(): Could not create the query");
+		calendar_model_set_status_message (model, NULL);
 		return;
 	}
 
@@ -2269,7 +2274,7 @@ calendar_model_mark_task_complete (CalendarModel *model,
 
 	ensure_task_complete (comp, -1);
 
-	if (!cal_client_update_object (priv->client, comp))
+	if (cal_client_update_object (priv->client, comp) != CAL_CLIENT_RESULT_SUCCESS)
 		g_message ("calendar_model_mark_task_complete(): Could not update the object!");
 }
 
