@@ -26,21 +26,28 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <gnome.h>
-#include <bonobo/bonobo-win.h>
-#include <bonobo/bonobo-ui-component.h>
+#include <glib.h>
+#include <gdk/gdkkeysyms.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-uidefs.h>
+#include <libgnomeui/gnome-dialog.h>
+#include <libgnomeui/gnome-dialog-util.h>
+#include <libgnomeui/gnome-stock.h>
+#include <libgnomeui/gnome-window-icon.h>
+#include <libgnomeui/gnome-messagebox.h>
 #include <bonobo/bonobo-ui-container.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <gal/widgets/e-unicode.h>
-#include <libgnomeui/gnome-dialog.h>
-#include <libgnomeui/gnome-dialog-util.h>
-#include <libgnomeui/gnome-window-icon.h>
+#include <gal/widgets/e-gui-utils.h>
+#include <e-util/e-dialog-utils.h>
 #include <evolution-shell-component-utils.h>
 #include "../print.h"
 #include "save-comp.h"
 #include "delete-comp.h"
 #include "send-comp.h"
 #include "changed-comp.h"
+#include "cancel-comp.h"
 #include "comp-editor.h"
 
 
@@ -56,8 +63,7 @@ struct _CompEditorPrivate {
 	/* The pages we have */
 	GList *pages;
 
-	/* Toplevel window for the dialog */
-	GtkWidget *window;
+	/* UI Component for the dialog */
 	BonoboUIComponent *uic;
 
 	/* Notebook to hold the pages */
@@ -68,11 +74,11 @@ struct _CompEditorPrivate {
 	gboolean changed;
 	gboolean needs_send;
 
-	gboolean existing_org;
-	gboolean user_org;
-
-	gboolean warned;
+ 	gboolean existing_org;
+ 	gboolean user_org;
 	
+ 	gboolean warned;
+ 	
 	gboolean updating;
 };
 
@@ -80,16 +86,17 @@ struct _CompEditorPrivate {
 
 static void comp_editor_class_init (CompEditorClass *class);
 static void comp_editor_init (CompEditor *editor);
+static gint comp_editor_key_press_event (GtkWidget *d, GdkEventKey *e);
 static void comp_editor_destroy (GtkObject *object);
 
 static void real_set_cal_client (CompEditor *editor, CalClient *client);
 static void real_edit_comp (CompEditor *editor, CalComponent *comp);
-static void real_send_comp (CompEditor *editor, CalComponentItipMethod method);
+static gboolean real_send_comp (CompEditor *editor, CalComponentItipMethod method);
+static gboolean prompt_to_save_changes (CompEditor *editor, gboolean send);
 static void delete_comp (CompEditor *editor);
 static void close_dialog (CompEditor *editor);
 
 static void page_changed_cb (GtkObject *obj, gpointer data);
-static void page_needs_send_cb (GtkObject *obj, gpointer data);
 static void page_summary_changed_cb (GtkObject *obj, const char *summary, gpointer data);
 static void page_dates_changed_cb (GtkObject *obj, CompEditorPageDates *dates, gpointer data);
 
@@ -138,8 +145,6 @@ static BonoboUIVerb verbs [] = {
 	BONOBO_UI_VERB_END
 };
 
-#define CLASS(page) (COMP_EDITOR_CLASS (GTK_OBJECT (page)->klass))
-
 static GtkObjectClass *parent_class;
 
 
@@ -161,7 +166,7 @@ comp_editor_get_type (void)
 			(GtkClassInitFunc) NULL
 		};
 
-		comp_editor_type = gtk_type_unique (GTK_TYPE_OBJECT,
+		comp_editor_type = gtk_type_unique (BONOBO_TYPE_WINDOW,
 						    &comp_editor_info);
 	}
 
@@ -173,15 +178,18 @@ static void
 comp_editor_class_init (CompEditorClass *klass)
 {
 	GtkObjectClass *object_class;
+	GtkWidgetClass *widget_class;
 
-	object_class = (GtkObjectClass *) klass;
+	object_class = GTK_OBJECT_CLASS (klass);
+	widget_class = GTK_WIDGET_CLASS (klass);
 
-	parent_class = gtk_type_class (GTK_TYPE_OBJECT);
+	parent_class = gtk_type_class (BONOBO_TYPE_WINDOW);
 
 	klass->set_cal_client = real_set_cal_client;
 	klass->edit_comp = real_edit_comp;
 	klass->send_comp = real_send_comp;
 
+	widget_class->key_press_event = comp_editor_key_press_event;
 	object_class->destroy = comp_editor_destroy;
 }
 
@@ -196,15 +204,16 @@ setup_widgets (CompEditor *editor)
 	priv = editor->priv;
 
 	/* Window and basic vbox */
-	priv->window = bonobo_window_new ("event-editor", "iCalendar Editor");
-	gtk_signal_connect (GTK_OBJECT (priv->window), "delete_event",
+	bonobo_window_construct (BONOBO_WINDOW (editor),
+				 "event-editor", "iCalendar Editor");
+	gtk_signal_connect (GTK_OBJECT (editor), "delete_event",
 			    GTK_SIGNAL_FUNC (delete_event_cb), editor);
 
 	priv->uic = bonobo_ui_component_new_default ();
 	container = bonobo_ui_container_new ();
-	bonobo_ui_container_set_win (container, BONOBO_WINDOW (priv->window));
+	bonobo_ui_container_set_win (container, BONOBO_WINDOW (editor));
 	bonobo_ui_component_set_container (priv->uic, BONOBO_OBJREF (container));
-	bonobo_ui_engine_config_set_path (bonobo_window_get_ui_engine (BONOBO_WINDOW (priv->window)),
+	bonobo_ui_engine_config_set_path (bonobo_window_get_ui_engine (BONOBO_WINDOW (editor)),
 					  "/evolution/UIConf/kvps");
 
 	bonobo_ui_component_add_verb_list_with_data (priv->uic, verbs, editor);
@@ -216,7 +225,7 @@ setup_widgets (CompEditor *editor)
 	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
 	gtk_widget_show (vbox);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
-	bonobo_window_set_contents (BONOBO_WINDOW (priv->window), vbox);
+	bonobo_window_set_contents (BONOBO_WINDOW (editor), vbox);
 
 	/* Notebook */
 	priv->notebook = GTK_NOTEBOOK (gtk_notebook_new ());
@@ -239,9 +248,25 @@ comp_editor_init (CompEditor *editor)
 	priv->pages = NULL;
 	priv->changed = FALSE;
 	priv->needs_send = FALSE;
-	priv->existing_org = FALSE;
-	priv->user_org = FALSE;
-	priv->warned = FALSE;
+ 	priv->existing_org = FALSE;
+ 	priv->user_org = FALSE;
+ 	priv->warned = FALSE;
+}
+
+
+static gint
+comp_editor_key_press_event (GtkWidget *d, GdkEventKey *e)
+{
+	if (e->keyval == GDK_Escape) {
+		if (prompt_to_save_changes (COMP_EDITOR (d), TRUE))
+			close_dialog (COMP_EDITOR (d));
+		return TRUE;
+	}
+
+	if (GTK_WIDGET_CLASS (parent_class)->key_press_event)
+		return (* GTK_WIDGET_CLASS (parent_class)->key_press_event) (d, e);
+
+	return FALSE;
 }
 
 /* Destroy handler for the calendar component editor */
@@ -257,11 +282,6 @@ comp_editor_destroy (GtkObject *object)
 
 	gtk_signal_disconnect_by_data (GTK_OBJECT (priv->client), editor);
 
-	if (priv->window) {
-		gtk_widget_destroy (priv->window);
-		priv->window = NULL;
-	}
-
 	/* We want to destroy the pages after the widgets get destroyed,
 	   since they have lots of signal handlers connected to the widgets
 	   with the pages as the data. */
@@ -280,13 +300,13 @@ comp_editor_destroy (GtkObject *object)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
-
 static gboolean
 save_comp (CompEditor *editor)
 {
 	CompEditorPrivate *priv;
 	CalComponent *clone;
 	GList *l;
+	CalClientResult result;
 
 	priv = editor->priv;
 
@@ -312,11 +332,29 @@ save_comp (CompEditor *editor)
 
 	priv->updating = TRUE;
 
-	if (!cal_client_update_object (priv->client, priv->comp)) {
+	result = cal_client_update_object (priv->client, priv->comp);
+	if (result != CAL_CLIENT_RESULT_SUCCESS) {
 		GtkWidget *dlg;
+		char *msg;
 
-		dlg = gnome_error_dialog (_("Could not update object!"));
+		switch (result) {
+		case CAL_CLIENT_RESULT_INVALID_OBJECT :
+			msg = g_strdup (_("Could not update invalid object"));
+			break;
+		case CAL_CLIENT_RESULT_NOT_FOUND :
+			msg = g_strdup (_("Object not found, not updated"));
+			break;
+		case CAL_CLIENT_RESULT_PERMISSION_DENIED :
+			msg = g_strdup (_("You don't have permissions to update this object"));
+			break;
+		default :
+			msg = g_strdup (_("Could not update object"));
+			break;
+		}
+
+		dlg = gnome_error_dialog (msg);
 		gnome_dialog_run_and_close (GNOME_DIALOG (dlg));
+		g_free (msg);
 
 		return FALSE;
 	} else {
@@ -341,13 +379,13 @@ save_comp_with_send (CompEditor *editor)
 	if (!save_comp (editor))
 		return FALSE;
 
-	if (send && send_component_dialog (priv->comp)) {
-		if (itip_organizer_is_user (priv->comp))
-			comp_editor_send_comp (editor, CAL_COMPONENT_METHOD_REQUEST);
-		else
-			comp_editor_send_comp (editor, CAL_COMPONENT_METHOD_REPLY);
-	}
-	
+ 	if (send && send_component_dialog (priv->comp, !priv->existing_org)) {
+ 		if (itip_organizer_is_user (priv->comp))
+ 			return comp_editor_send_comp (editor, CAL_COMPONENT_METHOD_REQUEST);
+ 		else
+ 			return comp_editor_send_comp (editor, CAL_COMPONENT_METHOD_REPLY);
+ 	}
+
 	return TRUE;
 }
 
@@ -376,7 +414,7 @@ prompt_to_save_changes (CompEditor *editor, gboolean send)
 	if (!priv->changed)
 		return TRUE;
 
-	switch (save_component_dialog (GTK_WINDOW (priv->window))) {
+	switch (save_component_dialog (GTK_WINDOW (editor))) {
 	case 0: /* Save */
 		if (send && save_comp_with_send (editor))
 			return TRUE;
@@ -392,6 +430,19 @@ prompt_to_save_changes (CompEditor *editor, gboolean send)
 	}
 }
 
+/* This sets the focus to the toplevel, so any field being edited is committed.
+   FIXME: In future we may also want to check some of the fields are valid,
+   e.g. the EDateEdit fields. */
+static void
+commit_all_fields (CompEditor *editor)
+{
+	CompEditorPrivate *priv;
+
+	priv = editor->priv;
+
+	gtk_window_set_focus (GTK_WINDOW (editor), NULL);
+}
+
 /* Closes the dialog box and emits the appropriate signals */
 static void
 close_dialog (CompEditor *editor)
@@ -400,8 +451,6 @@ close_dialog (CompEditor *editor)
 
 	priv = editor->priv;
 
-	g_assert (priv->window != NULL);
-
 	gtk_object_destroy (GTK_OBJECT (editor));
 }
 
@@ -610,8 +659,6 @@ comp_editor_append_page (CompEditor *editor,
 	gtk_notebook_append_page (priv->notebook, page_widget, label_widget);
 
 	/* Listen for things happening on the page */
-	gtk_signal_connect (GTK_OBJECT (page), "needs_send",
-			    GTK_SIGNAL_FUNC (page_needs_send_cb), editor);
 	gtk_signal_connect (GTK_OBJECT (page), "changed",
 			    GTK_SIGNAL_FUNC (page_changed_cb), editor);
 	gtk_signal_connect (GTK_OBJECT (page), "summary_changed",
@@ -807,7 +854,7 @@ set_title_from_comp (CompEditor *editor)
 
 	priv = editor->priv;
 	title = make_title_from_comp (priv->comp);
-	gtk_window_set_title (GTK_WINDOW (priv->window), title);
+	gtk_window_set_title (GTK_WINDOW (editor), title);
 	g_free (title);
 }
 
@@ -819,7 +866,7 @@ set_icon_from_comp (CompEditor *editor)
 
 	priv = editor->priv;
 	file = make_icon_from_comp (priv->comp);
-	gnome_window_icon_set_from_file (GTK_WINDOW (priv->window), file);
+	gnome_window_icon_set_from_file (GTK_WINDOW (editor), file);
 }
 
 static void
@@ -892,27 +939,42 @@ real_edit_comp (CompEditor *editor, CalComponent *comp)
 	if (comp)
 		priv->comp = cal_component_clone (comp);
 
-	priv->existing_org = cal_component_has_organizer (comp);
-	priv->user_org = itip_organizer_is_user (comp);
-	priv->warned = FALSE;
-	
+ 	priv->existing_org = cal_component_has_organizer (comp);
+ 	priv->user_org = itip_organizer_is_user (comp);
+ 	priv->warned = FALSE;
+ 		
 	set_title_from_comp (editor);
 	set_icon_from_comp (editor);
 	fill_widgets (editor);
 }
 
 
-static void
+static gboolean
 real_send_comp (CompEditor *editor, CalComponentItipMethod method)
 {
 	CompEditorPrivate *priv;
-
-	g_return_if_fail (editor != NULL);
-	g_return_if_fail (IS_COMP_EDITOR (editor));
+	CalComponent *tmp_comp;
+	
+	g_return_val_if_fail (editor != NULL, FALSE);
+	g_return_val_if_fail (IS_COMP_EDITOR (editor), FALSE);
 
 	priv = editor->priv;
 
-	itip_send_comp (method, priv->comp, priv->client, NULL);
+	if (itip_send_comp (method, priv->comp, priv->client, NULL)) {
+		tmp_comp = priv->comp;
+		gtk_object_ref (GTK_OBJECT (tmp_comp));
+		comp_editor_edit_comp (editor, tmp_comp);
+		gtk_object_unref (GTK_OBJECT (tmp_comp));
+		
+		comp_editor_set_changed (editor, TRUE);
+		save_comp (editor);
+
+		return TRUE;
+	}
+
+	comp_editor_set_changed (editor, TRUE);
+
+	return FALSE;
 }
 
 
@@ -939,6 +1001,19 @@ comp_editor_edit_comp (CompEditor *editor, CalComponent *comp)
 		klass->edit_comp (editor, comp);
 }
 
+CalComponent *
+comp_editor_get_comp (CompEditor *editor)
+{
+	CompEditorPrivate *priv;
+
+	g_return_val_if_fail (editor != NULL, NULL);
+	g_return_val_if_fail (IS_COMP_EDITOR (editor), NULL);
+
+	priv = editor->priv;
+
+	return priv->comp;
+}
+
 CalComponent *
 comp_editor_get_current_comp (CompEditor *editor)
 {
@@ -991,18 +1066,37 @@ comp_editor_delete_comp (CompEditor *editor)
  *
  *
  **/
-void
+gboolean
 comp_editor_send_comp (CompEditor *editor, CalComponentItipMethod method)
 {
 	CompEditorClass *klass;
 
-	g_return_if_fail (editor != NULL);
-	g_return_if_fail (IS_COMP_EDITOR (editor));
+	g_return_val_if_fail (editor != NULL, FALSE);
+	g_return_val_if_fail (IS_COMP_EDITOR (editor), FALSE);
 
 	klass = COMP_EDITOR_CLASS (GTK_OBJECT (editor)->klass);
 
 	if (klass->send_comp)
-		klass->send_comp (editor, method);
+		return klass->send_comp (editor, method);
+
+	return FALSE;
+}
+
+gboolean
+comp_editor_close (CompEditor *editor)
+{
+	gboolean close;
+	
+	g_return_val_if_fail (editor != NULL, FALSE);
+	g_return_val_if_fail (IS_COMP_EDITOR (editor), FALSE);
+
+	commit_all_fields (editor);
+	
+	close = prompt_to_save_changes (editor, TRUE);
+	if (close)
+		close_dialog (editor);
+
+	return close;
 }
 
 /**
@@ -1014,7 +1108,10 @@ comp_editor_send_comp (CompEditor *editor, CalComponentItipMethod method)
  *
  **/
 void
-comp_editor_merge_ui (CompEditor *editor, const char *filename, BonoboUIVerb *verbs)
+comp_editor_merge_ui (CompEditor *editor,
+		      const char *filename,
+		      BonoboUIVerb *verbs,
+		      EPixmap *component_pixmaps)
 {
 	CompEditorPrivate *priv;
 
@@ -1025,6 +1122,9 @@ comp_editor_merge_ui (CompEditor *editor, const char *filename, BonoboUIVerb *ve
 
 	bonobo_ui_util_set_ui (priv->uic, EVOLUTION_DATADIR, filename, "evolution-calendar");
 	bonobo_ui_component_add_verb_list_with_data (priv->uic, verbs, editor);
+
+	if (component_pixmaps != NULL)
+		e_pixmaps_update (priv->uic, component_pixmaps);
 }
 
 /**
@@ -1078,21 +1178,8 @@ comp_editor_focus (CompEditor *editor)
 
 	priv = editor->priv;
 
-	gtk_widget_show (priv->window);
-	raise_and_focus (priv->window);
-}
-
-/* This sets the focus to the toplevel, so any field being edited is committed.
-   FIXME: In future we may also want to check some of the fields are valid,
-   e.g. the EDateEdit fields. */
-static void
-commit_all_fields (CompEditor *editor)
-{
-	CompEditorPrivate *priv;
-
-	priv = editor->priv;
-
-	gtk_window_set_focus (GTK_WINDOW (priv->window), NULL);
+	gtk_widget_show (GTK_WIDGET (editor));
+	raise_and_focus (GTK_WIDGET (editor));
 }
 
 /* Menu Commands */
@@ -1117,90 +1204,38 @@ save_close_cmd (GtkWidget *widget, gpointer data)
 		close_dialog (editor);
 }
 
-static void
-save_as_ok (GtkWidget *widget, gpointer data)
-{
-	CompEditor *editor = COMP_EDITOR (data);
-	CompEditorPrivate *priv;
-	struct stat s;
-	char *path;
-	int ret = 0;
-
-	priv = editor->priv;
-
-	path = gtk_file_selection_get_filename (GTK_FILE_SELECTION (priv->filesel));
-
-	if (stat (path, &s) == 0) {
-		GtkWidget *dlg;
-		GtkWidget *text;
-
-		dlg = gnome_dialog_new (_("Overwrite file?"),
-					GNOME_STOCK_BUTTON_YES,
-					GNOME_STOCK_BUTTON_NO,
-					NULL);
-		text = gtk_label_new (_("A file by that name already exists.\nOverwrite it?"));
-		gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dlg)->vbox), text, TRUE, TRUE, 4);
-		gtk_window_set_policy (GTK_WINDOW (dlg), FALSE, TRUE, FALSE);
-		gtk_widget_show (text);
-
-		ret = gnome_dialog_run_and_close (GNOME_DIALOG (dlg));
-	}
-
-	if (ret == 0) {
-		FILE *file;
-		gchar *ical_string;
-
-		ical_string = cal_client_get_component_as_string (priv->client, priv->comp);
-		if (ical_string == NULL) {
-			g_warning ("Couldn't convert item to a string");
-			gtk_main_quit ();
-			return;
-		}
-
-		file = fopen (path, "w");
-		if (file == NULL) {
-			g_warning ("Couldn't save item");
-			gtk_main_quit ();
-			return;
-		}
-
-		fprintf (file, ical_string);
-		g_free (ical_string);
-		fclose (file);
-
-		gtk_main_quit ();
-	}
-}
-
 static void
 save_as_cmd (GtkWidget *widget, gpointer data)
 {
 	CompEditor *editor = COMP_EDITOR (data);
 	CompEditorPrivate *priv;
-	GtkFileSelection *fs;
-	char *path;
-
+	char *filename;
+	char *ical_string;
+	FILE *file;
+	
 	priv = editor->priv;
 
 	commit_all_fields (editor);
 
-	fs = GTK_FILE_SELECTION (gtk_file_selection_new (_("Save As...")));
-	path = g_strdup_printf ("%s/", g_get_home_dir ());
-	gtk_file_selection_set_filename (fs, path);
-	g_free (path);
-
-	gtk_signal_connect (GTK_OBJECT (fs->ok_button), "clicked",
-			    GTK_SIGNAL_FUNC (save_as_ok), editor);
-	gtk_signal_connect (GTK_OBJECT (fs->cancel_button), "clicked",
-			    GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
-
-	priv->filesel = GTK_WIDGET (fs);
-	gtk_widget_show (priv->filesel);
-	gtk_grab_add (priv->filesel);
-	gtk_main ();
-
-	gtk_widget_destroy (priv->filesel);
-	priv->filesel = NULL;
+	filename = e_file_dialog_save (_("Save as..."));
+	if (filename == NULL)
+		return;
+	
+	ical_string = cal_client_get_component_as_string (priv->client, priv->comp);
+	if (ical_string == NULL) {
+		g_warning ("Couldn't convert item to a string");
+		return;
+	}
+	
+	file = fopen (filename, "w");
+	if (file == NULL) {
+		g_warning ("Couldn't save item");
+		return;
+	}
+	
+	fprintf (file, ical_string);
+	g_free (ical_string);
+	fclose (file);
 }
 
 static void
@@ -1214,8 +1249,13 @@ delete_cmd (GtkWidget *widget, gpointer data)
 
 	vtype = cal_component_get_vtype (priv->comp);
 
-	if (delete_component_dialog (priv->comp, FALSE, 1, vtype, priv->window))
+	if (delete_component_dialog (priv->comp, FALSE, 1, vtype, GTK_WIDGET (editor))) {
+		if (itip_organizer_is_user (priv->comp) 
+		    && cancel_component_dialog (priv->comp, TRUE))
+			itip_send_comp (CAL_COMPONENT_METHOD_CANCEL, priv->comp, priv->client, NULL);
+
 		delete_comp (editor);
+	}
 }
 
 static void
@@ -1284,17 +1324,6 @@ page_changed_cb (GtkObject *obj, gpointer data)
 	
 }
 
-static void
-page_needs_send_cb (GtkObject *obj, gpointer data)
-{
-	CompEditor *editor = COMP_EDITOR (data);
-	CompEditorPrivate *priv;
-
-	priv = editor->priv;
-
-	priv->needs_send = TRUE;
-}
-
 /* Page signal callbacks */
 static void
 page_summary_changed_cb (GtkObject *obj, const char *summary, gpointer data)
