@@ -7,10 +7,9 @@
  *          Federico Mena-Quintero <federico@ximian.com>
  *          Seth Alves <alves@hungry.com>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -52,10 +51,10 @@ struct _EventEditorClass {
 	CompEditorClass parent_class;
 };
 
-GtkType      event_editor_get_type       (void);
-EventEditor *event_editor_construct      (EventEditor *ee);
-EventEditor *event_editor_new            (void);
-void         event_editor_update_widgets (EventEditor *ee);
+GtkType      event_editor_get_type     (void);
+EventEditor *event_editor_construct    (EventEditor *ee);
+EventEditor *event_editor_new          (void);
+void         event_editor_show_meeting (EventEditor *ee);
 
 
 
