@@ -42,7 +42,7 @@ struct _TaskEditorPrivate {
 
 	EMeetingModel *model;
 	
-	gboolean meeting_shown;
+	gboolean assignment_shown;
 	gboolean updating;	
 };
 
@@ -51,7 +51,7 @@ struct _TaskEditorPrivate {
 static void task_editor_class_init (TaskEditorClass *class);
 static void task_editor_init (TaskEditor *te);
 static void task_editor_edit_comp (CompEditor *editor, CalComponent *comp);
-static void task_editor_send_comp (CompEditor *editor, CalComponentItipMethod method);
+static gboolean task_editor_send_comp (CompEditor *editor, CalComponentItipMethod method);
 static void task_editor_destroy (GtkObject *object);
 
 static void assign_task_cmd (GtkWidget *widget, gpointer data);
@@ -133,23 +133,23 @@ set_menu_sens (TaskEditor *te)
 	
 	priv = te->priv;
 
-	existing = comp_editor_get_existing_org (COMP_EDITOR (te));
-	user = comp_editor_get_user_org (COMP_EDITOR (te));
-
-	sens = priv->meeting_shown;
-	comp_editor_set_ui_prop (COMP_EDITOR (te), 
-				 "/commands/ActionAssignTask", 
-				 "sensitive", sens ? "0" : "1");
-
-	sens = priv->meeting_shown && existing && !user;
-	comp_editor_set_ui_prop (COMP_EDITOR (te), 
-				 "/commands/ActionRefreshTask", 
-				 "sensitive", sens ? "1" : "0");
-
-	sens = priv->meeting_shown && existing && user;
-	comp_editor_set_ui_prop (COMP_EDITOR (te), 
-				 "/commands/ActionCancelTask", 
-				 "sensitive", sens ? "1" : "0");
+ 	existing = comp_editor_get_existing_org (COMP_EDITOR (te));
+ 	user = comp_editor_get_user_org (COMP_EDITOR (te));
+ 
+  	sens = priv->assignment_shown;
+  	comp_editor_set_ui_prop (COMP_EDITOR (te), 
+  				 "/commands/ActionAssignTask", 
+  				 "sensitive", sens ? "0" : "1");
+  
+ 	sens = priv->assignment_shown && existing && !user;
+  	comp_editor_set_ui_prop (COMP_EDITOR (te), 
+  				 "/commands/ActionRefreshTask", 
+  				 "sensitive", sens ? "1" : "0");
+ 
+ 	sens = priv->assignment_shown && existing && user;
+  	comp_editor_set_ui_prop (COMP_EDITOR (te), 
+  				 "/commands/ActionCancelTask", 
+  				 "sensitive", sens ? "1" : "0");
 }
 
 static void
@@ -193,9 +193,9 @@ task_editor_init (TaskEditor *te)
 				 COMP_EDITOR_PAGE (priv->meet_page),
 				 _("Assignment"));
 
-	comp_editor_merge_ui (COMP_EDITOR (te), "evolution-task-editor.xml", verbs);
+	comp_editor_merge_ui (COMP_EDITOR (te), "evolution-task-editor.xml", verbs, NULL);
 
-	priv->meeting_shown = TRUE;
+	priv->assignment_shown = TRUE;
 	priv->updating = FALSE;	
 
 	init_widgets (te);
@@ -228,11 +228,11 @@ task_editor_edit_comp (CompEditor *editor, CalComponent *comp)
 
 	if (attendees == NULL) {
 		comp_editor_remove_page (editor, COMP_EDITOR_PAGE (priv->meet_page));
-		priv->meeting_shown = FALSE;
+		priv->assignment_shown = FALSE;
 	} else {
 		GSList *l;
 
-		if (!priv->meeting_shown)
+		if (!priv->assignment_shown)
 			comp_editor_append_page (COMP_EDITOR (te),
 						 COMP_EDITOR_PAGE (priv->meet_page),
 						 _("Assignment"));
@@ -245,37 +245,37 @@ task_editor_edit_comp (CompEditor *editor, CalComponent *comp)
 			gtk_object_unref (GTK_OBJECT (ia));
 		}
 
-		if (organizer.value != NULL) {
-			GList *addresses, *l;
-			const char *strip;
-			int row;
+ 		if (organizer.value != NULL) {
+ 			GList *addresses, *l;
+ 			const char *strip;
+ 			int row;
+ 
+ 			strip = itip_strip_mailto (organizer.value);
+			
+ 			addresses = itip_addresses_get ();
+ 			for (l = addresses; l != NULL; l = l->next) {
+ 				ItipAddress *a = l->data;
+ 				
+ 				if (e_meeting_model_find_attendee (priv->model, a->address, &row))
+ 					e_meeting_model_restricted_add (priv->model, row);
+ 			}
+ 			itip_addresses_free (addresses);
+ 		}
 
-			strip = itip_strip_mailto (organizer.value);
+ 		if (comp_editor_get_user_org (editor))
+ 			e_meeting_model_restricted_clear (priv->model);
 
-			addresses = itip_addresses_get ();
-			for (l = addresses; l != NULL; l = l->next) {
-				ItipAddress *a = l->data;
-				
-				if (e_meeting_model_find_attendee (priv->model, a->address, &row))
-					e_meeting_model_restricted_add (priv->model, row);
-			}
-			itip_addresses_free (addresses);
-		}
-		
-		if (comp_editor_get_user_org (editor))
-			e_meeting_model_restricted_clear (priv->model);
-		
-		priv->meeting_shown = TRUE;		
+		priv->assignment_shown = TRUE;		
 	}
 	cal_component_free_attendee_list (attendees);
 
 	set_menu_sens (te);
-	comp_editor_set_needs_send (COMP_EDITOR (te), priv->meeting_shown && itip_organizer_is_user (comp));
+	comp_editor_set_needs_send (COMP_EDITOR (te), priv->assignment_shown && itip_organizer_is_user (comp));
 
 	priv->updating = FALSE;
 }
 
-static void
+static gboolean
 task_editor_send_comp (CompEditor *editor, CalComponentItipMethod method)
 {
 	TaskEditor *te = TASK_EDITOR (editor);
@@ -292,15 +292,21 @@ task_editor_send_comp (CompEditor *editor, CalComponentItipMethod method)
 	comp = meeting_page_get_cancel_comp (priv->meet_page);
 	if (comp != NULL) {
 		CalClient *client;
+		gboolean result;
 		
 		client = e_meeting_model_get_cal_client (priv->model);
-		itip_send_comp (CAL_COMPONENT_METHOD_CANCEL, comp, client, NULL);
+		result = itip_send_comp (CAL_COMPONENT_METHOD_CANCEL, comp, client, NULL);
 		gtk_object_unref (GTK_OBJECT (comp));
+
+		if (!result)
+			return FALSE;
 	}
 
  parent:
 	if (parent_class->send_comp)
-		parent_class->send_comp (editor, method);
+		return parent_class->send_comp (editor, method);
+
+	return FALSE;
 }
 
 /* Destroy handler for the event editor */
@@ -341,27 +347,44 @@ task_editor_new (void)
 }
 
 static void
-assign_task_cmd (GtkWidget *widget, gpointer data)
+show_assignment (TaskEditor *te)
 {
-	TaskEditor *te = TASK_EDITOR (data);
 	TaskEditorPrivate *priv;
 
 	priv = te->priv;
 
-	if (!priv->meeting_shown) {
+	if (!priv->assignment_shown) {
 		comp_editor_append_page (COMP_EDITOR (te),
 					 COMP_EDITOR_PAGE (priv->meet_page),
 					 _("Assignment"));
-		priv->meeting_shown = TRUE;
+		priv->assignment_shown = TRUE;
 
 		set_menu_sens (te);
-		comp_editor_set_needs_send (COMP_EDITOR (te), priv->meeting_shown);
+		comp_editor_set_needs_send (COMP_EDITOR (te), priv->assignment_shown);
+		comp_editor_set_changed (COMP_EDITOR (te), TRUE);
 	}
 
 	comp_editor_show_page (COMP_EDITOR (te),
 			       COMP_EDITOR_PAGE (priv->meet_page));
 }
 
+void
+task_editor_show_assignment (TaskEditor *te)
+{
+	g_return_if_fail (te != NULL);
+	g_return_if_fail (IS_TASK_EDITOR (te));
+
+	show_assignment (te);
+}
+
+static void
+assign_task_cmd (GtkWidget *widget, gpointer data)
+{
+	TaskEditor *te = TASK_EDITOR (data);
+
+	show_assignment (te);
+}
+
 static void
 refresh_task_cmd (GtkWidget *widget, gpointer data)
 {
@@ -377,7 +400,7 @@ cancel_task_cmd (GtkWidget *widget, gpointer data)
 	CalComponent *comp;
 	
 	comp = comp_editor_get_current_comp (COMP_EDITOR (te));
-	if (cancel_component_dialog (comp)) {
+	if (cancel_component_dialog (comp, FALSE)) {
 		comp_editor_send_comp (COMP_EDITOR (te), CAL_COMPONENT_METHOD_CANCEL);
 		comp_editor_delete_comp (COMP_EDITOR (te));
 	}
