@@ -7,10 +7,9 @@
  *          Federico Mena-Quintero <federico@ximian.com>
  *          Seth Alves <alves@hungry.com>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -54,9 +53,9 @@ struct _TaskEditorClass {
 };
 
 GtkType     task_editor_get_type       (void);
-TaskEditor *task_editor_construct      (TaskEditor *ee);
+TaskEditor *task_editor_construct      (TaskEditor *te);
 TaskEditor *task_editor_new            (void);
-
+void        task_editor_show_assignment(TaskEditor *te);
 
 
 #endif /* __TASK_EDITOR_H__ */
