@@ -182,6 +182,66 @@ itip_addresses_free (GList *addresses)
 	g_list_free (addresses);
 }
 
+gboolean
+itip_organizer_is_user (CalComponent *comp)
+{
+	CalComponentOrganizer organizer;
+	GList *addresses, *l;
+	const char *strip;
+	gboolean user_org = FALSE;
+	
+	if (!cal_component_has_organizer (comp))
+		return FALSE;
+
+	cal_component_get_organizer (comp, &organizer);
+	if (organizer.value != NULL) {
+		strip = itip_strip_mailto (organizer.value);
+
+		addresses = itip_addresses_get ();
+		for (l = addresses; l != NULL; l = l->next) {
+			ItipAddress *a = l->data;
+			
+			if (!g_strcasecmp (a->address, strip)) {
+				user_org = TRUE;
+				break;
+			}
+		}
+		itip_addresses_free (addresses);
+	}
+
+	return user_org;
+}
+
+gboolean
+itip_sentby_is_user (CalComponent *comp)
+{
+	CalComponentOrganizer organizer;
+	GList *addresses, *l;
+	const char *strip;
+	gboolean user_sentby = FALSE;
+	
+	if (!cal_component_has_organizer (comp))
+		return FALSE;
+
+	cal_component_get_organizer (comp, &organizer);
+	if (organizer.sentby != NULL) {
+		strip = itip_strip_mailto (organizer.sentby);
+
+		addresses = itip_addresses_get ();
+		for (l = addresses; l != NULL; l = l->next) {
+			ItipAddress *a = l->data;
+			
+			if (!g_strcasecmp (a->address, strip)) {
+				user_sentby = TRUE;
+				break;
+			}
+		}
+		itip_addresses_free (addresses);
+	}
+
+	return user_sentby;
+}
+				 
 const gchar *
 itip_strip_mailto (const gchar *address) 
 {
@@ -201,14 +261,90 @@ typedef struct {
 	icalcomponent *zones;
 } ItipUtilTZData;
 
+
+static void
+foreach_tzid_callback (icalparameter *param, gpointer data)
+{
+	ItipUtilTZData *tz_data = data;	
+	const char *tzid;
+	icaltimezone *zone = NULL;
+	icalcomponent *vtimezone_comp;
+
+	/* Get the TZID string from the parameter. */
+	tzid = icalparameter_get_tzid (param);
+	if (!tzid || g_hash_table_lookup (tz_data->tzids, tzid))
+		return;
+
+	/* Look for the timezone */
+	if (tz_data->zones != NULL)
+		zone = icalcomponent_get_timezone (tz_data->zones, tzid);
+	if (zone == NULL)
+		zone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
+	if (zone == NULL && tz_data->client != NULL)
+		cal_client_get_timezone (tz_data->client, tzid, &zone);
+	if (zone == NULL)
+		return;
+
+	/* Convert it to a string and add it to the hash. */
+	vtimezone_comp = icaltimezone_get_component (zone);
+	if (!vtimezone_comp)
+		return;
+
+	icalcomponent_add_component (tz_data->icomp, icalcomponent_new_clone (vtimezone_comp));
+	g_hash_table_insert (tz_data->tzids, (char *)tzid, (char *)tzid);	
+}
+
+static icalcomponent *
+comp_toplevel_with_zones (CalComponentItipMethod method, CalComponent *comp, CalClient *client, icalcomponent *zones)
+{
+	icalcomponent *top_level, *icomp;
+	icalproperty *prop;
+	icalvalue *value;
+	ItipUtilTZData tz_data;
+
+	top_level = cal_util_new_top_level ();
+
+	prop = icalproperty_new (ICAL_METHOD_PROPERTY);
+	value = icalvalue_new_method (itip_methods_enum[method]);
+	icalproperty_set_value (prop, value);
+	icalcomponent_add_property (top_level, prop);
+
+	icomp = cal_component_get_icalcomponent (comp);
+	icomp = icalcomponent_new_clone (icomp);
+	
+	tz_data.tzids = g_hash_table_new (g_str_hash, g_str_equal);
+	tz_data.icomp = top_level;
+	tz_data.client = client;
+	tz_data.zones = zones;
+	icalcomponent_foreach_tzid (icomp, foreach_tzid_callback, &tz_data);
+	g_hash_table_destroy (tz_data.tzids);
+
+	icalcomponent_add_component (top_level, icomp);
+
+	return top_level;
+}
+
+static gboolean
+users_has_attendee (GList *users, const char *address)
+{
+	GList *l;
+
+	for (l = users; l != NULL; l = l->next) {
+		if (!g_strcasecmp (address, l->data))
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
 static GNOME_Evolution_Composer_RecipientList *
-comp_to_list (CalComponentItipMethod method, CalComponent *comp)
+comp_to_list (CalComponentItipMethod method, CalComponent *comp, GList *users)
 {
 	GNOME_Evolution_Composer_RecipientList *to_list;
 	GNOME_Evolution_Composer_Recipient *recipient;
 	CalComponentOrganizer organizer;
 	GSList *attendees, *l;
-	gint cntr, len;
+	gint len;
 
 	switch (method) {
 	case CAL_COMPONENT_METHOD_REQUEST:
@@ -224,18 +360,23 @@ comp_to_list (CalComponentItipMethod method, CalComponent *comp)
 		
 		to_list = GNOME_Evolution_Composer_RecipientList__alloc ();
 		to_list->_maximum = len;
-		to_list->_length = len;
+		to_list->_length = 0;
 		to_list->_buffer = CORBA_sequence_GNOME_Evolution_Composer_Recipient_allocbuf (len);
 		
-		for (cntr = 0, l = attendees; cntr < len; cntr++, l = l->next) {
+		for (l = attendees; l != NULL; l = l->next) {
 			CalComponentAttendee *att = l->data;
-			
-			recipient = &(to_list->_buffer[cntr]);
+
+			if (users_has_attendee (users, att->value))
+				continue;
+
+			recipient = &(to_list->_buffer[to_list->_length]);
 			if (att->cn)
 				recipient->name = CORBA_string_dup (att->cn);
 			else
 				recipient->name = CORBA_string_dup ("");
 			recipient->address = CORBA_string_dup (itip_strip_mailto (att->value));
+			
+			to_list->_length++;
 		}
 		cal_component_free_attendee_list (attendees);
 		break;
@@ -283,7 +424,6 @@ comp_subject (CalComponentItipMethod method, CalComponent *comp)
 	CalComponentText caltext;
 	const char *description, *prefix = NULL;
 	GSList *alist;
-	int *sequence;
 	CORBA_char *subject;
 
 	cal_component_get_summary (comp, &caltext);
@@ -381,99 +521,36 @@ comp_content_type (CalComponent *comp, CalComponentItipMethod method)
 
 }
 
-static void
-foreach_tzid_callback (icalparameter *param, gpointer data)
+static gboolean
+comp_server_send (CalComponentItipMethod method, CalComponent *comp, CalClient *client, 
+		  icalcomponent *zones, GList **users)
 {
-	ItipUtilTZData *tz_data = data;	
-	const char *tzid;
-	icaltimezone *zone = NULL;
-	icalcomponent *vtimezone_comp;
+	CalClientSendResult result;
+	icalcomponent *top_level, *new_top_level = NULL;
+	char error_msg[256];
+	gboolean retval = FALSE;
+	
+	top_level = comp_toplevel_with_zones (method, comp, client, zones);
+	result = cal_client_send_object (client, top_level, &new_top_level, users, error_msg);
 
-	/* Get the TZID string from the parameter. */
-	tzid = icalparameter_get_tzid (param);
-	if (!tzid || g_hash_table_lookup (tz_data->tzids, tzid))
-		return;
-
-	/* Look for the timezone */
-	if (tz_data->zones != NULL)
-		zone = icalcomponent_get_timezone (tz_data->zones, tzid);
-	if (zone == NULL)
-		zone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
-	if (zone == NULL && tz_data->client != NULL)
-		cal_client_get_timezone (tz_data->client, tzid, &zone);
-	if (zone == NULL)
-		return;
-
-	/* Convert it to a string and add it to the hash. */
-	vtimezone_comp = icaltimezone_get_component (zone);
-	if (!vtimezone_comp)
-		return;
-
-	icalcomponent_add_component (tz_data->icomp, icalcomponent_new_clone (vtimezone_comp));
-	g_hash_table_insert (tz_data->tzids, (char *)tzid, (char *)tzid);	
-}
-
-static char *
-comp_string (CalComponentItipMethod method, CalComponent *comp, CalClient *client, icalcomponent *zones)
-{
-	icalcomponent *top_level, *icomp;
-	icalproperty *prop;
-	icalvalue *value;
-	gchar *ical_string;
-	ItipUtilTZData tz_data;
+	if (result == CAL_CLIENT_SEND_SUCCESS) {
+		icalcomponent *ical_comp;
 		
-	top_level = cal_util_new_top_level ();
+		ical_comp = icalcomponent_get_inner (new_top_level);
+		icalcomponent_remove_component (new_top_level, ical_comp);
+		cal_component_set_icalcomponent (comp, ical_comp);
+		icalcomponent_free (new_top_level);
+		
+		retval = TRUE;
+	} else if (result == CAL_CLIENT_SEND_BUSY) {
+		e_notice (NULL, GNOME_MESSAGE_BOX_ERROR, error_msg);
 
-	prop = icalproperty_new (ICAL_METHOD_PROPERTY);
-	value = icalvalue_new_method (itip_methods_enum[method]);
-	icalproperty_set_value (prop, value);
-	icalcomponent_add_property (top_level, prop);
-
-	icomp = cal_component_get_icalcomponent (comp);
-
-	if (method == CAL_COMPONENT_METHOD_REPLY) {
-		struct icaltimetype dtstamp;
-		gboolean add_it = FALSE;
-
-		/* workaround for Outlook expecting a X-MICROSOFT-CDO-REPLYTIME
-		   on every METHOD=REPLY message. If the component has any of
-		   the X-MICROSOFT-* properties, we add the REPLYTIME one */
-		prop = icalcomponent_get_first_property (icomp, ICAL_X_PROPERTY);
-		while (prop) {
-			const char *x_name;
-
-			x_name = icalproperty_get_x_name (prop);
-			if (!strncmp (x_name, "X-MICROSOFT-", strlen ("X-MICROSOFT-"))) {
-				add_it = TRUE;
-				break;
-			}
-			prop = icalcomponent_get_next_property (icomp, ICAL_X_PROPERTY);
-		}
-
-		if (add_it) {
-			dtstamp = icaltime_from_timet_with_zone (
-				time (NULL), 0, icaltimezone_get_utc_timezone ());
-			prop = icalproperty_new_x (icaltime_as_ical_string (dtstamp));
-			icalproperty_set_x_name (prop, "X-MICROSOFT-CDO-REPLYTIME");
-			icalcomponent_add_property (icomp, prop);
-		}
+		retval = FALSE;
 	}
-		
-	/* Add the timezones */
-	tz_data.tzids = g_hash_table_new (g_str_hash, g_str_equal);
-	tz_data.icomp = top_level;
-	tz_data.client = client;
-	tz_data.zones = zones;
-	icalcomponent_foreach_tzid (icomp, foreach_tzid_callback, &tz_data);
-	g_hash_table_destroy (tz_data.tzids);
 
-	icalcomponent_add_component (top_level, icomp);
-	ical_string = icalcomponent_as_ical_string (top_level);
-	icalcomponent_remove_component (top_level, icomp);
-	
 	icalcomponent_free (top_level);
-	
-	return ical_string;	
+
+	return retval;
 }
 
 static gboolean
@@ -541,9 +618,6 @@ static void
 comp_sentby (CalComponent *comp)
 {
 	CalComponentOrganizer organizer;
-	GList *addresses, *l;
-	const char *strip;
-	gboolean is_user = FALSE;
 	
 	cal_component_get_organizer (comp, &organizer);
 	if (!organizer.value) {
@@ -560,19 +634,8 @@ comp_sentby (CalComponent *comp)
 		
 		return;
 	}
-	
-	strip = itip_strip_mailto (organizer.value);
 
-	addresses = itip_addresses_get ();
-	for (l = addresses; l != NULL; l = l->next) {
-		ItipAddress *a = l->data;
-		
-		if (!strcmp (a->address, strip)) {
-			is_user = TRUE;
-			break;
-		}
-	}
-	if (!is_user) {
+	if (!itip_organizer_is_user (comp) && !itip_sentby_is_user (comp)) {
 		ItipAddress *a = itip_addresses_get_default ();
 		
 		organizer.value = g_strdup (organizer.value);
@@ -588,8 +651,6 @@ comp_sentby (CalComponent *comp)
 		g_free ((char *)organizer.language);
 		itip_address_free (a);
 	}
-	
-	itip_addresses_free (addresses);
 }
 static CalComponent *
 comp_minimal (CalComponent *comp, gboolean attendee)
@@ -682,6 +743,9 @@ comp_compliant (CalComponentItipMethod method, CalComponent *comp)
 	/* We delete incoming alarms anyhow, and this helps with outlook */
 	cal_component_remove_all_alarms (clone);
 
+	/* Strip X-LIC-ERROR stuff */
+	cal_component_strip_errors (clone);
+	
 	/* Comply with itip spec */
 	switch (method) {
 	case CAL_COMPONENT_METHOD_PUBLISH:
@@ -718,13 +782,15 @@ comp_compliant (CalComponentItipMethod method, CalComponent *comp)
 	return clone;
 }
 
-void
+gboolean
 itip_send_comp (CalComponentItipMethod method, CalComponent *send_comp,
 		CalClient *client, icalcomponent *zones)
 {
 	BonoboObjectClient *bonobo_server;
 	GNOME_Evolution_Composer composer_server;
 	CalComponent *comp = NULL;
+	icalcomponent *top_level = NULL;
+	GList *users;
 	GNOME_Evolution_Composer_RecipientList *to_list = NULL;
 	GNOME_Evolution_Composer_RecipientList *cc_list = NULL;
 	GNOME_Evolution_Composer_RecipientList *bcc_list = NULL;
@@ -733,21 +799,35 @@ itip_send_comp (CalComponentItipMethod method, CalComponent *send_comp,
 	GNOME_Evolution_Composer_AttachmentData *attach_data = NULL;
 	char *ical_string;
 	CORBA_Environment ev;
+	gboolean retval = FALSE;
 	
 	CORBA_exception_init (&ev);
 
 	/* Obtain an object reference for the Composer. */
 	bonobo_server = bonobo_object_activate (GNOME_EVOLUTION_COMPOSER_OAFIID, 0);
-	g_return_if_fail (bonobo_server != NULL);
+	g_return_val_if_fail (bonobo_server != NULL, FALSE);
 	composer_server = BONOBO_OBJREF (bonobo_server);
-
+	
+	/* Give the server a chance to manipulate the comp */
+	if (method != CAL_COMPONENT_METHOD_PUBLISH) {
+		if (!comp_server_send (method, send_comp, client, zones, &users))
+			goto cleanup;
+	}
+	
+	/* Tidy up the comp */
 	comp = comp_compliant (method, send_comp);
 	if (comp == NULL)
 		goto cleanup;
-	
-	to_list = comp_to_list (method, comp);
-	if (to_list == NULL)
-		goto cleanup;
+
+	/* Recipients */
+	to_list = comp_to_list (method, comp, users);
+	if (method != CAL_COMPONENT_METHOD_PUBLISH) {
+		if (to_list == NULL || to_list->_length == 0) {
+			/* We sent them all via the server */
+			retval = TRUE;
+			goto cleanup;
+		}
+	}
 	
 	cc_list = GNOME_Evolution_Composer_RecipientList__alloc ();
 	cc_list->_maximum = cc_list->_length = 0;
@@ -767,7 +847,8 @@ itip_send_comp (CalComponentItipMethod method, CalComponent *send_comp,
 	/* Content type */
 	content_type = comp_content_type (comp, method);
 
-	ical_string = comp_string (method, comp, client, zones);
+	top_level = comp_toplevel_with_zones (method, comp, client, zones);
+	ical_string = icalcomponent_as_ical_string (top_level);
 	attach_data = GNOME_Evolution_Composer_AttachmentData__alloc ();
 	attach_data->_length = strlen (ical_string) + 1;
 	attach_data->_maximum = attach_data->_length;	
@@ -785,10 +866,14 @@ itip_send_comp (CalComponentItipMethod method, CalComponent *send_comp,
 		GNOME_Evolution_Composer_show (composer_server, &ev);
 		if (BONOBO_EX (&ev))
 			g_warning ("Unable to show the composer while sending iTip message");
+		else
+			retval = TRUE;
 	} else {		
 		GNOME_Evolution_Composer_send (composer_server, &ev);
 		if (BONOBO_EX (&ev))
 			g_warning ("Unable to send iTip message");
+		else
+			retval = TRUE;
 	}
 	
  cleanup:
@@ -796,7 +881,9 @@ itip_send_comp (CalComponentItipMethod method, CalComponent *send_comp,
 
 	if (comp != NULL)
 		gtk_object_unref (GTK_OBJECT (comp));
-		
+	if (top_level != NULL)
+		icalcomponent_free (top_level);
+
 	if (to_list != NULL)
 		CORBA_free (to_list);
 	if (cc_list != NULL)
@@ -816,5 +903,7 @@ itip_send_comp (CalComponentItipMethod method, CalComponent *send_comp,
 		CORBA_free (description);
 	if (attach_data != NULL)
 		CORBA_free (attach_data);
+
+	return retval;
 }
 
