@@ -34,6 +34,7 @@
 #include <liboaf/liboaf.h>
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-generic-factory.h>
+#include <bonobo/bonobo-exception.h>
 
 #include <gal/widgets/e-cursors.h>
 
@@ -41,14 +42,18 @@
 #include "calendar-commands.h"
 #include "calendar-config.h"
 #include "component-factory.h"
+#include "e-comp-editor-registry.h"
 #include "comp-editor-factory.h"
 #include "control-factory.h"
 #include "itip-control-factory.h"
 #include "tasks-control-factory.h"
 
+ECompEditorRegistry *comp_editor_registry = NULL;
+
 /* The component editor factory */
 static CompEditorFactory *comp_editor_factory = NULL;
 
+
 static void
 init_bonobo (int argc, char **argv)
 {
@@ -60,6 +65,9 @@ init_bonobo (int argc, char **argv)
 
 	if (bonobo_init (CORBA_OBJECT_NIL, CORBA_OBJECT_NIL, CORBA_OBJECT_NIL) == FALSE)
 		g_error (_("Could not initialize Bonobo"));
+
+	if (!bonobo_activate ())
+		g_error (_("Could not activate Bonobo"));
 }
 
 /* Factory function for the calendar component factory; just creates and
@@ -103,7 +111,7 @@ launch_alarm_daemon (void)
 	CORBA_exception_init (&ev);
 	an = oaf_activate_from_id ("OAFIID:GNOME_Evolution_Calendar_AlarmNotify", 0, NULL, &ev);
 
-	if (ev._major != CORBA_NO_EXCEPTION) {
+	if (BONOBO_EX (&ev)) {
 		g_message ("add_alarms(): Could not activate the alarm notification service");
 		CORBA_exception_free (&ev);
 		return;
@@ -114,7 +122,7 @@ launch_alarm_daemon (void)
 
 	CORBA_exception_init (&ev);
 	bonobo_object_release_unref (an, &ev);
-	if (ev._major != CORBA_NO_EXCEPTION)
+	if (BONOBO_EX (&ev))
 		g_message ("add_alarms(): Could not unref the alarm notification service");
 
 	CORBA_exception_free (&ev);
@@ -143,6 +151,8 @@ main (int argc, char **argv)
 				G_LOG_LEVEL_WARNING);
 #endif
 
+	comp_editor_registry = E_COMP_EDITOR_REGISTRY (e_comp_editor_registry_new ());
+	
 	calendar_config_init ();
 
 	control_factory_init ();
@@ -159,7 +169,6 @@ main (int argc, char **argv)
 	bonobo_main ();
 
 	alarm_done ();
-	calendar_config_write_on_exit ();
 
 	gnome_vfs_shutdown ();
 
