@@ -26,7 +26,7 @@
 #include "cal.h"
 #include "cal-backend.h"
 #include "query.h"
-#include "wombat.h"
+#include "Evolution-Wombat.h"
 
 #define PARENT_TYPE         BONOBO_X_OBJECT_TYPE
 
@@ -337,6 +337,11 @@ impl_Cal_get_alarms_in_range (PortableServer_Servant servant,
 		return NULL;
 	}
 
+	if (!seq) {
+		bonobo_exception_set (ev, ex_GNOME_Evolution_Calendar_Cal_NotFound);
+		return NULL;
+	}
+
 	return seq;
 }
 
@@ -388,12 +393,24 @@ impl_Cal_update_objects (PortableServer_Servant servant,
 {
 	Cal *cal;
 	CalPrivate *priv;
+	CalBackendResult result;
 
 	cal = CAL (bonobo_object_from_servant (servant));
 	priv = cal->priv;
 
-	if (!cal_backend_update_objects (priv->backend, calobj))
+	result = cal_backend_update_objects (priv->backend, calobj);
+	switch (result) {
+	case CAL_BACKEND_RESULT_INVALID_OBJECT :
 		bonobo_exception_set (ev, ex_GNOME_Evolution_Calendar_Cal_InvalidObject);
+		break;
+	case CAL_BACKEND_RESULT_NOT_FOUND :
+		bonobo_exception_set (ev, ex_GNOME_Evolution_Calendar_Cal_NotFound);
+		break;
+	case CAL_BACKEND_RESULT_PERMISSION_DENIED :
+		bonobo_exception_set (ev, ex_GNOME_Evolution_Calendar_Cal_PermissionDenied);
+		break;
+	default :
+	}
 }
 
 /* Cal::remove_object method */
@@ -404,12 +421,71 @@ impl_Cal_remove_object (PortableServer_Servant servant,
 {
 	Cal *cal;
 	CalPrivate *priv;
+	CalBackendResult result;
 
 	cal = CAL (bonobo_object_from_servant (servant));
 	priv = cal->priv;
 
-	if (!cal_backend_remove_object (priv->backend, uid))
+	result = cal_backend_remove_object (priv->backend, uid);
+	switch (result) {
+	case CAL_BACKEND_RESULT_INVALID_OBJECT :
+		bonobo_exception_set (ev, ex_GNOME_Evolution_Calendar_Cal_InvalidObject);
+		break;
+	case CAL_BACKEND_RESULT_NOT_FOUND :
 		bonobo_exception_set (ev, ex_GNOME_Evolution_Calendar_Cal_NotFound);
+		break;
+	case CAL_BACKEND_RESULT_PERMISSION_DENIED :
+		bonobo_exception_set (ev, ex_GNOME_Evolution_Calendar_Cal_PermissionDenied);
+		break;
+	default :
+	}
+}
+
+/* Cal::send_object method */
+static GNOME_Evolution_Calendar_CalObj
+impl_Cal_send_object (PortableServer_Servant servant,
+		      const GNOME_Evolution_Calendar_CalObj calobj,
+		      GNOME_Evolution_Calendar_UserList **user_list,
+		      CORBA_Environment *ev)
+{
+	Cal *cal;
+	CalPrivate *priv;
+	CORBA_char *calobj_copy;
+	char *new_calobj;
+	GNOME_Evolution_Calendar_Cal_Busy *err;
+	CalBackendSendResult result;
+	char error_msg[256];
+	
+	cal = CAL (bonobo_object_from_servant (servant));
+	priv = cal->priv;
+
+	result = cal_backend_send_object (priv->backend, calobj, &new_calobj, user_list, error_msg);
+	switch (result) {
+	case CAL_BACKEND_SEND_SUCCESS:
+		calobj_copy = CORBA_string_dup (new_calobj);
+		g_free (new_calobj);
+
+		return calobj_copy;
+
+	case CAL_BACKEND_SEND_INVALID_OBJECT:
+		bonobo_exception_set (ev, ex_GNOME_Evolution_Calendar_Cal_InvalidObject);
+		break;
+
+	case CAL_BACKEND_SEND_BUSY:
+		err = GNOME_Evolution_Calendar_Cal_Busy__alloc ();
+		err->errorMsg = CORBA_string_dup (error_msg);
+		CORBA_exception_set (ev, CORBA_USER_EXCEPTION, ex_GNOME_Evolution_Calendar_Cal_Busy, err);
+		break;
+
+	case CAL_BACKEND_SEND_PERMISSION_DENIED:
+		bonobo_exception_set (ev, ex_GNOME_Evolution_Calendar_Cal_PermissionDenied);
+		break;
+
+	default :
+		g_assert_not_reached ();
+	}
+
+	return NULL;
 }
 
 /* Cal::getQuery implementation */
@@ -428,7 +504,7 @@ impl_Cal_get_query (PortableServer_Servant servant,
 	cal = CAL (bonobo_object_from_servant (servant));
 	priv = cal->priv;
 
-	query = query_new (priv->backend, ql, sexp);
+	query = cal_backend_get_query (priv->backend, ql, sexp);
 	if (!query) {
 		bonobo_exception_set (ev, ex_GNOME_Evolution_Calendar_Cal_CouldNotCreate);
 		return CORBA_OBJECT_NIL;
@@ -642,6 +718,7 @@ cal_class_init (CalClass *klass)
 	epv->getAlarmsForObject = impl_Cal_get_alarms_for_object;
 	epv->updateObjects = impl_Cal_update_objects;
 	epv->removeObject = impl_Cal_remove_object;
+	epv->sendObject = impl_Cal_send_object; 
 	epv->getQuery = impl_Cal_get_query;
 }
 
