@@ -29,10 +29,19 @@
 /* (from glibc headers:
    POSIX says that <sys/types.h> must be included (by the caller) before <regex.h>.  */
 
+#include <stdio.h>
+#include <stdlib.h>
 #include <sys/types.h>
 #include <regex.h>
 #include <string.h>
+#include <unistd.h>
 #include <ctype.h>
+#include <fcntl.h>
+#include <errno.h>
+
+#include <signal.h>
+#include <sys/wait.h>
+
 
 #ifdef HAVE_ALLOCA_H
 #include <alloca.h>
@@ -47,6 +56,7 @@
 #include "camel-exception.h"
 #include "camel-multipart.h"
 #include "camel-stream-mem.h"
+#include "camel-stream-fs.h"
 #include "camel-search-private.h"
 
 #include "camel-url.h"
@@ -54,6 +64,8 @@
 #define d(x)
 
 typedef struct {
+	CamelFilterSearchGetMessageFunc get_message;
+	void *get_message_data;
 	CamelMimeMessage *message;
 	CamelMessageInfo *info;
 	const char *source;
@@ -78,9 +90,9 @@ static ESExpResult *system_flag (struct _ESExp *f, int argc, struct _ESExpResult
 static ESExpResult *get_sent_date (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms);
 static ESExpResult *get_received_date (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms);
 static ESExpResult *get_current_date (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms);
-static ESExpResult *get_score (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms);
 static ESExpResult *get_source (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms);
 static ESExpResult *get_size (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms);
+static ESExpResult *pipe_message (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms);
 
 /* builtin functions */
 static struct {
@@ -106,11 +118,26 @@ static struct {
 	{ "get-sent-date",      (ESExpFunc *) get_sent_date,      0 },
 	{ "get-received-date",  (ESExpFunc *) get_received_date,  0 },
 	{ "get-current-date",   (ESExpFunc *) get_current_date,   0 },
-	{ "get-score",          (ESExpFunc *) get_score,          0 },
 	{ "get-source",         (ESExpFunc *) get_source,         0 },
 	{ "get-size",           (ESExpFunc *) get_size,           0 },
+	{ "pipe-message",       (ESExpFunc *) pipe_message,       0 },
 };
 
+
+static CamelMimeMessage *
+camel_filter_search_get_message (FilterMessageSearch *fms, struct _ESExp *sexp)
+{
+	if (fms->message)
+		return fms->message;
+	
+	fms->message = fms->get_message (fms->get_message_data, fms->ex);
+	
+	if (fms->message == NULL)
+		e_sexp_fatal_error (sexp, _("Failed to retrieve message"));
+	
+	return fms->message;
+}
+
 static ESExpResult *
 check_header (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms, camel_search_match_t how)
 {
@@ -124,21 +151,24 @@ check_header (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMess
 		camel_search_t type = CAMEL_SEARCH_TYPE_ENCODED;
 		CamelContentType *ct;
 		const char *charset = NULL;
-
+		
 		if (strcasecmp(name, "x-camel-mlist") == 0) {
 			header = camel_message_info_mlist(fms->info);
 			type = CAMEL_SEARCH_TYPE_MLIST;
 		} else {
-			header = camel_medium_get_header(CAMEL_MEDIUM(fms->message), argv[0]->value.string);
+			CamelMimeMessage *message = camel_filter_search_get_message (fms, f);
+			
+			header = camel_medium_get_header (CAMEL_MEDIUM (message), argv[0]->value.string);
+			/* FIXME: what about Resent-To, Resent-Cc and Resent-From? */
 			if (strcasecmp("to", name) == 0 || strcasecmp("cc", name) == 0 || strcasecmp("from", name) == 0)
 				type = CAMEL_SEARCH_TYPE_ADDRESS_ENCODED;
 			else {
-				ct = camel_mime_part_get_content_type(CAMEL_MIME_PART(fms->message));
+				ct = camel_mime_part_get_content_type (CAMEL_MIME_PART (message));
 				if (ct)
 					charset = e_iconv_charset_name(header_content_type_param(ct, "charset"));
 			}
 		}
-
+		
 		if (header) {
 			for (i=1; i<argc && !matched; i++) {
 				if (argv[i]->type == ESEXP_RES_STRING)
@@ -187,13 +217,16 @@ header_soundex (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMe
 static ESExpResult *
 header_exists (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms)
 {
+	CamelMimeMessage *message;
 	gboolean matched = FALSE;
 	ESExpResult *r;
 	int i;
 	
+	message = camel_filter_search_get_message (fms, f);
+	
 	for (i = 0; i < argc && !matched; i++) {
 		if (argv[i]->type == ESEXP_RES_STRING)
-			matched = camel_medium_get_header (CAMEL_MEDIUM (fms->message), argv[i]->value.string) != NULL;
+			matched = camel_medium_get_header (CAMEL_MEDIUM (message), argv[i]->value.string) != NULL;
 	}
 	
 	r = e_sexp_result_new (f, ESEXP_RES_BOOL);
@@ -206,11 +239,14 @@ static ESExpResult *
 header_regex (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms)
 {
 	ESExpResult *r = e_sexp_result_new (f, ESEXP_RES_BOOL);
+	CamelMimeMessage *message;
 	regex_t pattern;
 	const char *contents;
 	
+	message = camel_filter_search_get_message (fms, f);
+	
 	if (argc > 1 && argv[0]->type == ESEXP_RES_STRING
-	    && (contents = camel_medium_get_header (CAMEL_MEDIUM (fms->message), argv[0]->value.string))
+	    && (contents = camel_medium_get_header (CAMEL_MEDIUM (message), argv[0]->value.string))
 	    && camel_search_build_match_regex(&pattern, CAMEL_SEARCH_MATCH_REGEX|CAMEL_SEARCH_MATCH_ICASE, argc-1, argv+1, fms->ex) == 0) {
 		r->value.bool = regexec (&pattern, contents, 0, NULL, 0) == 0;
 		regfree (&pattern);
@@ -250,12 +286,14 @@ static ESExpResult *
 header_full_regex (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms)
 {
 	ESExpResult *r = e_sexp_result_new (f, ESEXP_RES_BOOL);
+	CamelMimeMessage *message;
 	regex_t pattern;
 	char *contents;
 	
 	if (camel_search_build_match_regex(&pattern, CAMEL_SEARCH_MATCH_REGEX|CAMEL_SEARCH_MATCH_ICASE|CAMEL_SEARCH_MATCH_NEWLINE,
 					   argc, argv, fms->ex) == 0) {
-		contents = get_full_header (fms->message);
+		message = camel_filter_search_get_message (fms, f);
+		contents = get_full_header (message);
 		r->value.bool = regexec (&pattern, contents, 0, NULL, 0) == 0;
 		g_free (contents);
 		regfree (&pattern);
@@ -284,10 +322,12 @@ static ESExpResult *
 body_contains (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms)
 {
 	ESExpResult *r = e_sexp_result_new (f, ESEXP_RES_BOOL);
+	CamelMimeMessage *message;
 	regex_t pattern;
 	
 	if (camel_search_build_match_regex (&pattern, CAMEL_SEARCH_MATCH_ICASE, argc, argv, fms->ex) == 0) {
-		r->value.bool = camel_search_message_body_contains ((CamelDataWrapper *)fms->message, &pattern);
+		message = camel_filter_search_get_message (fms, f);
+		r->value.bool = camel_search_message_body_contains ((CamelDataWrapper *) message, &pattern);
 		regfree (&pattern);
 	} else
 		r->value.bool = FALSE;
@@ -299,11 +339,13 @@ static ESExpResult *
 body_regex (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms)
 {
 	ESExpResult *r = e_sexp_result_new(f, ESEXP_RES_BOOL);
+	CamelMimeMessage *message;
 	regex_t pattern;
 	
 	if (camel_search_build_match_regex(&pattern, CAMEL_SEARCH_MATCH_ICASE|CAMEL_SEARCH_MATCH_REGEX|CAMEL_SEARCH_MATCH_NEWLINE,
 					   argc, argv, fms->ex) == 0) {
-		r->value.bool = camel_search_message_body_contains ((CamelDataWrapper *)fms->message, &pattern);
+		message = camel_filter_search_get_message (fms, f);
+		r->value.bool = camel_search_message_body_contains ((CamelDataWrapper *) message, &pattern);
 		regfree (&pattern);
 	} else
 		r->value.bool = FALSE;
@@ -337,13 +379,12 @@ static ESExpResult *
 system_flag (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms)
 {
 	ESExpResult *r;
-	gboolean truth = FALSE;
 	
-	if (argc == 1)
-		truth = camel_system_flag_get (fms->info->flags, argv[0]->value.string);
+	if (argc != 1 || argv[0]->type != ESEXP_RES_STRING)
+		e_sexp_fatal_error(f, _("Invalid arguments to (system-flag)"));
 	
 	r = e_sexp_result_new (f, ESEXP_RES_BOOL);
-	r->value.bool = truth;
+	r->value.bool = camel_system_flag_get (fms->info->flags, argv[0]->value.string);
 	
 	return r;
 }
@@ -353,6 +394,9 @@ user_tag (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageS
 {
 	ESExpResult *r;
 	const char *tag;
+
+	if (argc != 1 || argv[0]->type != ESEXP_RES_STRING)
+		e_sexp_fatal_error(f, _("Invalid arguments to (user-tag)"));
 	
 	tag = camel_tag_get (&fms->info->user_tags, argv[0]->value.string);
 	
@@ -365,10 +409,12 @@ user_tag (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageS
 static ESExpResult *
 get_sent_date (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms)
 {
+	CamelMimeMessage *message;
 	ESExpResult *r;
 	
+	message = camel_filter_search_get_message (fms, f);
 	r = e_sexp_result_new (f, ESEXP_RES_INT);
-	r->value.number = camel_mime_message_get_date (fms->message, NULL);
+	r->value.number = camel_mime_message_get_date (message, NULL);
 	
 	return r;
 }
@@ -376,10 +422,12 @@ get_sent_date (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMes
 static ESExpResult *
 get_received_date (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms)
 {
+	CamelMimeMessage *message;
 	ESExpResult *r;
 	
+	message = camel_filter_search_get_message (fms, f);
 	r = e_sexp_result_new (f, ESEXP_RES_INT);
-	r->value.number = camel_mime_message_get_date_received (fms->message, NULL);
+	r->value.number = camel_mime_message_get_date_received (message, NULL);
 	
 	return r;
 }
@@ -395,31 +443,14 @@ get_current_date (struct _ESExp *f, int argc, struct _ESExpResult **argv, Filter
 	return r;
 }
 
-static ESExpResult *
-get_score (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms)
-{
-	ESExpResult *r;
-	const char *tag;
-	
-	tag = camel_tag_get (&fms->info->user_tags, "score");
-	
-	r = e_sexp_result_new (f, ESEXP_RES_INT);
-	if (tag)
-		r->value.number = atoi (tag);
-	else
-		r->value.number = 0;
-	
-	return r;
-}
-
 static ESExpResult *
 get_source (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms)
 {
+	CamelMimeMessage *message;
 	ESExpResult *r;
 	char *src = NULL;
 	char *tmp;
 	
-	r = e_sexp_result_new (f, ESEXP_RES_STRING);
 	if (fms->source) {
 		CamelURL *url;
 		
@@ -429,18 +460,24 @@ get_source (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessag
 			camel_url_free (url);
 		}
 	} else {
-		src = g_strdup (camel_mime_message_get_source (fms->message));
+		message = camel_filter_search_get_message (fms, f);
+		src = g_strdup (camel_mime_message_get_source (message));
 	}
 	
 	/* This is an abusive hack */
-	if ( src && (tmp = strstr (src, "://")) ) {
+	if (src && (tmp = strstr (src, "://"))) {
 		tmp += 3;
 		tmp = strchr (tmp, '/');
 		if (tmp)
 			*tmp = '\0';
 	}
 	
-	r->value.string = src;
+	if (src) {
+		r = e_sexp_result_new (f, ESEXP_RES_STRING);
+		r->value.string = src;
+	} else {
+		r = e_sexp_result_new (f, ESEXP_RES_UNDEFINED);
+	}
 	
 	return r;
 }
@@ -457,6 +494,117 @@ get_size (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageS
 	return r;
 }
 
+static int
+run_command (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms)
+{
+	CamelMimeMessage *message;
+	CamelStream *stream;
+	int result, status;
+	int in_fds[2];
+	pid_t pid;
+	
+	if (argc < 1 || argv[0]->value.string[0] == '\0')
+		return 0;
+	
+	if (pipe (in_fds) == -1) {
+		camel_exception_setv (fms->ex, CAMEL_EXCEPTION_SYSTEM,
+				      _("Failed to create pipe to '%s': %s"),
+				      argv[0]->value.string, g_strerror (errno));
+		return -1;
+	}
+	
+	if (!(pid = fork ())) {
+		/* child process */
+		GPtrArray *args;
+		int maxfd, i;
+		
+		if (dup2 (in_fds[0], STDIN_FILENO) < 0)
+			_exit (255);
+		
+		setsid ();
+		
+		maxfd = sysconf (_SC_OPEN_MAX);
+		if (maxfd > 0) {
+			for (i = 0; i < maxfd; i++) {
+				if (i != STDIN_FILENO && i != STDOUT_FILENO && i != STDERR_FILENO)
+					close (i);
+			}
+		}
+		
+		args = g_ptr_array_new ();
+		for (i = 0; i < argc; i++)
+			g_ptr_array_add (args, argv[i]->value.string);
+		g_ptr_array_add (args, NULL);
+		
+		execvp (argv[0]->value.string, (char **) args->pdata);
+		
+		g_ptr_array_free (args, TRUE);
+		
+		d(printf ("Could not execute %s: %s\n", argv[0]->value.string, g_strerror (errno)));
+		_exit (255);
+	} else if (pid < 0) {
+		camel_exception_setv (fms->ex, CAMEL_EXCEPTION_SYSTEM,
+				      _("Failed to create create child process '%s': %s"),
+				      argv[0]->value.string, g_strerror (errno));
+		return -1;
+	}
+	
+	/* parent process */
+	close (in_fds[0]);
+	fcntl (in_fds[1], F_SETFL, O_NONBLOCK);
+	
+	stream = camel_stream_fs_new_with_fd (in_fds[1]);
+	
+	message = camel_filter_search_get_message (fms, f);
+	
+	camel_data_wrapper_write_to_stream (CAMEL_DATA_WRAPPER (message), stream);
+	camel_stream_flush (stream);
+	camel_object_unref (CAMEL_OBJECT (stream));
+	
+	result = waitpid (pid, &status, 0);
+	
+	if (result == -1 && errno == EINTR) {
+		/* child process is hanging... */
+		kill (pid, SIGTERM);
+		sleep (1);
+		result = waitpid (pid, &status, WNOHANG);
+		if (result == 0) {
+			/* ...still hanging, set phasers to KILL */
+			kill (pid, SIGKILL);
+			sleep (1);
+			result = waitpid (pid, &status, WNOHANG);
+		}
+	}
+	
+	if (result != -1 && WIFEXITED (status))
+		return WEXITSTATUS (status);
+	else
+		return -1;
+}
+
+static ESExpResult *
+pipe_message (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageSearch *fms)
+{
+	ESExpResult *r;
+	int retval, i;
+	
+	/* make sure all args are strings */
+	for (i = 0; i < argc; i++) {
+		if (argv[i]->type != ESEXP_RES_STRING) {
+			retval = -1;
+			goto done;
+		}
+	}
+	
+	retval = run_command (f, argc, argv, fms);
+	
+ done:
+	r = e_sexp_result_new (f, ESEXP_RES_INT);
+	r->value.number = retval;
+	
+	return r;
+}
+
 
 /**
  * camel_filter_search_match:
@@ -469,8 +617,9 @@ get_size (struct _ESExp *f, int argc, struct _ESExpResult **argv, FilterMessageS
  * Returns one of CAMEL_SEARCH_MATCHED, CAMEL_SEARCH_NOMATCH, or CAMEL_SEARCH_ERROR.
  **/
 int
-camel_filter_search_match (CamelMimeMessage *message, CamelMessageInfo *info,
-			   const char *source, const char *expression, CamelException *ex)
+camel_filter_search_match (CamelFilterSearchGetMessageFunc get_message, void *data,
+			   CamelMessageInfo *info, const char *source,
+			   const char *expression, CamelException *ex)
 {
 	FilterMessageSearch fms;
 	ESExp *sexp;
@@ -478,7 +627,9 @@ camel_filter_search_match (CamelMimeMessage *message, CamelMessageInfo *info,
 	gboolean retval;
 	int i;
 	
-	fms.message = message;
+	fms.get_message = get_message;
+	fms.get_message_data = data;
+	fms.message = NULL;
 	fms.info = info;
 	fms.source = source;
 	fms.ex = ex;
@@ -499,6 +650,7 @@ camel_filter_search_match (CamelMimeMessage *message, CamelMessageInfo *info,
 					      e_sexp_error (sexp), expression);
 		goto error;
 	}
+	
 	result = e_sexp_eval (sexp);
 	if (result == NULL) {
 		if (!camel_exception_is_set (ex))
@@ -515,9 +667,16 @@ camel_filter_search_match (CamelMimeMessage *message, CamelMessageInfo *info,
 	e_sexp_result_free (sexp, result);
 	e_sexp_unref (sexp);
 	
+	if (fms.message)
+		camel_object_unref (CAMEL_OBJECT (fms.message));
+	
 	return retval;
 	
-error:
+ error:
+	if (fms.message)
+		camel_object_unref (CAMEL_OBJECT (fms.message));
+	
 	e_sexp_unref (sexp);
+	
 	return CAMEL_SEARCH_ERROR;
 }
