@@ -3,45 +3,52 @@
  *
  *  Authors: Michael Zucchi <notzed@ximian.com>
  *
- *  This program is free software; you can redistribute it and/or 
- *  modify it under the terms of the GNU General Public License as 
- *  published by the Free Software Foundation; either version 2 of the
- *  License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- *  USA
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+
 #include <unistd.h>
 #include <ctype.h>
 #include <string.h>
 #include <errno.h>
 #include <stdlib.h>
 
+#include <gal/util/e-iconv.h>
+
 #include "camel-folder-summary.h"
 
 #include <camel/camel-file-utils.h>
 #include <camel/camel-mime-filter.h>
 #include <camel/camel-mime-filter-index.h>
 #include <camel/camel-mime-filter-charset.h>
-#include <camel/camel-mime-filter-save.h>
 #include <camel/camel-mime-filter-basic.h>
 #include <camel/camel-mime-filter-html.h>
 #include <camel/camel-mime-message.h>
 #include <camel/camel-multipart.h>
 #include <camel/camel-stream-mem.h>
 
+#include <camel/camel-stream-null.h>
+#include <camel/camel-stream-filter.h>
+
 #include "hash-table-utils.h"
 #include "e-util/md5-utils.h"
 #include "e-util/e-memory.h"
@@ -206,11 +213,18 @@ camel_folder_summary_finalize (CamelObject *obj)
 		camel_object_unref((CamelObject *)p->filter_64);
 	if (p->filter_qp)
 		camel_object_unref((CamelObject *)p->filter_qp);
+	if (p->filter_uu)
+		camel_object_unref((CamelObject *)p->filter_uu);
 	if (p->filter_save)
 		camel_object_unref((CamelObject *)p->filter_save);
 	if (p->filter_html)
 		camel_object_unref((CamelObject *)p->filter_html);
 
+	if (p->filter_stream)
+		camel_object_unref((CamelObject *)p->filter_stream);
+	if (p->index)
+		camel_object_unref((CamelObject *)p->index);
+
 #ifdef ENABLE_THREADS
 	g_mutex_free(p->summary_lock);
 	g_mutex_free(p->io_lock);
@@ -281,11 +295,16 @@ void camel_folder_summary_set_filename(CamelFolderSummary *s, const char *name)
  *
  * Unlike earlier behaviour, build_content need not be set to perform indexing.
  **/
-void camel_folder_summary_set_index(CamelFolderSummary *s, ibex *index)
+void camel_folder_summary_set_index(CamelFolderSummary *s, CamelIndex *index)
 {
 	struct _CamelFolderSummaryPrivate *p = _PRIVATE(s);
 
+	if (p->index)
+		camel_object_unref((CamelObject *)p->index);
+
 	p->index = index;
+	if (index)
+		camel_object_ref((CamelObject *)index);
 }
 
 /**
@@ -335,18 +354,17 @@ camel_folder_summary_index(CamelFolderSummary *s, int i)
 {
 	CamelMessageInfo *info = NULL;
 
-	CAMEL_SUMMARY_LOCK(s, ref_lock);
 	CAMEL_SUMMARY_LOCK(s, summary_lock);
+	CAMEL_SUMMARY_LOCK(s, ref_lock);
 
 	if (i<s->messages->len)
 		info = g_ptr_array_index(s->messages, i);
 
-	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
-
 	if (info)
 		info->refcount++;
 
 	CAMEL_SUMMARY_UNLOCK(s, ref_lock);
+	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 
 	return info;
 }
@@ -368,8 +386,8 @@ camel_folder_summary_array(CamelFolderSummary *s)
 	GPtrArray *res = g_ptr_array_new();
 	int i;
 	
-	CAMEL_SUMMARY_LOCK(s, ref_lock);
 	CAMEL_SUMMARY_LOCK(s, summary_lock);
+	CAMEL_SUMMARY_LOCK(s, ref_lock);
 
 	g_ptr_array_set_size(res, s->messages->len);
 	for (i=0;i<s->messages->len;i++) {
@@ -377,8 +395,8 @@ camel_folder_summary_array(CamelFolderSummary *s)
 		info->refcount++;
 	}
 
-	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 	CAMEL_SUMMARY_UNLOCK(s, ref_lock);
+	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 
 	return res;
 }
@@ -420,17 +438,16 @@ camel_folder_summary_uid(CamelFolderSummary *s, const char *uid)
 {
 	CamelMessageInfo *info;
 
-	CAMEL_SUMMARY_LOCK(s, ref_lock);
 	CAMEL_SUMMARY_LOCK(s, summary_lock);
+	CAMEL_SUMMARY_LOCK(s, ref_lock);
 
 	info = g_hash_table_lookup(s->messages_uid, uid);
 
-	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
-
 	if (info)
 		info->refcount++;
 
 	CAMEL_SUMMARY_UNLOCK(s, ref_lock);
+	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 
 	return info;
 }
@@ -532,7 +549,8 @@ camel_folder_summary_load(CamelFolderSummary *s)
 	int i;
 	CamelMessageInfo *mi;
 
-	g_assert(s->summary_path);
+	if (s->summary_path == NULL)
+		return 0;
 
 	in = fopen(s->summary_path, "r");
 	if (in == NULL)
@@ -611,18 +629,23 @@ camel_folder_summary_save(CamelFolderSummary *s)
 	int i;
 	guint32 count;
 	CamelMessageInfo *mi;
+	char *path;
 
-	g_assert(s->summary_path);
-
-	if ((s->flags & CAMEL_SUMMARY_DIRTY) == 0)
+	if (s->summary_path == NULL
+	    || (s->flags & CAMEL_SUMMARY_DIRTY) == 0)
 		return 0;
 
-	fd = open(s->summary_path, O_RDWR|O_CREAT, 0600);
+	path = alloca(strlen(s->summary_path)+4);
+	sprintf(path, "%s~", s->summary_path);
+	fd = open(path, O_RDWR|O_CREAT|O_TRUNC, 0600);
 	if (fd == -1)
 		return -1;
 	out = fdopen(fd, "w");
 	if ( out == NULL ) {
+		i = errno;
+		unlink(path);
 		close(fd);
+		errno = i;
 		return -1;
 	}
 
@@ -631,13 +654,16 @@ camel_folder_summary_save(CamelFolderSummary *s)
 	CAMEL_SUMMARY_LOCK(s, io_lock);
 
 	if ( ((CamelFolderSummaryClass *)(CAMEL_OBJECT_GET_CLASS(s)))->summary_header_save(s, out) == -1) {
+		i = errno;
 		fclose(out);
 		CAMEL_SUMMARY_UNLOCK(s, io_lock);
+		unlink(path);
+		errno = i;
 		return -1;
 	}
 
 	/* now write out each message ... */
-	/* FIXME: check returns */
+	/* we check ferorr when done for i/o errors */
 
 	count = s->messages->len;
 	for (i=0;i<count;i++) {
@@ -651,8 +677,19 @@ camel_folder_summary_save(CamelFolderSummary *s)
 
 	CAMEL_SUMMARY_UNLOCK(s, io_lock);
 
-	if (fclose(out) == -1)
+	if (ferror(out) != 0 ||  fclose(out) != 0) {
+		i = errno;
+		unlink(path);
+		errno = i;
 		return -1;
+	}
+
+	if (rename(path, s->summary_path) == -1) {
+		i = errno;
+		unlink(path);
+		errno = i;
+		return -1;
+	}
 
 	s->flags &= ~CAMEL_SUMMARY_DIRTY;
 	return 0;
@@ -673,7 +710,8 @@ int camel_folder_summary_header_load(CamelFolderSummary *s)
 	FILE *in;
 	int ret;
 
-	g_assert(s->summary_path);
+	if (s->summary_path == NULL)
+		return 0;
 
 	in = fopen(s->summary_path, "r");
 	if (in == NULL)
@@ -688,10 +726,11 @@ int camel_folder_summary_header_load(CamelFolderSummary *s)
 	return ret;
 }
 
-static void
+static int
 summary_assign_uid(CamelFolderSummary *s, CamelMessageInfo *info)
 {
 	const char *uid;
+	CamelMessageInfo *mi;
 
 	uid = camel_message_info_uid(info);
 	if (uid == NULL || uid[0] == 0) {
@@ -701,9 +740,11 @@ summary_assign_uid(CamelFolderSummary *s, CamelMessageInfo *info)
 
 	CAMEL_SUMMARY_LOCK(s, summary_lock);
 
-	while (g_hash_table_lookup(s->messages_uid, uid)) {
-		g_warning("Trying to insert message with clashing uid (%s).  new uid re-assigned", camel_message_info_uid(info));
+	while ((mi = g_hash_table_lookup(s->messages_uid, uid))) {
 		CAMEL_SUMMARY_UNLOCK(s, summary_lock);
+		if (mi == info)
+			return 0;
+		g_warning("Trying to insert message with clashing uid (%s).  new uid re-assigned", camel_message_info_uid(info));
 		camel_message_info_set_uid(info, camel_folder_summary_next_uid_string(s));
 		uid = camel_message_info_uid(info);
 		info->flags |= CAMEL_MESSAGE_FOLDER_FLAGGED;
@@ -711,6 +752,7 @@ summary_assign_uid(CamelFolderSummary *s, CamelMessageInfo *info)
 	}
 
 	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
+	return 1;
 }
 
 /**
@@ -730,7 +772,8 @@ void camel_folder_summary_add(CamelFolderSummary *s, CamelMessageInfo *info)
 	if (info == NULL)
 		return;
 
-	summary_assign_uid(s, info);
+	if (summary_assign_uid(s, info) == 0)
+		return;
 
 	CAMEL_SUMMARY_LOCK(s, summary_lock);
 
@@ -853,6 +896,7 @@ CamelMessageInfo *camel_folder_summary_info_new_from_parser(CamelFolderSummary *
 	int len;
 	struct _CamelFolderSummaryPrivate *p = _PRIVATE(s);
 	off_t start;
+	CamelIndexName *name = NULL;
 
 	/* should this check the parser is in the right state, or assume it is?? */
 
@@ -871,15 +915,21 @@ CamelMessageInfo *camel_folder_summary_info_new_from_parser(CamelFolderSummary *
 
 		if (p->index) {
 			if (p->filter_index == NULL)
-				p->filter_index = camel_mime_filter_index_new_ibex(p->index);
-			camel_mime_filter_index_set_name(p->filter_index, (char *)camel_message_info_uid(info));
-			ibex_unindex(p->index, (char *)camel_message_info_uid(info));
-			ibex_index_buffer(p->index, (char *)camel_message_info_uid(info), "ibexindexed", strlen("ibexindexed"), NULL);
+				p->filter_index = camel_mime_filter_index_new_index(p->index);
+			camel_index_delete_name(p->index, camel_message_info_uid(info));
+			name = camel_index_add_name(p->index, camel_message_info_uid(info));
+			camel_mime_filter_index_set_name(p->filter_index, name);
 		}
 
 		/* always scan the content info, even if we dont save it */
 		info->content = summary_build_content_info(s, info, mp);
 
+		if (name) {
+			camel_index_write_name(p->index, name);
+			camel_object_unref((CamelObject *)name);
+			camel_mime_filter_index_set_name(p->filter_index, NULL);
+		}
+
 		CAMEL_SUMMARY_UNLOCK(s, filter_lock);
 
 		info->size = camel_mime_parser_tell(mp) - start;
@@ -900,20 +950,42 @@ CamelMessageInfo *camel_folder_summary_info_new_from_message(CamelFolderSummary
 {
 	CamelMessageInfo *info;
 	struct _CamelFolderSummaryPrivate *p = _PRIVATE(s);
+	CamelIndexName *name = NULL;
 
 	info = ((CamelFolderSummaryClass *)(CAMEL_OBJECT_GET_CLASS(s)))->message_info_new_from_message(s, msg);
 
 	/* assign a unique uid, this is slightly 'wrong' as we do not really
-	 * know if we are going to store this in the summary, but no matter */
-	summary_assign_uid(s, info);
+	 * know if we are going to store this in the summary, but we need it set for indexing */
+	if (p->index)
+		summary_assign_uid(s, info);
+
+	CAMEL_SUMMARY_LOCK(s, filter_lock);
 
 	if (p->index) {
-		ibex_unindex(p->index, (char *)camel_message_info_uid(info));
-		ibex_index_buffer(p->index, (char *)camel_message_info_uid(info), "ibexindexed", strlen("ibexindexed"), NULL);
+		if (p->filter_index == NULL)
+			p->filter_index = camel_mime_filter_index_new_index(p->index);
+		camel_index_delete_name(p->index, camel_message_info_uid(info));
+		name = camel_index_add_name(p->index, camel_message_info_uid(info));
+		camel_mime_filter_index_set_name(p->filter_index, name);
+
+		if (p->filter_stream == NULL) {
+			CamelStream *null = camel_stream_null_new();
+
+			p->filter_stream = camel_stream_filter_new_with_stream(null);
+			camel_object_unref((CamelObject *)null);
+		}
 	}
 
 	info->content = summary_build_content_info_message(s, info, (CamelMimePart *)msg);
 
+	if (name) {
+		camel_index_write_name(p->index, name);
+		camel_object_unref((CamelObject *)name);
+		camel_mime_filter_index_set_name(p->filter_index, NULL);
+	}
+
+	CAMEL_SUMMARY_UNLOCK(s, filter_lock);
+
 	return info;
 }
 
@@ -1061,18 +1133,18 @@ void camel_folder_summary_remove_uid(CamelFolderSummary *s, const char *uid)
         CamelMessageInfo *oldinfo;
         char *olduid;
 
-	CAMEL_SUMMARY_LOCK(s, ref_lock);
 	CAMEL_SUMMARY_LOCK(s, summary_lock);
+	CAMEL_SUMMARY_LOCK(s, ref_lock);
         if (g_hash_table_lookup_extended(s->messages_uid, uid, (void *)&olduid, (void *)&oldinfo)) {
 		/* make sure it doesn't vanish while we're removing it */
 		oldinfo->refcount++;
-		CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 		CAMEL_SUMMARY_UNLOCK(s, ref_lock);
+		CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 		camel_folder_summary_remove(s, oldinfo);
 		camel_folder_summary_info_free(s, oldinfo);
         } else {
-		CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 		CAMEL_SUMMARY_UNLOCK(s, ref_lock);
+		CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 	}
 }
 
@@ -1477,7 +1549,7 @@ message_info_new(CamelFolderSummary *s, struct _header_raw *h)
 	CamelMessageInfo *mi;
 	const char *received;
 	guchar digest[16];
-	struct _header_references *refs, *scan;
+	struct _header_references *refs, *irt, *scan;
 	char *msgid;
 	int count;
 	char *subject, *from, *to, *cc, *mlist;
@@ -1492,7 +1564,7 @@ message_info_new(CamelFolderSummary *s, struct _header_raw *h)
 	     && (strcasecmp(charset, "us-ascii") == 0))
 		charset = NULL;
 
-	charset = camel_charset_to_iconv(charset);
+	charset = e_iconv_charset_name(charset);
 
 	subject = summary_format_string(h, "subject", charset);
 	from = summary_format_address(h, "from");
@@ -1540,21 +1612,30 @@ message_info_new(CamelFolderSummary *s, struct _header_raw *h)
 		memcpy(mi->message_id.id.hash, digest, sizeof(mi->message_id.id.hash));
 		g_free(msgid);
 	}
-	/* if we have a references, use that, otherwise, see if we have an in-reply-to
-	   header, with parsable content, otherwise *shrug* */
-	if ((refs = header_references_decode(header_raw_find(&h, "references", NULL))) != NULL
-	    || (refs = header_references_decode(header_raw_find(&h, "in-reply-to", NULL))) != NULL) {
+	
+	/* decode our references and in-reply-to headers */
+	refs = header_references_decode (header_raw_find (&h, "references", NULL));
+	irt = header_references_inreplyto_decode (header_raw_find (&h, "in-reply-to", NULL));
+	if (refs || irt) {
+		if (irt) {
+			/* The References field is populated from the ``References'' and/or ``In-Reply-To''
+			   headers. If both headers exist, take the first thing in the In-Reply-To header
+			   that looks like a Message-ID, and append it to the References header. */
+			
+			if (refs)
+				irt->next = refs;
+			
+			refs = irt;
+		}
+		
 		count = header_references_list_size(&refs);
 		mi->references = g_malloc(sizeof(*mi->references) + ((count-1) * sizeof(mi->references->references[0])));
 		count = 0;
 		scan = refs;
 		while (scan) {
-			/* FIXME: the id might be NULL because of a small bug in camel-mime-utils */
-			if (scan->id) {
-				md5_get_digest(scan->id, strlen(scan->id), digest);
-				memcpy(mi->references->references[count].id.hash, digest, sizeof(mi->message_id.id.hash));
-				count++;
-			}
+			md5_get_digest(scan->id, strlen(scan->id), digest);
+			memcpy(mi->references->references[count].id.hash, digest, sizeof(mi->message_id.id.hash));
+			count++;
 			scan = scan->next;
 		}
 		mi->references->size = count;
@@ -1633,7 +1714,8 @@ message_info_load(CamelFolderSummary *s, FILE *in)
 
 	for (i=0;i<count;i++) {
 		char *name;
-		camel_file_util_decode_string(in, &name);
+		if (camel_file_util_decode_string(in, &name) == -1 || name == NULL)
+			goto error;
 		camel_flag_set(&mi->user_flags, name, TRUE);
 		g_free(name);
 	}
@@ -1643,8 +1725,9 @@ message_info_load(CamelFolderSummary *s, FILE *in)
 
 	for (i=0;i<count;i++) {
 		char *name, *value;
-		camel_file_util_decode_string(in, &name);
-		camel_file_util_decode_string(in, &value);
+		if (camel_file_util_decode_string(in, &name) == -1 || name == NULL
+		    || camel_file_util_decode_string(in, &value) == -1)
+			goto error;
 		camel_tag_set(&mi->user_tags, name, value);
 		g_free(name);
 		g_free(value);
@@ -1742,7 +1825,7 @@ content_info_new (CamelFolderSummary *s, struct _header_raw *h)
 	
 	ci = camel_folder_summary_content_info_new (s);
 	
-	charset = camel_charset_locale_name ();
+	charset = e_iconv_locale_charset();
 	ci->id = header_msgid_decode (header_raw_find (&h, "content-id", NULL));
 	ci->description = header_decode_string (header_raw_find (&h, "content-description", NULL), NULL);
 	ci->encoding = header_content_encoding_decode (header_raw_find (&h, "content-transfer-encoding", NULL));
@@ -1769,11 +1852,14 @@ content_info_load(CamelFolderSummary *s, FILE *in)
 	g_free(subtype);
 	if (camel_file_util_decode_uint32(in, &count) == -1 || count > 500)
 		goto error;
-	    
+	
 	for (i=0;i<count;i++) {
 		char *name, *value;
 		camel_folder_summary_decode_token(in, &name);
 		camel_folder_summary_decode_token(in, &value);
+		if (!(name && value))
+			goto error;
+		
 		header_content_type_set_param(ct, name, value);
 		/* TODO: do this so we dont have to double alloc/free */
 		g_free(name);
@@ -1792,7 +1878,7 @@ content_info_load(CamelFolderSummary *s, FILE *in)
 	if (!ferror(in))
 		return ci;
 
-error:
+ error:
 	camel_folder_summary_content_info_free(s, ci);
 	return NULL;
 }
@@ -1902,6 +1988,13 @@ summary_build_content_info(CamelFolderSummary *s, CamelMessageInfo *msginfo, Cam
 					else
 						camel_mime_filter_reset((CamelMimeFilter *)p->filter_qp);
 					enc_id = camel_mime_parser_filter_add(mp, (CamelMimeFilter *)p->filter_qp);
+				} else if (!strcasecmp (encoding, "x-uuencode")) {
+					d(printf(" decoding x-uuencode\n"));
+					if (p->filter_uu == NULL)
+						p->filter_uu = camel_mime_filter_basic_new_type(CAMEL_MIME_FILTER_BASIC_UU_DEC);
+					else
+						camel_mime_filter_reset((CamelMimeFilter *)p->filter_uu);
+					enc_id = camel_mime_parser_filter_add(mp, (CamelMimeFilter *)p->filter_uu);
 				} else {
 					d(printf(" ignoring encoding %s\n", encoding));
 				}
@@ -1990,7 +2083,7 @@ summary_build_content_info(CamelFolderSummary *s, CamelMessageInfo *msginfo, Cam
 }
 
 /* build the content-info, from a message */
-/* this needs no lock, as we copy all data, and ibex is threadsafe */
+/* this needs the filter lock since it uses filters to perform indexing */
 static CamelMessageContentInfo *
 summary_build_content_info_message(CamelFolderSummary *s, CamelMessageInfo *msginfo, CamelMimePart *object)
 {
@@ -2036,14 +2129,25 @@ summary_build_content_info_message(CamelFolderSummary *s, CamelMessageInfo *msgi
 			child->parent = info;
 			my_list_append((struct _node **)&info->childs, (struct _node *)child);
 		}
-	} else if (p->index
+	} else if (p->filter_stream
 		   && header_content_type_is(CAMEL_DATA_WRAPPER(containee)->mime_type, "text", "*")) {
-		/* index all text parts if we're indexing */
-		CamelStreamMem *mem = (CamelStreamMem *)camel_stream_mem_new();
+		int html_id = -1, idx_id = -1;
 
-		camel_data_wrapper_write_to_stream(containee, (CamelStream *)mem);
-		ibex_index_buffer(p->index, (char *)camel_message_info_uid(msginfo), mem->buffer->data, mem->buffer->len, NULL);
-		camel_object_unref((CamelObject *)mem);
+		/* pre-attach html filter if required, otherwise just index filter */
+		if (header_content_type_is(CAMEL_DATA_WRAPPER(containee)->mime_type, "text", "html")) {
+			if (p->filter_html == NULL)
+				p->filter_html = camel_mime_filter_html_new();
+			else
+				camel_mime_filter_reset((CamelMimeFilter *)p->filter_html);
+			html_id = camel_stream_filter_add(p->filter_stream, (CamelMimeFilter *)p->filter_html);
+		}
+		idx_id = camel_stream_filter_add(p->filter_stream, (CamelMimeFilter *)p->filter_index);
+
+		camel_data_wrapper_write_to_stream(containee, (CamelStream *)p->filter_stream);
+		camel_stream_flush((CamelStream *)p->filter_stream);
+
+		camel_stream_filter_remove(p->filter_stream, idx_id);
+		camel_stream_filter_remove(p->filter_stream, html_id);
 	}
 
 	return info;
@@ -2150,8 +2254,49 @@ camel_flag_list_free(CamelFlag **list)
 	*list = NULL;
 }
 
-const char
-*camel_tag_get(CamelTag **list, const char *name)
+/**
+ * camel_flag_list_copy:
+ * @to: 
+ * @from: 
+ * 
+ * Copy a flag list, return true if the destination list @to changed.
+ * 
+ * Return value: 
+ **/
+gboolean
+camel_flag_list_copy(CamelFlag **to, CamelFlag **from)
+{
+	CamelFlag *flag, *tmp;
+	int changed = FALSE;
+
+	if (*to == NULL && from == NULL)
+		return FALSE;
+
+	/* Remove any now-missing flags */
+	flag = (CamelFlag *)to;
+	while (flag->next) {
+		tmp = flag->next;
+		if (!camel_flag_get(from, tmp->name)) {
+			flag->next = tmp->next;
+			g_free(tmp);
+			changed = TRUE;
+		} else {
+			flag = tmp;
+		}
+	}
+
+	/* Add any new flags */
+	flag = *from;
+	while (flag) {
+		changed |= camel_flag_set(to, flag->name, TRUE);
+		flag = flag->next;
+	}
+
+	return changed;
+}
+
+const char *
+camel_tag_get(CamelTag **list, const char *name)
 {
 	CamelTag *tag;
 
@@ -2231,6 +2376,54 @@ int		camel_tag_list_size(CamelTag **list)
 	return count;
 }
 
+static void
+rem_tag(char *key, char *value, CamelTag **to)
+{
+	camel_tag_set(to, key, NULL);
+}
+
+/**
+ * camel_tag_list_copy:
+ * @to: 
+ * @from: 
+ * 
+ * Copy a list of tags.
+ * 
+ * Return value: 
+ **/
+gboolean
+camel_tag_list_copy(CamelTag **to, CamelTag **from)
+{
+	int changed = FALSE;
+	CamelTag *tag;
+	GHashTable *left;
+
+	if (*to == NULL && from == NULL)
+		return FALSE;
+
+	left = g_hash_table_new(g_str_hash, g_str_equal);
+	tag = *to;
+	while (tag) {
+		g_hash_table_insert(left, tag->name, tag);
+		tag = tag->next;
+	}
+
+	tag = *from;
+	while (tag) {
+		changed |= camel_tag_set(to, tag->name, tag->value);
+		g_hash_table_remove(left, tag->name);
+		tag = tag->next;
+	}
+
+	if (g_hash_table_size(left)>0) {
+		g_hash_table_foreach(left, (GHFunc)rem_tag, to);
+		changed = TRUE;
+	}
+	g_hash_table_destroy(left);
+
+	return changed;
+}
+
 /**
  * camel_tag_list_free:
  * @list: 
@@ -2350,20 +2543,21 @@ camel_message_info_new_from_header (struct _header_raw *header)
 	CamelMessageInfo *info;
 	char *subject, *from, *to, *cc, *mlist;
 	struct _header_content_type *ct = NULL;
-	const char *content, *charset = NULL;
-
+	const char *content, *date, *charset = NULL;
+	
 	if ((content = header_raw_find(&header, "Content-Type", NULL))
 	    && (ct = header_content_type_decode(content))
 	    && (charset = header_content_type_param(ct, "charset"))
 	    && (strcasecmp(charset, "us-ascii") == 0))
 		charset = NULL;
 
-	charset = camel_charset_to_iconv(charset);
+	charset = e_iconv_charset_name(charset);
 	
 	subject = summary_format_string(header, "subject", charset);
 	from = summary_format_address(header, "from");
 	to = summary_format_address(header, "to");
 	cc = summary_format_address(header, "cc");
+	date = header_raw_find(&header, "date", NULL);
 	mlist = header_raw_check_mailing_list(&header);
 
 	if (ct)
@@ -2376,7 +2570,21 @@ camel_message_info_new_from_header (struct _header_raw *header)
 	camel_message_info_set_to(info, to);
 	camel_message_info_set_cc(info, cc);
 	camel_message_info_set_mlist(info, mlist);
-
+	
+	if (date)
+		info->date_sent = header_decode_date (date, NULL);
+	else
+		info->date_sent = time (NULL);
+	
+	date = header_raw_find (&header, "received", NULL);
+	if (date && (date = strrchr (date, ';')))
+		date++;
+	
+	if (date)
+		info->date_received = header_decode_date (date, NULL);
+	else
+		info->date_received = time (NULL);
+	
 	return info;
 }
 
@@ -2523,34 +2731,35 @@ camel_message_info_set_string (CamelMessageInfo *mi, int type, char *str)
 }
 #endif
 
-#if 0
-static void
-content_info_dump(CamelMessageContentInfo *ci, int depth)
+
+void
+camel_content_info_dump (CamelMessageContentInfo *ci, int depth)
 {
 	char *p;
-
-	p = alloca(depth*4+1);
-	memset(p, ' ', depth*4);
-	p[depth*4] = 0;
-
+	
+	p = alloca (depth * 4 + 1);
+	memset (p, ' ', depth * 4);
+	p[depth * 4] = 0;
+	
 	if (ci == NULL) {
-		printf("%s<empty>\n", p);
+		printf ("%s<empty>\n", p);
 		return;
 	}
-
-	printf("%scontent-type: %s/%s\n", p, ci->type->type, ci->type->subtype);
-	printf("%scontent-transfer-encoding: %s\n", p, ci->encoding);
-	printf("%scontent-description: %s\n", p, ci->description);
-	printf("%ssize: %lu\n", p, (unsigned long)ci->size);
+	
+	printf ("%scontent-type: %s/%s\n", p, ci->type->type ? ci->type->type : "(null)",
+	       ci->type->subtype ? ci->type->subtype : "(null)");
+	printf ("%scontent-transfer-encoding: %s\n", p, ci->encoding ? ci->encoding : "(null)");
+	printf ("%scontent-description: %s\n", p, ci->description ? ci->description : "(null)");
+	printf ("%ssize: %lu\n", p, (unsigned long) ci->size);
 	ci = ci->childs;
 	while (ci) {
-		content_info_dump(ci, depth+1);
+		camel_content_info_dump (ci, depth + 1);
 		ci = ci->next;
 	}
 }
 
 void
-message_info_dump(CamelMessageInfo *mi)
+camel_message_info_dump (CamelMessageInfo *mi)
 {
 	if (mi == NULL) {
 		printf("No message?\n");
@@ -2564,106 +2773,5 @@ message_info_dump(CamelMessageInfo *mi)
 	printf("From: %s\n", camel_message_info_from(mi));
 	printf("UID: %s\n", camel_message_info_uid(mi));
 	printf("Flags: %04x\n", mi->flags & 0xffff);
-	content_info_dump(mi->content, 0);
+	camel_content_info_dump(mi->content, 0);
 }
-
-int main(int argc, char **argv)
-{
-	CamelMimeParser *mp;
-	int fd;
-	CamelFolderSummary *s;
-	char *buffer;
-	int len;
-	int i;
-	ibex *index;
-
-	/*g_tk_init(&argc, &argv);*/
-
-#if 0
-	{
-		int i;
-		char *s;
-		char buf[1024];
-
-		for (i=0;i<434712;i++) {
-			memcpy(buf, "                                                         ", 50);
-			buf[50] = 0;
-#if 0
-			s = g_strdup(buf);
-			g_free(s);
-#endif
-		}
-		return 0;
-	}
-#endif
-
-	if (argc < 2 ) {
-		printf("usage: %s mbox\n", argv[0]);
-		return 1;
-	}
-
-	fd = open(argv[1], O_RDONLY);
-
-	index = ibex_open("index.ibex", O_CREAT|O_RDWR, 0600);
-
-	mp = camel_mime_parser_new();
-	camel_mime_parser_scan_from(mp, TRUE);
-/*	camel_mime_parser_set_header_regex(mp, "^(content-[^:]*|subject|from|to|date):");*/
-	camel_mime_parser_init_with_fd(mp, fd);
-
-	s = camel_folder_summary_new();
-	camel_folder_summary_set_build_content(s, TRUE);
-/*	camel_folder_summary_set_index(s, index);*/
-
-	while (camel_mime_parser_step(mp, &buffer, &len) == HSCAN_FROM) {
-		/*printf("Parsing message ...\n");*/
-		camel_folder_summary_add_from_parser(s, mp);
-		if (camel_mime_parser_step(mp, &buffer, &len) != HSCAN_FROM_END) {
-			g_warning("Uknown state encountered, excpecting %d, got %d\n", HSCAN_FROM_END, camel_mime_parser_state(mp));
-			break;
-		}
-	}
-
-	printf("Printing summary\n");
-	for (i=0;i<camel_folder_summary_count(s);i++) {
-		CamelMessageInfo *info = camel_folder_summary_index(s, i);
-		message_info_dump(info);
-		camel_folder_summary_info_free(info);
-	}
-
-	printf("Saivng summary\n");
-	camel_folder_summary_set_filename(s, "index.summary");
-	camel_folder_summary_save(s);
-
-	{
-		CamelFolderSummary *n;
-
-		printf("\nLoading summary\n");
-		n = camel_folder_summary_new();
-		camel_folder_summary_set_build_content(n, TRUE);
-		camel_folder_summary_set_filename(n, "index.summary");
-		camel_folder_summary_load(n);
-
-		printf("Printing summary\n");
-		for (i=0;i<camel_folder_summary_count(n);i++) {
-			CamelMessageInfo *info = camel_folder_summary_index(s, i);
-			message_info_dump(info);
-			camel_folder_summary_info_free(info);
-		}
-		camel_object_unref(n);		
-	}
-
-
-	camel_object_unref(mp);
-	camel_object_unref(s);
-
-	printf("summarised %d messages\n", camel_folder_summary_count(s));
-#if 0
-	printf("g_strdup count = %d\n", strdup_count);
-	printf("g_malloc count = %d\n", malloc_count);
-	printf("g_free count = %d\n", free_count);
-#endif
-	return 0;
-}
-
-#endif
