@@ -6,19 +6,19 @@
  *  Copyright 2000 Ximian, Inc. (www.ximian.com)
  *  Copyright 2001 Ximian Inc. (www.ximian.com)
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  */
 
@@ -99,6 +99,8 @@ camel_search_build_match_regex (regex_t *pattern, camel_search_flags_t type, int
 	flags = REG_EXTENDED|REG_NOSUB;
 	if (type & CAMEL_SEARCH_MATCH_ICASE)
 		flags |= REG_ICASE;
+	if (type & CAMEL_SEARCH_MATCH_NEWLINE)
+		flags |= REG_NEWLINE;
 	err = regcomp (pattern, match->str, flags);
 	if (err != 0) {
 		/* regerror gets called twice to get the full error string 
@@ -192,13 +194,15 @@ header_soundex (const char *header, const char *match)
 	return truth;
 }
 
+/* FIXME: This is stupidly slow and needs to be removed */
 static gunichar
 utf8_get (const char **inp)
 {
 	const unsigned char *p = *inp;
 	gunichar c;
-	
-	g_return_val_if_fail (p != NULL, 0);
+
+	if (p == NULL)
+		return 0;
 	
 	c = g_utf8_get_char (p);
 	*inp = g_unichar_validate (c) ? g_utf8_next_char (p) : NULL;
@@ -206,7 +210,7 @@ utf8_get (const char **inp)
 	return c;
 }
 
-static const char *
+const char *
 camel_ustrstrcase (const char *haystack, const char *needle)
 {
 	gunichar *nuni, *puni;
@@ -344,7 +348,7 @@ header_match(const char *value, const char *match, camel_search_match_t how)
 	
 	if (how == CAMEL_SEARCH_MATCH_SOUNDEX)
 		return header_soundex (value, match);
-	
+
 	vlen = strlen(value);
 	mlen = strlen(match);
 	if (vlen < mlen)
@@ -394,9 +398,9 @@ gboolean
 camel_search_header_match (const char *value, const char *match, camel_search_match_t how, camel_search_t type, const char *default_charset)
 {
 	const char *name, *addr;
-	int truth = FALSE;
+	int truth = FALSE, i;
 	CamelInternetAddress *cia;
-	char *v;
+	char *v, *vdom, *mdom;
 
 	while (*value && isspace (*value))
 		value++;
@@ -407,6 +411,25 @@ camel_search_header_match (const char *value, const char *match, camel_search_ma
 		truth = header_match(v, match, how);
 		g_free(v);
 		break;
+	case CAMEL_SEARCH_TYPE_MLIST:
+		/* Special mailing list old-version domain hack
+		   If one of the mailing list names doesn't have an @ in it, its old-style, so
+		   only match against the pre-domain part, which should be common */
+
+		vdom = strchr(value, '@');
+		mdom = strchr(match, '@');
+		if (mdom == NULL && vdom != NULL) {
+			v = alloca(vdom-value+1);
+			memcpy(v, value, vdom-value);
+			v[vdom-value] = 0;
+			value = (char *)v;
+		} else if (mdom != NULL && vdom == NULL) {
+			v = alloca(mdom-match+1);
+			memcpy(v, match, mdom-match);
+			v[mdom-match] = 0;
+			match = (char *)v;
+		}
+		/* Falls through */
 	case CAMEL_SEARCH_TYPE_ASIS:
 		truth = header_match(value, match, how);
 		break;
@@ -423,11 +446,9 @@ camel_search_header_match (const char *value, const char *match, camel_search_ma
 		else
 			camel_address_unformat((CamelAddress *)cia, value);
 
-		if (camel_address_length((CamelAddress *)cia) == 1) {
-			camel_internet_address_get(cia, 0, &name, &addr);
-			truth = (name && header_match(name, match, how))
-				|| (addr && header_match(addr, match, how));
-		}
+		for (i=0; !truth && camel_internet_address_get(cia, i, &name, &addr);i++)
+			truth = (name && header_match(name, match, how)) || (addr && header_match(addr, match, how));
+
 		camel_object_unref((CamelObject *)cia);
 		break;
 	}
@@ -449,9 +470,6 @@ camel_search_message_body_contains (CamelDataWrapper *object, regex_t *pattern)
 	if (containee == NULL)
 		return FALSE;
 	
-	/* TODO: I find it odd that get_part and get_content_object do not
-	   add a reference, probably need fixing for multithreading */
-	
 	/* using the object types is more accurate than using the mime/types */
 	if (CAMEL_IS_MULTIPART (containee)) {
 		parts = camel_multipart_get_number (CAMEL_MULTIPART (containee));
@@ -476,3 +494,157 @@ camel_search_message_body_contains (CamelDataWrapper *object, regex_t *pattern)
 	return truth;
 }
 
+static __inline__ guint32
+camel_utf8_getc(const unsigned char **ptr)
+{
+	register unsigned char *p = (unsigned char *)*ptr;
+	register unsigned char c, r;
+	register guint32 v, m;
+
+again:
+	r = *p++;
+loop:
+	if (r < 0x80) {
+		*ptr = p;
+		v = r;
+	} else if (r < 0xfe) { /* valid start char? */
+		v = r;
+		m = 0x7f80;	/* used to mask out the length bits */
+		do {
+			c = *p++;
+			if ((c & 0xc0) != 0x80) {
+				r = c;
+				goto loop;
+			}
+			v = (v<<6) | (c & 0x3f);
+			r<<=1;
+			m<<=5;
+		} while (r & 0x40);
+		
+		*ptr = p;
+
+		v &= ~m;
+	} else {
+		goto again;
+	}
+
+	return v;
+}
+
+struct _camel_search_words *
+camel_search_words_split(const unsigned char *in)
+{
+	int type = CAMEL_SEARCH_WORD_SIMPLE, all = 0;
+	GString *w;
+	struct _camel_search_word *word;
+	struct _camel_search_words *words;
+	GPtrArray *list = g_ptr_array_new();
+	guint32 c;
+	int utf8len;
+	char utf8[8];
+
+	words = g_malloc0(sizeof(*words));	
+	w = g_string_new("");
+
+	do {
+		c = camel_utf8_getc(&in);
+		if (c == 0 || g_unichar_isspace(c)) {
+			if (w->len) {
+				word = g_malloc0(sizeof(*word));
+				word->word = g_strdup(w->str);
+				word->type = type;
+				g_ptr_array_add(list, word);
+				all |= type;
+				type = CAMEL_SEARCH_WORD_SIMPLE;
+				g_string_truncate(w, 0);
+			}
+		} else {
+			if (!g_unichar_isalnum(c))
+				type = CAMEL_SEARCH_WORD_COMPLEX;
+			else
+				c = g_unichar_tolower(c);
+			if (c > 0x80)
+				type |= CAMEL_SEARCH_WORD_8BIT;
+
+			utf8len = g_unichar_to_utf8(c, utf8);
+			utf8[utf8len] = 0;
+			g_string_append(w, utf8);
+		}
+	} while (c);
+
+	g_string_free(w, TRUE);
+	words->len = list->len;
+	words->words = (struct _camel_search_word **)list->pdata;
+	words->type = all;
+	g_ptr_array_free(list, FALSE);
+
+	return words;
+}
+
+/* takes an existing 'words' list, and converts it to another consisting of
+   only simple words, with any punctuation etc stripped */
+struct _camel_search_words *
+camel_search_words_simple(struct _camel_search_words *wordin)
+{
+	int i;
+	const unsigned char *ptr, *start, *last;
+	int type = CAMEL_SEARCH_WORD_SIMPLE, all = 0;
+	GPtrArray *list = g_ptr_array_new();
+	struct _camel_search_word *word;
+	struct _camel_search_words *words;
+	guint32 c;
+
+	words = g_malloc0(sizeof(*words));	
+
+	for (i=0;i<wordin->len;i++) {
+		if ((wordin->words[i]->type & CAMEL_SEARCH_WORD_COMPLEX) == 0) {
+			word = g_malloc0(sizeof(*word));
+			word->type = wordin->words[i]->type;
+			word->word = g_strdup(wordin->words[i]->word);
+			g_ptr_array_add(list, word);
+		} else {
+			ptr = wordin->words[i]->word;
+			start = last = ptr;
+			do {
+				c = camel_utf8_getc(&ptr);
+				if (c == 0 || !g_unichar_isalnum(c)) {
+					if (last > start) {
+						word = g_malloc0(sizeof(*word));
+						word->word = g_strndup(start, last-start);
+						word->type = type;
+						g_ptr_array_add(list, word);
+						all |= type;
+						type = CAMEL_SEARCH_WORD_SIMPLE;
+					}
+					start = ptr;
+				}
+				if (c > 0x80)
+					type = CAMEL_SEARCH_WORD_8BIT;
+				last = ptr;
+			} while (c);
+		}
+	}
+
+	words->len = list->len;
+	words->words = (struct _camel_search_word **)list->pdata;
+	words->type = all;
+	g_ptr_array_free(list, FALSE);
+
+	return words;
+}
+
+void
+camel_search_words_free(struct _camel_search_words *words)
+{
+	int i;
+
+	for (i=0;i<words->len;i++) {
+		struct _camel_search_word *word = words->words[i];
+
+		g_free(word->word);
+		g_free(word);
+	}
+	g_free(words->words);
+	g_free(words);
+}
+
