@@ -53,14 +53,19 @@ static void vee_refresh_info(CamelFolder *folder, CamelException *ex);
 static void vee_sync (CamelFolder *folder, gboolean expunge, CamelException *ex);
 static void vee_expunge (CamelFolder *folder, CamelException *ex);
 
+static void vee_freeze (CamelFolder *folder);
+static void vee_thaw (CamelFolder *folder);
+
 static CamelMimeMessage *vee_get_message (CamelFolder *folder, const gchar *uid, CamelException *ex);
-static void vee_move_messages_to(CamelFolder *source, GPtrArray *uids, CamelFolder *dest, CamelException *ex);
+static void vee_append_message(CamelFolder *folder, CamelMimeMessage *message, const CamelMessageInfo *info, char **appended_uid, CamelException *ex);
+static void vee_transfer_messages_to(CamelFolder *source, GPtrArray *uids, CamelFolder *dest, GPtrArray **transferred_uids, gboolean delete_originals, CamelException *ex);
 
 static GPtrArray *vee_search_by_expression(CamelFolder *folder, const char *expression, CamelException *ex);
 static GPtrArray *vee_search_by_uids(CamelFolder *folder, const char *expression, GPtrArray *uids, CamelException *ex);
 
 static void vee_set_message_flags (CamelFolder *folder, const char *uid, guint32 flags, guint32 set);
 static void vee_set_message_user_flag (CamelFolder *folder, const char *uid, const char *name, gboolean value);
+static void vee_set_message_user_tag(CamelFolder *folder, const char *uid, const char *name, const char *value);
 static void vee_rename(CamelFolder *folder, const char *new);
 
 static void camel_vee_folder_class_init (CamelVeeFolderClass *klass);
@@ -123,15 +128,20 @@ camel_vee_folder_class_init (CamelVeeFolderClass *klass)
 	folder_class->expunge = vee_expunge;
 
 	folder_class->get_message = vee_get_message;
-	folder_class->move_messages_to = vee_move_messages_to;
+	folder_class->append_message = vee_append_message;
+	folder_class->transfer_messages_to = vee_transfer_messages_to;
 
 	folder_class->search_by_expression = vee_search_by_expression;
 	folder_class->search_by_uids = vee_search_by_uids;
 
 	folder_class->set_message_flags = vee_set_message_flags;
 	folder_class->set_message_user_flag = vee_set_message_user_flag;
+	folder_class->set_message_user_tag = vee_set_message_user_tag;
 
 	folder_class->rename = vee_rename;
+
+	folder_class->freeze = vee_freeze;
+	folder_class->thaw = vee_thaw;
 }
 
 static void
@@ -333,7 +343,8 @@ void
 camel_vee_folder_add_folder(CamelVeeFolder *vf, CamelFolder *sub)
 {
 	struct _CamelVeeFolderPrivate *p = _PRIVATE(vf), *up = _PRIVATE(folder_unmatched);
-
+	int i;
+	
 	if (vf == (CamelVeeFolder *)sub) {
 		g_warning("Adding a virtual folder to itself as source, ignored");
 		return;
@@ -345,10 +356,17 @@ camel_vee_folder_add_folder(CamelVeeFolder *vf, CamelFolder *sub)
 	if (g_list_find(p->folders, sub) == NULL) {
 		camel_object_ref((CamelObject *)sub);
 		p->folders = g_list_append(p->folders, sub);
+		
+		/* update the freeze state of 'sub' to match our freeze state */
+		for (i = 0; i < p->freeze_count; i++)
+			camel_folder_freeze (sub);
 	}
-	if ((vf->flags & CAMEL_STORE_FOLDER_PRIVATE) == 0) {
+	if ((vf->flags & CAMEL_STORE_FOLDER_PRIVATE) == 0 && !CAMEL_IS_VEE_FOLDER(sub)) {
 		camel_object_ref((CamelObject *)sub);
 		up->folders = g_list_append(up->folders, sub);
+		/* update the freeze state of 'sub' to match Unmatched's freeze state */
+		for (i = 0; i < up->freeze_count; i++)
+			camel_folder_freeze (sub);
 	}
 
 	CAMEL_VEE_FOLDER_UNLOCK(vf, subfolder_lock);
@@ -374,7 +392,8 @@ camel_vee_folder_remove_folder(CamelVeeFolder *vf, CamelFolder *sub)
 {
 	struct _CamelVeeFolderPrivate *p = _PRIVATE(vf), *up = _PRIVATE(folder_unmatched);
 	int killun = FALSE;
-
+	int i;
+	
 	CAMEL_VEE_FOLDER_LOCK(vf, subfolder_lock);
 
 	CAMEL_VEE_FOLDER_LOCK(vf, changed_lock);
@@ -385,13 +404,17 @@ camel_vee_folder_remove_folder(CamelVeeFolder *vf, CamelFolder *sub)
 		CAMEL_VEE_FOLDER_UNLOCK(vf, subfolder_lock);
 		return;
 	}
-
+	
 	camel_object_unhook_event((CamelObject *)sub, "folder_changed", (CamelObjectEventHookFunc) folder_changed, vf);
 	camel_object_unhook_event((CamelObject *)sub, "message_changed", (CamelObjectEventHookFunc) message_changed, vf);
 	camel_object_unhook_event((CamelObject *)sub, "deleted", (CamelObjectEventHookFunc) subfolder_deleted, vf);
 
 	p->folders = g_list_remove(p->folders, sub);
-
+	
+	/* undo the freeze state that we have imposed on this source folder */
+	for (i = 0; i < p->freeze_count; i++)
+		camel_folder_thaw (sub);
+	
 	CAMEL_VEE_FOLDER_UNLOCK(vf, subfolder_lock);
 
 	CAMEL_VEE_FOLDER_LOCK(folder_unmatched, subfolder_lock);
@@ -401,11 +424,19 @@ camel_vee_folder_remove_folder(CamelVeeFolder *vf, CamelFolder *sub)
 			killun = TRUE;
 			up->folders = g_list_remove(up->folders, sub);
 			camel_object_unref((CamelObject *)sub);
+			
+			/* undo the freeze state that Unmatched has imposed on this source folder */
+			for (i = 0; i < p->freeze_count; i++)
+				camel_folder_thaw (sub);
 		}
 	} else if ((vf->flags & CAMEL_STORE_FOLDER_PRIVATE) == 0) {
 		if (g_list_find(up->folders, sub) != NULL) {
 			up->folders = g_list_remove(up->folders, sub);
 			camel_object_unref((CamelObject *)sub);
+			
+			/* undo the freeze state that Unmatched has imposed on this source folder */
+			for (i = 0; i < p->freeze_count; i++)
+				camel_folder_thaw (sub);
 		}
 		if (g_list_find(up->folders, sub) == NULL) {
 			killun = TRUE;
@@ -510,6 +541,38 @@ camel_vee_folder_hash_folder(CamelFolder *folder, char buffer[8])
 	}
 }
 
+/**
+ * camel_vee_folder_get_location:
+ * @vf: 
+ * @vinfo: 
+ * @realuid: if not NULL, set to the uid of the real message, must be
+ * g_free'd by caller.
+ * 
+ * Find the real folder (and uid)
+ * 
+ * Return value: 
+ **/
+CamelFolder *
+camel_vee_folder_get_location(CamelVeeFolder *vf, const CamelVeeMessageInfo *vinfo, char **realuid)
+{
+	/* locking?  yes?  no?  although the vfolderinfo is valid when obtained
+	   the folder in it might not necessarily be so ...? */
+	if (CAMEL_IS_VEE_FOLDER(vinfo->folder)) {
+		CamelFolder *folder;
+		const CamelVeeMessageInfo *vfinfo;
+
+		vfinfo = (CamelVeeMessageInfo *)camel_folder_get_message_info(vinfo->folder, camel_message_info_uid(vinfo)+8);
+		folder = camel_vee_folder_get_location((CamelVeeFolder *)vinfo->folder, vfinfo, realuid);
+		camel_folder_free_message_info(vinfo->folder, (CamelMessageInfo *)vfinfo);
+		return folder;
+	} else {
+		if (realuid)
+			*realuid = g_strdup(camel_message_info_uid(vinfo)+8);
+
+		return vinfo->folder;
+	}
+}
+
 static void vee_refresh_info(CamelFolder *folder, CamelException *ex)
 {
 	CamelVeeFolder *vf = (CamelVeeFolder *)folder;
@@ -548,8 +611,13 @@ vee_sync(CamelFolder *folder, gboolean expunge, CamelException *ex)
 		CamelFolder *f = node->data;
 
 		camel_folder_sync(f, expunge, ex);
-		if (camel_exception_is_set(ex))
+		if (camel_exception_is_set(ex)) {
+			char *desc;
+
+			camel_object_get(f, NULL, CAMEL_OBJECT_DESCRIPTION, &desc, NULL);
+			camel_exception_setv(ex, ex->id, _("Error storing `%s': %s"), desc, ex->desc);
 			break;
+		}
 
 		if (expunge && vee_folder_build_folder(vf, f, ex) == -1)
 			break;
@@ -570,7 +638,7 @@ vee_sync(CamelFolder *folder, gboolean expunge, CamelException *ex)
 static void
 vee_expunge (CamelFolder *folder, CamelException *ex)
 {
-	((CamelFolderClass *)((CamelObject *)folder)->classfuncs)->sync(folder, TRUE, ex);
+	((CamelFolderClass *)((CamelObject *)folder)->klass)->sync(folder, TRUE, ex);
 }
 
 static CamelMimeMessage *
@@ -601,10 +669,14 @@ vee_search_by_expression(CamelFolder *folder, const char *expression, CamelExcep
 	CamelVeeFolder *vf = (CamelVeeFolder *)folder;
 	struct _CamelVeeFolderPrivate *p = _PRIVATE(vf);
 	GHashTable *searched = g_hash_table_new(NULL, NULL);
-
+	
 	CAMEL_VEE_FOLDER_LOCK(vf, subfolder_lock);
-
-	expr = g_strdup_printf("(and %s %s)", vf->expression, expression);
+	
+	if (vf != folder_unmatched)
+		expr = g_strdup_printf ("(and %s %s)", vf->expression, expression);
+	else
+		expr = g_strdup (expression);
+	
 	node = p->folders;
 	while (node) {
 		CamelFolder *f = node->data;
@@ -726,31 +798,30 @@ vee_set_message_user_flag(CamelFolder *folder, const char *uid, const char *name
 }
 
 static void
-vee_move_messages_to (CamelFolder *folder, GPtrArray *uids, CamelFolder *dest, CamelException *ex)
+vee_set_message_user_tag(CamelFolder *folder, const char *uid, const char *name, const char *value)
 {
 	CamelVeeMessageInfo *mi;
-	int i;
-	
-	for (i = 0; i < uids->len && !camel_exception_is_set (ex); i++) {
-		mi = (CamelVeeMessageInfo *) camel_folder_summary_uid (folder->summary, uids->pdata[i]);
-		if (mi) {
-			/* noop if it we're moving from the same vfolder (uh, which should't happen but who knows) */
-			if (folder != mi->folder) {
-				GPtrArray *uids;
-				
-				uids = g_ptr_array_new ();
-				g_ptr_array_add (uids, (char *) (camel_message_info_uid (mi) + 8));
-				camel_folder_move_messages_to (mi->folder, uids, dest, ex);
-				g_ptr_array_free (uids, TRUE);
-			}
-			camel_folder_summary_info_free (folder->summary, (CamelMessageInfo *)mi);
-		} else {
-			camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID_UID,
-					      _("No such message: %s"), uids->pdata[i]);
-		}
+
+	mi = (CamelVeeMessageInfo *)camel_folder_summary_uid(folder->summary, uid);
+	if (mi) {
+		camel_folder_set_message_user_tag(mi->folder, camel_message_info_uid(mi) + 8, name, value);
+		camel_folder_summary_info_free(folder->summary, (CamelMessageInfo *)mi);
+		((CamelFolderClass *)camel_vee_folder_parent)->set_message_user_tag(folder, uid, name, value);
 	}
 }
 
+static void
+vee_append_message(CamelFolder *folder, CamelMimeMessage *message, const CamelMessageInfo *info, char **appended_uid, CamelException *ex)
+{
+	camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot copy or move messages into a Virtual Folder"));
+}
+
+static void
+vee_transfer_messages_to (CamelFolder *folder, GPtrArray *uids, CamelFolder *dest, GPtrArray **transferred_uids, gboolean delete_originals, CamelException *ex)
+{
+	camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot copy or move messages into a Virtual Folder"));
+}
+
 static void vee_rename(CamelFolder *folder, const char *new)
 {
 	CamelVeeFolder *vf = (CamelVeeFolder *)folder;
@@ -959,10 +1030,12 @@ folder_added_uid(char *uidin, void *value, struct _update_data *u)
 	if ( (mi = vee_folder_add_uid(u->vf, u->source, uidin, u->hash)) ) {
 		camel_folder_change_info_add_uid(u->vf->changes, camel_message_info_uid(mi));
 
-		if (g_hash_table_lookup_extended(unmatched_uids, camel_message_info_uid(mi), (void **)&oldkey, (void **)&n)) {
-			g_hash_table_insert(unmatched_uids, oldkey, (void *)(n+1));
-		} else {
-			g_hash_table_insert(unmatched_uids, g_strdup(camel_message_info_uid(mi)), (void *)1);
+		if (!CAMEL_IS_VEE_FOLDER(u->source)) {
+			if (g_hash_table_lookup_extended(unmatched_uids, camel_message_info_uid(mi), (void **)&oldkey, (void **)&n)) {
+				g_hash_table_insert(unmatched_uids, oldkey, (void *)(n+1));
+			} else {
+				g_hash_table_insert(unmatched_uids, g_strdup(camel_message_info_uid(mi)), (void *)1);
+			}
 		}
 	}
 }
@@ -1024,7 +1097,8 @@ vee_folder_build_folder(CamelVeeFolder *vf, CamelFolder *source, CamelException
 					camel_folder_summary_remove_index(folder->summary, i);
 					camel_folder_change_info_remove_uid(vf->changes, camel_message_info_uid(mi));
 					i--;
-					if (g_hash_table_lookup_extended(unmatched_uids, uid, (void **)&oldkey, (void **)&n)) {
+					if (!CAMEL_IS_VEE_FOLDER(source)
+					    && g_hash_table_lookup_extended(unmatched_uids, uid, (void **)&oldkey, (void **)&n)) {
 						if (n == 1) {
 							g_hash_table_remove(unmatched_uids, oldkey);
 							g_free(oldkey);
@@ -1066,7 +1140,8 @@ vee_folder_build_folder(CamelVeeFolder *vf, CamelFolder *source, CamelException
 	}
 
 	/* now allhash contains all potentially new uid's for the unmatched folder, process */
-	g_hash_table_foreach(allhash, (GHFunc)unmatched_check_uid, &u);
+	if (!CAMEL_IS_VEE_FOLDER(source))
+		g_hash_table_foreach(allhash, (GHFunc)unmatched_check_uid, &u);
 
 	/* copy any changes so we can raise them outside the lock */
 	if (camel_folder_change_info_changed(folder_unmatched->changes)) {
@@ -1127,7 +1202,7 @@ folder_changed_add_uid(CamelFolder *sub, const char *uid, const char hash[8], Ca
 	vuid = camel_message_info_uid(vinfo);
 	camel_folder_change_info_add_uid(vf->changes,  vuid);
 
-	if ((vf->flags & CAMEL_STORE_FOLDER_PRIVATE) == 0) {
+	if ((vf->flags & CAMEL_STORE_FOLDER_PRIVATE) == 0 && !CAMEL_IS_VEE_FOLDER(sub)) {
 		if (g_hash_table_lookup_extended(unmatched_uids, vuid, (void **)&oldkey, (void **)&n)) {
 			g_hash_table_insert(unmatched_uids, oldkey, (void *)(n+1));
 		} else {
@@ -1161,7 +1236,7 @@ folder_changed_remove_uid(CamelFolder *sub, const char *uid, const char hash[8],
 		camel_folder_summary_info_free(folder->summary, (CamelMessageInfo *)vinfo);
 	}
 
-	if ((vf->flags & CAMEL_STORE_FOLDER_PRIVATE) == 0) {
+	if ((vf->flags & CAMEL_STORE_FOLDER_PRIVATE) == 0 && !CAMEL_IS_VEE_FOLDER(sub)) {
 		if (keep) {
 			if (g_hash_table_lookup_extended(unmatched_uids, vuid, (void **)&oldkey, (void **)&n)) {
 				if (n == 1) {
@@ -1366,7 +1441,7 @@ folder_changed_change(CamelSession *session, CamelSessionThreadMsg *msg)
 				memcpy(vuid, hash, 8);
 				strcpy(vuid+8, uid);
 				
-				if (g_hash_table_lookup(unmatched_uids, vuid) == NULL) {
+				if (!CAMEL_IS_VEE_FOLDER(sub) && g_hash_table_lookup(unmatched_uids, vuid) == NULL) {
 					dd(printf("  adding uid '%s' to Unmatched [newly unmatched]\n", (char *)uid));
 					vinfo = (CamelVeeMessageInfo *)camel_folder_get_message_info((CamelFolder *)folder_unmatched, vuid);
 					if (vinfo == NULL) {
@@ -1538,3 +1613,54 @@ subfolder_deleted(CamelFolder *f, void *event_data, CamelVeeFolder *vf)
 {
 	camel_vee_folder_remove_folder(vf, f);
 }
+
+
+static void
+vee_freeze (CamelFolder *folder)
+{
+	CamelVeeFolder *vfolder = (CamelVeeFolder *) folder;
+	struct _CamelVeeFolderPrivate *p = _PRIVATE (vfolder);
+	GList *node;
+	
+	CAMEL_VEE_FOLDER_LOCK (vfolder, subfolder_lock);
+	
+	p->freeze_count++;
+	
+	node = p->folders;
+	while (node) {
+		CamelFolder *f = node->data;
+		
+		camel_folder_freeze (f);
+		node = node->next;
+	}
+	
+	CAMEL_VEE_FOLDER_UNLOCK (vfolder, subfolder_lock);
+	
+	/* call parent implementation */
+	CAMEL_FOLDER_CLASS (camel_vee_folder_parent)->freeze (folder);
+}
+
+static void
+vee_thaw (CamelFolder *folder)
+{
+	CamelVeeFolder *vfolder = (CamelVeeFolder *) folder;
+	struct _CamelVeeFolderPrivate *p = _PRIVATE (vfolder);
+	GList *node;
+	
+	CAMEL_VEE_FOLDER_LOCK (vfolder, subfolder_lock);
+	
+	p->freeze_count--;
+	
+	node = p->folders;
+	while (node) {
+		CamelFolder *f = node->data;
+		
+		camel_folder_thaw (f);
+		node = node->next;
+	}
+	
+	CAMEL_VEE_FOLDER_UNLOCK (vfolder, subfolder_lock);
+	
+	/* call parent implementation */
+	CAMEL_FOLDER_CLASS (camel_vee_folder_parent)->thaw (folder);
+}
