@@ -1,26 +1,27 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
-/* e-msg-composer-attachment-bar.c
+/*
+ *  Authors: Ettore Perazzoli <ettore@ximian.com>
+ *           Jeffrey Stedfast <fejj@ximian.com>
  *
- * Copyright (C) 1999  Ximian, Inc.
+ *  Copyright 1999-2002 Ximian, Inc. (www.ximian.com)
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
- * published by the Free Software Foundation; either version 2 of the
- * License as published by the Free Software Foundation.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
- * Author: Ettore Perazzoli
  */
 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -220,7 +221,23 @@ pixbuf_for_mime_type (const char *mime_type)
 	const char *icon_name;
 	char *filename = NULL;
 	GdkPixbuf *pixbuf;
-	
+
+	/* Special-case these two since GNOME VFS doesn't know about them and
+	   they are used every time the user forwards one or more messages
+	   inline.  (See #9786.)  */
+	if (strcmp (mime_type, "message/digest") == 0
+	    || strcmp (mime_type, "multipart/digest") == 0
+	    || strcmp (mime_type, "message/rfc822") == 0) {
+		char *name;
+
+		name = g_concat_dir_and_file (EVOLUTION_ICONSDIR, "mail.png");
+		pixbuf = gdk_pixbuf_new_from_file (name);
+		g_free (name);
+
+		if (pixbuf != NULL)
+			return pixbuf;
+	}
+
 	icon_name = gnome_vfs_mime_get_value (mime_type, "icon-filename");
 	if (icon_name) {
 		if (*icon_name == '/') {
@@ -444,18 +461,22 @@ static void
 add_from_user (EMsgComposerAttachmentBar *bar)
 {
 	EMsgComposer *composer;
-	char *file_name;
+	GPtrArray *file_list;
 	gboolean is_inline = FALSE;
+	int i;
 	
 	composer = E_MSG_COMPOSER (gtk_widget_get_toplevel (GTK_WIDGET (bar)));
 	
-	file_name = e_msg_composer_select_file_attachment (composer, &is_inline);
-	if (!file_name)
+	file_list = e_msg_composer_select_file_attachments (composer, &is_inline);
+	if (!file_list)
 		return;
 	
-	add_from_file (bar, file_name, is_inline ? "inline" : "attachment");
+	for (i = 0; i < file_list->len; i++) {
+		add_from_file (bar, file_list->pdata[i], is_inline ? "inline" : "attachment");
+		g_free (file_list->pdata[i]);
+	}
 	
-	g_free (file_name);
+	g_ptr_array_free (file_list, TRUE);
 }
 
 
@@ -570,6 +591,8 @@ destroy (GtkObject *object)
 	
 	free_attachment_list (bar);
 	
+	g_free (bar->priv);
+	
 	if (GTK_OBJECT_CLASS (parent_class)->destroy != NULL)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
