@@ -21,6 +21,7 @@
  * Author: Ettore Perazzoli
  */
 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -44,10 +45,13 @@
 #include <gal/widgets/e-unicode.h>
 
 #include <camel/camel.h>
+#include "evolution-folder-selector-button.h"
 #include "e-msg-composer-hdrs.h"
 #include "mail/mail-config.h"
 #include "addressbook/backend/ebook/e-book-util.h"
 
+extern EvolutionShellClient *global_shell_client;
+
 
 
 /* Indexes in the GtkTable assigned to various items */
@@ -57,7 +61,9 @@
 #define LINE_TO      2
 #define LINE_CC      3
 #define LINE_BCC     4
-#define LINE_SUBJECT 5
+#define LINE_POSTTO  5
+#define LINE_SUBJECT 6
+
 
 typedef struct {
 	GtkWidget *label;
@@ -66,14 +72,17 @@ typedef struct {
 
 struct _EMsgComposerHdrsPrivate {
 	GNOME_Evolution_Addressbook_SelectNames corba_select_names;
-
+	
+	/* ui component */
+	BonoboUIComponent *uic;
+	
 	/* The tooltips.  */
 	GtkTooltips *tooltips;
 	
 	GSList *from_options;
 	
 	/* Standard headers.  */
-	EMsgComposerHdrPair from, reply_to, to, cc, bcc, subject;
+	EMsgComposerHdrPair from, reply_to, to, cc, bcc, post_to, subject;
 };
 
 
@@ -87,7 +96,7 @@ enum {
 	LAST_SIGNAL
 };
 
-static gint signals[LAST_SIGNAL];
+static int signals[LAST_SIGNAL];
 
 
 static gboolean
@@ -95,24 +104,24 @@ setup_corba (EMsgComposerHdrs *hdrs)
 {
 	EMsgComposerHdrsPrivate *priv;
 	CORBA_Environment ev;
-
+	
 	priv = hdrs->priv;
-
+	
 	g_assert (priv->corba_select_names == CORBA_OBJECT_NIL);
-
+	
 	CORBA_exception_init (&ev);
-
+	
 	priv->corba_select_names = oaf_activate_from_id (SELECT_NAMES_OAFIID, 0, NULL, &ev);
-
+	
 	/* OAF seems to be broken -- it can return a CORBA_OBJECT_NIL without
            raising an exception in `ev'.  */
 	if (ev._major != CORBA_NO_EXCEPTION || priv->corba_select_names == CORBA_OBJECT_NIL) {
 		CORBA_exception_free (&ev);
 		return FALSE;
 	}
-
+	
 	CORBA_exception_free (&ev);
-
+	
 	return TRUE;
 }
 
@@ -139,13 +148,12 @@ e_msg_composer_hdrs_and_string_create (EMsgComposerHdrs *hdrs, const char *strin
 	emchas->string = g_strdup (string);
 	if (emchas->hdrs)
 		gtk_object_ref (GTK_OBJECT (emchas->hdrs));
-
+	
 	return emchas;
 }
 
 static void
-address_button_clicked_cb (GtkButton *button,
-			   gpointer data)
+address_button_clicked_cb (GtkButton *button, gpointer data)
 {
 	EMsgComposerHdrsAndString *emchas;
 	EMsgComposerHdrs *hdrs;
@@ -168,8 +176,15 @@ static void
 from_changed (GtkWidget *item, gpointer data)
 {
 	EMsgComposerHdrs *hdrs = E_MSG_COMPOSER_HDRS (data);
+	const char *reply_to;
 	
 	hdrs->account = gtk_object_get_data (GTK_OBJECT (item), "account");
+	
+	/* we do this rather than calling e_msg_composer_hdrs_set_reply_to()
+	   because we don't want to change the visibility of the header */
+	reply_to = hdrs->account->id->reply_to;
+	e_entry_set_text (E_ENTRY (hdrs->priv->reply_to.entry), reply_to ? reply_to : "");
+	
 	gtk_signal_emit (GTK_OBJECT (hdrs), signals [FROM_CHANGED]);
 }
 
@@ -178,7 +193,7 @@ create_from_optionmenu (EMsgComposerHdrs *hdrs)
 {
 	GtkWidget *omenu, *menu, *first = NULL;
 	const GSList *accounts;
-	GtkWidget *item;
+	GtkWidget *item, *hbox;
 	int i = 0, history = 0;
 	int default_account;
 	
@@ -202,7 +217,6 @@ create_from_optionmenu (EMsgComposerHdrs *hdrs)
 		}
 		
 		if (account->id->address && *account->id->address) {
-
 			if (strcmp (account->name, account->id->address))
 				label = g_strdup_printf ("%s <%s> (%s)", account->id->name,
 							 account->id->address, account->name);
@@ -228,7 +242,7 @@ create_from_optionmenu (EMsgComposerHdrs *hdrs)
 			
 			gtk_menu_append (GTK_MENU (menu), item);
 			gtk_widget_show (item);
-			++i;
+			i++;
 		}
 		
 		accounts = accounts->next;
@@ -241,7 +255,14 @@ create_from_optionmenu (EMsgComposerHdrs *hdrs)
 		gtk_signal_emit_by_name (GTK_OBJECT (first), "activate", hdrs);
 	}
 	
-	return omenu;
+	hbox = gtk_hbox_new (FALSE, 3);
+	gtk_box_pack_start_defaults (GTK_BOX (hbox), omenu);
+	gtk_widget_show (omenu);
+	gtk_widget_show (hbox);
+	
+	gtk_object_set_data (GTK_OBJECT (hbox), "from_menu", omenu);
+	
+	return hbox;
 }
 
 static void
@@ -252,13 +273,12 @@ addressbook_entry_changed (BonoboListener    *listener,
 			   gpointer           user_data)
 {
 	EMsgComposerHdrs *hdrs = E_MSG_COMPOSER_HDRS (user_data);
-
+	
 	gtk_signal_emit (GTK_OBJECT (hdrs), signals[HDRS_CHANGED]);
 }
 
 static GtkWidget *
-create_addressbook_entry (EMsgComposerHdrs *hdrs,
-			  const char *name)
+create_addressbook_entry (EMsgComposerHdrs *hdrs, const char *name)
 {
 	EMsgComposerHdrsPrivate *priv;
 	GNOME_Evolution_Addressbook_SelectNames corba_select_names;
@@ -267,46 +287,46 @@ create_addressbook_entry (EMsgComposerHdrs *hdrs,
 	CORBA_Environment ev;
 	BonoboControlFrame *cf;
 	Bonobo_PropertyBag pb = CORBA_OBJECT_NIL;
-
+	
 	priv = hdrs->priv;
 	corba_select_names = priv->corba_select_names;
-
+	
 	CORBA_exception_init (&ev);
-
+	
 	GNOME_Evolution_Addressbook_SelectNames_addSection (
 		corba_select_names, name, name, &ev);
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		CORBA_exception_free (&ev);
 		return NULL;
 	}
-
+	
 	corba_control =
 		GNOME_Evolution_Addressbook_SelectNames_getEntryBySection (
 			corba_select_names, name, &ev);
-
+	
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		CORBA_exception_free (&ev);
 		return NULL;
 	}
-
+	
 	CORBA_exception_free (&ev);
-
+	
 	control_widget = bonobo_widget_new_control_from_objref (
 		corba_control, CORBA_OBJECT_NIL);
-
+	
 	cf = bonobo_widget_get_control_frame (BONOBO_WIDGET (control_widget));
 	pb = bonobo_control_frame_get_control_property_bag (cf, NULL);
-
+	
 	bonobo_event_source_client_add_listener (
 		pb, addressbook_entry_changed,
 		"Bonobo/Property:change:entry_changed",
 		NULL, hdrs);
-
+	
 	return control_widget;
 }
 
 static EMsgComposerHdrPair 
-header_new_recipient (EMsgComposerHdrs *hdrs, const gchar *name, const gchar *tip)
+header_new_recipient (EMsgComposerHdrs *hdrs, const char *name, const char *tip)
 {
 	EMsgComposerHdrsPrivate *priv;
 	EMsgComposerHdrPair ret;
@@ -321,29 +341,28 @@ header_new_recipient (EMsgComposerHdrs *hdrs, const gchar *name, const gchar *ti
 		e_msg_composer_hdrs_and_string_create(hdrs, name),
 		(GtkDestroyNotify) e_msg_composer_hdrs_and_string_free,
 		FALSE, FALSE);
-
+	
 	gtk_tooltips_set_tip (
 		hdrs->priv->tooltips, ret.label,
 		_("Click here for the address book"),
 		NULL);
-
+	
 	ret.entry = create_addressbook_entry (hdrs, name);
-
+	
 	return ret;
 }
 
 static void
 entry_changed (GtkWidget *entry, EMsgComposerHdrs *hdrs)
 {
-	gchar *tmp;
-	gchar *subject;
-
+	char *subject, *tmp;
+	
 	tmp = e_msg_composer_hdrs_get_subject (hdrs);
 	subject = e_utf8_to_gtk_string (GTK_WIDGET (entry), tmp);
-
+	
 	gtk_signal_emit (GTK_OBJECT (hdrs), signals[SUBJECT_CHANGED], subject);
 	g_free (tmp);
-
+	
 	gtk_signal_emit (GTK_OBJECT (hdrs), signals[HDRS_CHANGED]);
 }
 
@@ -351,15 +370,14 @@ static void
 create_headers (EMsgComposerHdrs *hdrs)
 {
 	EMsgComposerHdrsPrivate *priv = hdrs->priv;
-
-	/*
-	 * From:
-	 */
-	priv->from.label = gtk_label_new (_("From:"));
-	priv->from.entry = create_from_optionmenu (hdrs);
-
+	static const char *posting_types[] = { "mail", NULL };
+	
 	/*
 	 * Reply-To:
+	 *
+	 * Create this before we call create_from_optionmenu,
+	 * because that causes from_changed to be called, which
+	 * expects the reply_to fields to be initialized.
 	 */
 	priv->reply_to.label = gtk_label_new (_("Reply-To:"));
 	priv->reply_to.entry = e_entry_new ();
@@ -368,9 +386,15 @@ create_headers (EMsgComposerHdrs *hdrs)
 			"use_ellipsis", TRUE,
 			"allow_newlines", FALSE,
 			NULL);
-
+	
 	/*
-	 * Subject:
+	 * From
+	 */
+	priv->from.label = gtk_label_new (_("From:"));
+	priv->from.entry = create_from_optionmenu (hdrs);
+	
+	/*
+	 * Subject
 	 */
 	priv->subject.label = gtk_label_new (_("Subject:"));
 	priv->subject.entry = e_entry_new ();
@@ -381,30 +405,31 @@ create_headers (EMsgComposerHdrs *hdrs)
 			NULL);
 	gtk_signal_connect (GTK_OBJECT (priv->subject.entry), "changed",
 			    GTK_SIGNAL_FUNC (entry_changed), hdrs);
-
+	
 	/*
-	 * To: CC: and Bcc:
+	 * To, CC, and Bcc
 	 */
 	priv->to = header_new_recipient (
 		hdrs, _("To:"),
 		_("Enter the recipients of the message"));
-
+	
 	priv->cc = header_new_recipient (
 		hdrs, _("Cc:"),
 		_("Enter the addresses that will receive a carbon copy of the message"));
-
+	
 	priv->bcc = header_new_recipient (
 		hdrs, _("Bcc:"),
 		 _("Enter the addresses that will receive a carbon copy of "
 		   "the message without appearing in the recipient list of "
 		   "the message."));
-}
-
-static GtkDirectionType
-focus_cb (GtkContainer *contain, GtkDirectionType dir, gpointer closure)
-{
-	g_message ("FOCUS: %d", dir);
-	return dir;
+	
+	/*
+	 * Post-To
+	 */
+	priv->post_to.label = gtk_label_new (_("Post To:"));
+	priv->post_to.entry = evolution_folder_selector_button_new (
+		global_shell_client, _("Posting destination"), NULL,
+		posting_types);
 }
 
 static void
@@ -416,7 +441,7 @@ attach_couple (EMsgComposerHdrs *hdrs, EMsgComposerHdrPair *pair, int line)
 		pad = GNOME_PAD;
 	else
 		pad = 2;
-
+	
 	gtk_table_attach (GTK_TABLE (hdrs),
 			  pair->label, 0, 1,
 			  line, line + 1,
@@ -432,19 +457,20 @@ static void
 attach_headers (EMsgComposerHdrs *hdrs)
 {
 	EMsgComposerHdrsPrivate *p = hdrs->priv;
-
+	
 	attach_couple (hdrs, &p->from, LINE_FROM);
 	attach_couple (hdrs, &p->reply_to, LINE_REPLYTO);
 	attach_couple (hdrs, &p->to, LINE_TO);
 	attach_couple (hdrs, &p->cc, LINE_CC);
 	attach_couple (hdrs, &p->bcc, LINE_BCC);
+	attach_couple (hdrs, &p->post_to, LINE_POSTTO);
 	attach_couple (hdrs, &p->subject, LINE_SUBJECT);
 }
 
 static void
-set_pair_visibility (EMsgComposerHdrs *h, EMsgComposerHdrPair *pair, gboolean visible)
+set_pair_visibility (EMsgComposerHdrs *h, EMsgComposerHdrPair *pair, int visible)
 {
-	if (visible){
+	if (visible & h->visible_mask) {
 		gtk_widget_show (pair->label);
 		gtk_widget_show (pair->entry);
 	} else {
@@ -454,42 +480,73 @@ set_pair_visibility (EMsgComposerHdrs *h, EMsgComposerHdrPair *pair, gboolean vi
 }
 
 static void
-headers_set_visibility (EMsgComposerHdrs *h, gint visible_flags)
+headers_set_visibility (EMsgComposerHdrs *h, int visible_flags)
 {
 	EMsgComposerHdrsPrivate *p = h->priv;
-
+	
+	/* To is always visible if we're not doing Post-To */
+	if (!(h->visible_mask & E_MSG_COMPOSER_VISIBLE_POSTTO))
+		visible_flags |= E_MSG_COMPOSER_VISIBLE_TO;
+	else
+		visible_flags |= E_MSG_COMPOSER_VISIBLE_POSTTO;
+	
 	set_pair_visibility (h, &p->from, visible_flags & E_MSG_COMPOSER_VISIBLE_FROM);
 	set_pair_visibility (h, &p->reply_to, visible_flags & E_MSG_COMPOSER_VISIBLE_REPLYTO);
+	set_pair_visibility (h, &p->to, visible_flags & E_MSG_COMPOSER_VISIBLE_TO);
 	set_pair_visibility (h, &p->cc, visible_flags & E_MSG_COMPOSER_VISIBLE_CC);
 	set_pair_visibility (h, &p->bcc, visible_flags & E_MSG_COMPOSER_VISIBLE_BCC);
+	set_pair_visibility (h, &p->post_to, visible_flags & E_MSG_COMPOSER_VISIBLE_POSTTO);
 	set_pair_visibility (h, &p->subject, visible_flags & E_MSG_COMPOSER_VISIBLE_SUBJECT);
 }
 
-void
-e_msg_composer_set_hdrs_visible (EMsgComposerHdrs *hdrs, gint visible_flags)
+static void
+headers_set_sensitivity (EMsgComposerHdrs *h)
 {
-	g_return_if_fail (hdrs != NULL);
-	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
+	bonobo_ui_component_set_prop (
+		h->priv->uic, "/commands/ViewFrom", "sensitive",
+		h->visible_mask & E_MSG_COMPOSER_VISIBLE_FROM ? "1" : "0", NULL);
+	
+	bonobo_ui_component_set_prop (
+		h->priv->uic, "/commands/ViewReplyTo", "sensitive",
+		h->visible_mask & E_MSG_COMPOSER_VISIBLE_REPLYTO ? "1" : "0", NULL);
+	
+	bonobo_ui_component_set_prop (
+		h->priv->uic, "/commands/ViewCC", "sensitive",
+		h->visible_mask & E_MSG_COMPOSER_VISIBLE_CC ? "1" : "0", NULL);
+	
+	bonobo_ui_component_set_prop (
+		h->priv->uic, "/commands/ViewBCC", "sensitive",
+		h->visible_mask & E_MSG_COMPOSER_VISIBLE_BCC ? "1" : "0", NULL);
+}
 
+void
+e_msg_composer_hdrs_set_visible_mask (EMsgComposerHdrs *hdrs, int visible_mask)
+{
+	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
+	
+	hdrs->visible_mask = visible_mask;
+	headers_set_sensitivity (hdrs);
+}
+
+void
+e_msg_composer_hdrs_set_visible (EMsgComposerHdrs *hdrs, int visible_flags)
+{
+	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
+	
 	headers_set_visibility (hdrs, visible_flags);
 	gtk_widget_queue_resize (GTK_WIDGET (hdrs));
 }
 
 static void
-setup_headers (EMsgComposerHdrs *hdrs, gint visible_flags)
+setup_headers (EMsgComposerHdrs *hdrs, int visible_flags)
 {
 	create_headers (hdrs);
 	attach_headers (hdrs);
-
-	/*
-	 * To: is always visible
-	 */
-	gtk_widget_show (hdrs->priv->to.label);
-	gtk_widget_show (hdrs->priv->to.entry);
 	
+	headers_set_sensitivity (hdrs);
 	headers_set_visibility (hdrs, visible_flags);
 }
-		
+
 
 /* GtkObject methods.  */
 
@@ -499,18 +556,19 @@ destroy (GtkObject *object)
 	EMsgComposerHdrs *hdrs;
 	EMsgComposerHdrsPrivate *priv;
 	GSList *l;
-
+	
 	hdrs = E_MSG_COMPOSER_HDRS (object);
 	priv = hdrs->priv;
-
+	
 	if (priv->corba_select_names != CORBA_OBJECT_NIL) {
 		CORBA_Environment ev;
 		CORBA_exception_init (&ev);
 		bonobo_object_release_unref (priv->corba_select_names, &ev);
 		CORBA_exception_free (&ev);
 	}
-
+	
 	gtk_object_destroy (GTK_OBJECT (priv->tooltips));
+	gtk_object_unref (GTK_OBJECT (priv->tooltips));
 	
 	l = priv->from_options;
 	while (l) {
@@ -535,12 +593,12 @@ static void
 class_init (EMsgComposerHdrsClass *class)
 {
 	GtkObjectClass *object_class;
-
+	
 	object_class = GTK_OBJECT_CLASS (class);
 	object_class->destroy = destroy;
-
+	
 	parent_class = gtk_type_class (gtk_table_get_type ());
-
+	
 	signals[SHOW_ADDRESS_DIALOG] =
 		gtk_signal_new ("show_address_dialog",
 				GTK_RUN_LAST,
@@ -549,7 +607,7 @@ class_init (EMsgComposerHdrsClass *class)
 						   show_address_dialog),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
-
+	
 	signals[SUBJECT_CHANGED] =
 		gtk_signal_new ("subject_changed",
 				GTK_RUN_LAST,
@@ -559,7 +617,7 @@ class_init (EMsgComposerHdrsClass *class)
 				gtk_marshal_NONE__STRING,
 				GTK_TYPE_NONE,
 				1, GTK_TYPE_STRING);
-
+	
 	signals[HDRS_CHANGED] =
 		gtk_signal_new ("hdrs_changed",
 				GTK_RUN_LAST,
@@ -568,7 +626,7 @@ class_init (EMsgComposerHdrsClass *class)
 						   hdrs_changed),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
-
+	
 	signals[FROM_CHANGED] =
 		gtk_signal_new ("from_changed",
 				GTK_RUN_LAST,
@@ -577,7 +635,7 @@ class_init (EMsgComposerHdrsClass *class)
 						   from_changed),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
-
+	
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 }
 
@@ -585,11 +643,11 @@ static void
 init (EMsgComposerHdrs *hdrs)
 {
 	EMsgComposerHdrsPrivate *priv;
-
+	
 	priv = g_new0 (EMsgComposerHdrsPrivate, 1);
-
+	
 	priv->tooltips = gtk_tooltips_new ();
-
+	
 	hdrs->priv = priv;
 }
 
@@ -598,7 +656,7 @@ GtkType
 e_msg_composer_hdrs_get_type (void)
 {
 	static GtkType type = 0;
-
+	
 	if (type == 0) {
 		static const GtkTypeInfo info = {
 			"EMsgComposerHdrs",
@@ -610,29 +668,32 @@ e_msg_composer_hdrs_get_type (void)
 			/* reserved_2 */ NULL,
 			(GtkClassInitFunc) NULL,
 		};
-
+		
 		type = gtk_type_unique (gtk_table_get_type (), &info);
 	}
-
+	
 	return type;
 }
 
 GtkWidget *
-e_msg_composer_hdrs_new (gint visible_flags)
+e_msg_composer_hdrs_new (BonoboUIComponent *uic, int visible_mask, int visible_flags)
 {
 	EMsgComposerHdrs *new;
 	EMsgComposerHdrsPrivate *priv;
 	
 	new = gtk_type_new (e_msg_composer_hdrs_get_type ());
 	priv = new->priv;
+	priv->uic = uic;
 	
-	if (! setup_corba (new)) {
+	if (!setup_corba (new)) {
 		gtk_widget_destroy (GTK_WIDGET (new));
 		return NULL;
 	}
 	
+	new->visible_mask = visible_mask;
+	
 	setup_headers (new, visible_flags);
-
+	
 	return GTK_WIDGET (new);
 }
 
@@ -640,27 +701,26 @@ static void
 set_recipients_from_destv (CamelMimeMessage *msg,
 			   EDestination **to_destv,
 			   EDestination **cc_destv,
-			   EDestination **bcc_destv)
+			   EDestination **bcc_destv,
+			   gboolean redirect)
 {
 	CamelInternetAddress *to_addr;
 	CamelInternetAddress *cc_addr;
 	CamelInternetAddress *bcc_addr;
 	CamelInternetAddress *target;
-	const gchar *text_addr;
-	gint i;
+	const char *text_addr, *header;
 	gboolean seen_hidden_list = FALSE;
-
+	int i;
+	
 	to_addr  = camel_internet_address_new ();
 	cc_addr  = camel_internet_address_new ();
 	bcc_addr = camel_internet_address_new ();
-
+	
 	if (to_destv) {
 		for (i = 0; to_destv[i] != NULL; ++i) {
 			text_addr = e_destination_get_address (to_destv[i]);
-
-
+			
 			if (text_addr && *text_addr) {
-
 				target = to_addr;
 				if (e_destination_is_evolution_list (to_destv[i])
 				    && !e_destination_list_show_addresses (to_destv[i])) {
@@ -672,12 +732,11 @@ set_recipients_from_destv (CamelMimeMessage *msg,
 			}
 		}
 	}
-
+	
 	if (cc_destv) {
 		for (i = 0; cc_destv[i] != NULL; ++i) {
 			text_addr = e_destination_get_address (cc_destv[i]);
 			if (text_addr && *text_addr) {
-
 				target = cc_addr;
 				if (e_destination_is_evolution_list (cc_destv[i])
 				    && !e_destination_list_show_addresses (cc_destv[i])) {
@@ -689,46 +748,48 @@ set_recipients_from_destv (CamelMimeMessage *msg,
 			}
 		}
 	}
-
+	
 	if (bcc_destv) {
 		for (i = 0; bcc_destv[i] != NULL; ++i) {
 			text_addr = e_destination_get_address (bcc_destv[i]);
-			if (text_addr && *text_addr) {
+			if (text_addr && *text_addr) {				
 				camel_address_decode (CAMEL_ADDRESS (bcc_addr), text_addr);
 			}
 		}
 	}
-
+	
+	header = redirect ? CAMEL_RECIPIENT_TYPE_RESENT_TO : CAMEL_RECIPIENT_TYPE_TO;
 	if (camel_address_length (CAMEL_ADDRESS (to_addr)) > 0) {
-		camel_mime_message_set_recipients (msg, CAMEL_RECIPIENT_TYPE_TO, to_addr);
+		camel_mime_message_set_recipients (msg, header, to_addr);
 	} else if (seen_hidden_list) {
-		camel_medium_set_header (CAMEL_MEDIUM (msg), CAMEL_RECIPIENT_TYPE_TO, "Undisclosed-Recipient:;");
+		camel_medium_set_header (CAMEL_MEDIUM (msg), header, "Undisclosed-Recipient:;");
 	}
-
+	
+	header = redirect ? CAMEL_RECIPIENT_TYPE_RESENT_CC : CAMEL_RECIPIENT_TYPE_CC;
 	if (camel_address_length (CAMEL_ADDRESS (cc_addr)) > 0) {
-		camel_mime_message_set_recipients (msg, CAMEL_RECIPIENT_TYPE_CC, cc_addr);
+		camel_mime_message_set_recipients (msg, header, cc_addr);
 	}
-
+	
+	header = redirect ? CAMEL_RECIPIENT_TYPE_RESENT_BCC : CAMEL_RECIPIENT_TYPE_BCC;
 	if (camel_address_length (CAMEL_ADDRESS (bcc_addr)) > 0) {
-		camel_mime_message_set_recipients (msg, CAMEL_RECIPIENT_TYPE_BCC, bcc_addr);
+		camel_mime_message_set_recipients (msg, header, bcc_addr);
 	}
-
+	
 	camel_object_unref (CAMEL_OBJECT (to_addr));
 	camel_object_unref (CAMEL_OBJECT (cc_addr));
 	camel_object_unref (CAMEL_OBJECT (bcc_addr));
 }
 
-void
-e_msg_composer_hdrs_to_message (EMsgComposerHdrs *hdrs,
-				CamelMimeMessage *msg)
+static void
+e_msg_composer_hdrs_to_message_internal (EMsgComposerHdrs *hdrs,
+					 CamelMimeMessage *msg,
+					 gboolean redirect)
 {
-	CamelInternetAddress *addr;
-	gchar *subject;
 	EDestination **to_destv, **cc_destv, **bcc_destv;
+	CamelInternetAddress *addr;
+	char *subject, *header;
 	
-	g_return_if_fail (hdrs != NULL);
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
-	g_return_if_fail (msg != NULL);
 	g_return_if_fail (CAMEL_IS_MIME_MESSAGE (msg));
 	
 	subject = e_msg_composer_hdrs_get_subject (hdrs);
@@ -736,7 +797,13 @@ e_msg_composer_hdrs_to_message (EMsgComposerHdrs *hdrs,
 	g_free (subject);
 	
 	addr = e_msg_composer_hdrs_get_from (hdrs);
-	camel_mime_message_set_from (msg, addr);
+	if (redirect) {
+		header = camel_address_encode (CAMEL_ADDRESS (addr));
+		camel_medium_set_header (CAMEL_MEDIUM (msg), "Resent-From", header);
+		g_free (header);
+	} else {
+		camel_mime_message_set_from (msg, addr);
+	}
 	camel_object_unref (CAMEL_OBJECT (addr));
 	
  	addr = e_msg_composer_hdrs_get_reply_to (hdrs);
@@ -744,21 +811,44 @@ e_msg_composer_hdrs_to_message (EMsgComposerHdrs *hdrs,
 		camel_mime_message_set_reply_to (msg, addr);
 		camel_object_unref (CAMEL_OBJECT (addr));
 	}
+	
+	if (hdrs->visible_mask & E_MSG_COMPOSER_VISIBLE_MASK_RECIPIENTS) {
+		to_destv  = e_msg_composer_hdrs_get_to (hdrs);
+		cc_destv  = e_msg_composer_hdrs_get_cc (hdrs);
+		bcc_destv = e_msg_composer_hdrs_get_bcc (hdrs);
+		
+		/* Attach destinations to the message. */
+		
+		set_recipients_from_destv (msg, to_destv, cc_destv, bcc_destv, redirect);
+		
+		e_destination_freev (to_destv);
+		e_destination_freev (cc_destv);
+		e_destination_freev (bcc_destv);
+	}
+	
+	if (hdrs->visible_mask & E_MSG_COMPOSER_VISIBLE_POSTTO) {
+		header = e_msg_composer_hdrs_get_post_to (hdrs);
+		camel_medium_set_header (CAMEL_MEDIUM (msg), "X-Evolution-PostTo", header);
+		g_free (header);
+	}
+}
 
-	to_destv  = e_msg_composer_hdrs_get_to (hdrs);
-	cc_destv  = e_msg_composer_hdrs_get_cc (hdrs);
-	bcc_destv = e_msg_composer_hdrs_get_bcc (hdrs);
 
-	/* Attach destinations to the message. */
+void
+e_msg_composer_hdrs_to_message (EMsgComposerHdrs *hdrs,
+				CamelMimeMessage *msg)
+{
+	e_msg_composer_hdrs_to_message_internal (hdrs, msg, FALSE);
+}
 
-	set_recipients_from_destv (msg, to_destv, cc_destv, bcc_destv);
 
-	e_destination_freev (to_destv);
-	e_destination_freev (cc_destv);
-	e_destination_freev (bcc_destv);
+void
+e_msg_composer_hdrs_to_redirect (EMsgComposerHdrs *hdrs,
+				 CamelMimeMessage *msg)
+{
+	e_msg_composer_hdrs_to_message_internal (hdrs, msg, TRUE);
 }
 
-
 
 /* FIXME: yea, this could be better... but it's doubtful it'll be used much */
 void
@@ -771,10 +861,9 @@ e_msg_composer_hdrs_set_from_account (EMsgComposerHdrs *hdrs,
 	int i = 0;
 	int default_account = 0;
 	
-	g_return_if_fail (hdrs != NULL);
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
 	
-	omenu = GTK_OPTION_MENU (hdrs->priv->from.entry);
+	omenu = GTK_OPTION_MENU (e_msg_composer_hdrs_get_from_omenu (hdrs));
 	
 	if (account_name)
 		default_account = -1;
@@ -788,10 +877,16 @@ e_msg_composer_hdrs_set_from_account (EMsgComposerHdrs *hdrs,
 		item = l->data;
 		
 		account = gtk_object_get_data (GTK_OBJECT (item), "account");
-		if (i == default_account ||
-		    (account_name && ((account->name && !strcmp (account_name, account->name))
-				      || (account->id->address && strstr (account_name, account->id->address))))) {
-			/* set the correct optionlist item */
+		if (account_name) {
+			if (account->name && !strcmp (account_name, account->name)) {
+				/* set the correct optionlist item */
+				gtk_option_menu_set_history (omenu, i);
+				gtk_signal_emit_by_name (GTK_OBJECT (item), "activate", hdrs);
+				
+				return;
+			}
+		} else if (i == default_account) {
+			/* set the default optionlist item */
 			gtk_option_menu_set_history (omenu, i);
 			gtk_signal_emit_by_name (GTK_OBJECT (item), "activate", hdrs);
 			
@@ -807,20 +902,22 @@ void
 e_msg_composer_hdrs_set_reply_to (EMsgComposerHdrs *hdrs,
 				  const char *reply_to)
 {
-	g_return_if_fail (hdrs != NULL);
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
 	
-	bonobo_widget_set_property (BONOBO_WIDGET (hdrs->priv->reply_to.entry),
-				    "text", reply_to, NULL);
+	e_entry_set_text (E_ENTRY (hdrs->priv->reply_to.entry), reply_to ? reply_to : "");
+	
+	if (reply_to && *reply_to)
+		set_pair_visibility (hdrs, &hdrs->priv->cc, TRUE);
 }
 
 void
 e_msg_composer_hdrs_set_to (EMsgComposerHdrs *hdrs,
 			    EDestination **to_destv)
 {
-	gchar *str;
+	char *str;
+	
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
-
+	
 	str = e_destination_exportv (to_destv);
 	bonobo_widget_set_property (BONOBO_WIDGET (hdrs->priv->to.entry), "destinations", str, NULL); 
 	g_free (str);
@@ -830,11 +927,14 @@ void
 e_msg_composer_hdrs_set_cc (EMsgComposerHdrs *hdrs,
 			    EDestination **cc_destv)
 {
-	gchar *str;
+	char *str;
+	
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
 	
 	str = e_destination_exportv (cc_destv);
-	bonobo_widget_set_property (BONOBO_WIDGET (hdrs->priv->cc.entry), "destinations", str, NULL); 
+	bonobo_widget_set_property (BONOBO_WIDGET (hdrs->priv->cc.entry), "destinations", str, NULL);
+	if (str && *str)
+		set_pair_visibility (hdrs, &hdrs->priv->cc, TRUE);
 	g_free (str);
 }
 
@@ -842,19 +942,31 @@ void
 e_msg_composer_hdrs_set_bcc (EMsgComposerHdrs *hdrs,
 			     EDestination **bcc_destv)
 {
-	gchar *str;
+	char *str;
+	
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
-
+	
 	str = e_destination_exportv (bcc_destv);
 	bonobo_widget_set_property (BONOBO_WIDGET (hdrs->priv->bcc.entry), "destinations", str, NULL); 
+	if (str && *str)
+		set_pair_visibility (hdrs, &hdrs->priv->bcc, TRUE);
 	g_free (str);
 }
 
+void
+e_msg_composer_hdrs_set_post_to (EMsgComposerHdrs *hdrs,
+				 const char *post_to)
+{
+	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
+	g_return_if_fail (post_to != NULL);
+	
+	evolution_folder_selector_button_set_uri (EVOLUTION_FOLDER_SELECTOR_BUTTON (hdrs->priv->post_to.entry), post_to);
+}
+
 void
 e_msg_composer_hdrs_set_subject (EMsgComposerHdrs *hdrs,
 				 const char *subject)
 {
-	g_return_if_fail (hdrs != NULL);
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
 	g_return_if_fail (subject != NULL);
 	
@@ -870,7 +982,6 @@ e_msg_composer_hdrs_get_from (EMsgComposerHdrs *hdrs)
 	const MailConfigAccount *account;
 	CamelInternetAddress *addr;
 	
-	g_return_val_if_fail (hdrs != NULL, NULL);
 	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
 	
 	account = hdrs->account;
@@ -889,41 +1000,34 @@ CamelInternetAddress *
 e_msg_composer_hdrs_get_reply_to (EMsgComposerHdrs *hdrs)
 {
 	CamelInternetAddress *addr;
-	gchar *reply_to;
+	const char *reply_to;
 	
-	g_return_val_if_fail (hdrs != NULL, NULL);
 	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
 	
-	gtk_object_get (GTK_OBJECT (hdrs->priv->reply_to.entry),
-			"text", &reply_to, NULL);
+	reply_to = e_entry_get_text (E_ENTRY (hdrs->priv->reply_to.entry));
 	
-	if (!reply_to || *reply_to == '\0') {
-		g_free (reply_to);
+	if (!reply_to || *reply_to == '\0')
 		return NULL;
-	}
 	
 	addr = camel_internet_address_new ();
 	if (camel_address_unformat (CAMEL_ADDRESS (addr), reply_to) == -1) {
-		g_free (reply_to);
 		camel_object_unref (CAMEL_OBJECT (addr));
 		return NULL;
 	}
 	
-	g_free (reply_to);
-	
 	return addr;
 }
 
 EDestination **
 e_msg_composer_hdrs_get_to (EMsgComposerHdrs *hdrs)
 {
-	gchar *str = NULL;
+	char *str = NULL;
 	EDestination **destv = NULL;
-
+	
 	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
-
+	
 	bonobo_widget_get_property (BONOBO_WIDGET (hdrs->priv->to.entry), "destinations", &str, NULL); 
-
+	
 	if (str != NULL) {
 		destv = e_destination_importv (str);
 		g_free (str);
@@ -935,36 +1039,36 @@ e_msg_composer_hdrs_get_to (EMsgComposerHdrs *hdrs)
 EDestination **
 e_msg_composer_hdrs_get_cc (EMsgComposerHdrs *hdrs)
 {
-	gchar *str = NULL;
+	char *str = NULL;
 	EDestination **destv = NULL;
-
+	
 	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
-
+	
 	bonobo_widget_get_property (BONOBO_WIDGET (hdrs->priv->cc.entry), "destinations", &str, NULL); 
-
+	
 	if (str != NULL) {
 		destv = e_destination_importv (str);
 		g_free (str);
 	}
-
+	
 	return destv;
 }
 
 EDestination **
 e_msg_composer_hdrs_get_bcc (EMsgComposerHdrs *hdrs)
 {
-	gchar *str = NULL;
+	char *str = NULL;
 	EDestination **destv = NULL;
-
+	
 	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
-
+	
 	bonobo_widget_get_property (BONOBO_WIDGET (hdrs->priv->bcc.entry), "destinations", &str, NULL); 
-
+	
 	if (str != NULL) {
 		destv = e_destination_importv (str);
 		g_free (str);
 	}
-
+	
 	return destv;
 }
 
@@ -975,50 +1079,60 @@ e_msg_composer_hdrs_get_recipients (EMsgComposerHdrs *hdrs)
 	EDestination **cc_destv;
 	EDestination **bcc_destv;
 	EDestination **recip_destv;
-	gint i, j, n;
+	int i, j, n;
 	
 	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
-
+	
 	to_destv  = e_msg_composer_hdrs_get_to (hdrs);
 	cc_destv  = e_msg_composer_hdrs_get_cc (hdrs);
 	bcc_destv = e_msg_composer_hdrs_get_bcc (hdrs);
-
+	
 	n = 0;
-
-	for (i = 0; to_destv && to_destv[i] != NULL; ++i, ++n);
-	for (i = 0; cc_destv && cc_destv[i] != NULL; ++i, ++n);
-	for (i = 0; bcc_destv && bcc_destv[i] != NULL; ++i, ++n);
-
+	
+	for (i = 0; to_destv && to_destv[i] != NULL; i++, n++);
+	for (i = 0; cc_destv && cc_destv[i] != NULL; i++, n++);
+	for (i = 0; bcc_destv && bcc_destv[i] != NULL; i++, n++);
+	
 	if (n == 0)
 		return NULL;
-
-	recip_destv = g_new (EDestination *, n+1);
-
+	
+	recip_destv = g_new (EDestination *, n + 1);
+	
 	j = 0;
-
-	for (i = 0; to_destv && to_destv[i] != NULL; ++i, ++j)
+	
+	for (i = 0; to_destv && to_destv[i] != NULL; i++, j++)
 		recip_destv[j] = to_destv[i];
-	for (i = 0; cc_destv && cc_destv[i] != NULL; ++i, ++j)
+	for (i = 0; cc_destv && cc_destv[i] != NULL; i++, j++)
 		recip_destv[j] = cc_destv[i];
-	for (i = 0; bcc_destv && bcc_destv[i] != NULL; ++i, ++j)
+	for (i = 0; bcc_destv && bcc_destv[i] != NULL; i++, j++)
 		recip_destv[j] = bcc_destv[i];
-
+	
 	g_assert (j == n);
 	recip_destv[j] = NULL;
-
+	
 	g_free (to_destv);
 	g_free (cc_destv);
 	g_free (bcc_destv);
-
+	
 	return recip_destv;
 }
 
+char *
+e_msg_composer_hdrs_get_post_to (EMsgComposerHdrs *hdrs)
+{
+	GNOME_Evolution_Folder *folder;
+
+	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
+	
+	folder = evolution_folder_selector_button_get_folder (EVOLUTION_FOLDER_SELECTOR_BUTTON (hdrs->priv->post_to.entry));
+	return folder ? g_strdup (folder->physicalUri) : NULL;
+}
+
 char *
 e_msg_composer_hdrs_get_subject (EMsgComposerHdrs *hdrs)
 {
-	gchar *subject;
+	char *subject;
 	
-	g_return_val_if_fail (hdrs != NULL, NULL);
 	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
 	
 	gtk_object_get (GTK_OBJECT (hdrs->priv->subject.entry),
@@ -1031,25 +1145,22 @@ e_msg_composer_hdrs_get_subject (EMsgComposerHdrs *hdrs)
 GtkWidget *
 e_msg_composer_hdrs_get_reply_to_entry (EMsgComposerHdrs *hdrs)
 {
-	g_return_val_if_fail (hdrs != NULL, NULL);
 	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
-
+	
 	return hdrs->priv->reply_to.entry;
 }
 
 GtkWidget *
 e_msg_composer_hdrs_get_to_entry (EMsgComposerHdrs *hdrs)
 {
-	g_return_val_if_fail (hdrs != NULL, NULL);
 	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
-
+	
 	return hdrs->priv->to.entry;
 }
 
 GtkWidget *
 e_msg_composer_hdrs_get_cc_entry (EMsgComposerHdrs *hdrs)
 {
-	g_return_val_if_fail (hdrs != NULL, NULL);
 	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
 	
 	return hdrs->priv->cc.entry;
@@ -1058,17 +1169,39 @@ e_msg_composer_hdrs_get_cc_entry (EMsgComposerHdrs *hdrs)
 GtkWidget *
 e_msg_composer_hdrs_get_bcc_entry (EMsgComposerHdrs *hdrs)
 {
-	g_return_val_if_fail (hdrs != NULL, NULL);
 	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
-
+	
 	return hdrs->priv->bcc.entry;
 }
 
+GtkWidget *
+e_msg_composer_hdrs_get_post_to_label (EMsgComposerHdrs *hdrs)
+{
+	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
+	
+	return hdrs->priv->post_to.entry;
+}
+
 GtkWidget *
 e_msg_composer_hdrs_get_subject_entry (EMsgComposerHdrs *hdrs)
 {
-	g_return_val_if_fail (hdrs != NULL, NULL);
 	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
-
+	
 	return hdrs->priv->subject.entry;
 }
+
+GtkWidget *
+e_msg_composer_hdrs_get_from_hbox (EMsgComposerHdrs *hdrs)
+{
+	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
+	
+	return hdrs->priv->from.entry;
+}
+
+GtkWidget *
+e_msg_composer_hdrs_get_from_omenu (EMsgComposerHdrs *hdrs)
+{
+	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
+	
+	return GTK_WIDGET (gtk_object_get_data (GTK_OBJECT (hdrs->priv->from.entry), "from_menu"));
+}
