@@ -33,12 +33,13 @@
 #include <camel/camel.h>
 #include "evolution-composer.h"
 #include "mail/mail-config.h"
+#include "e-util/e-html-utils.h"
 
 #define PARENT_TYPE BONOBO_OBJECT_TYPE
 static BonoboObjectClass *parent_class = NULL;
 
 static void (*send_cb) (EMsgComposer *, gpointer);
-static void (*postpone_cb) (EMsgComposer *, gpointer);
+static void (*save_draft_cb) (EMsgComposer *, int, gpointer);
 
 /* CORBA interface implementation.  */
 
@@ -115,9 +116,10 @@ impl_Composer_set_multipart_type (PortableServer_Servant servant,
 }
 
 static void
-impl_Composer_set_body_text (PortableServer_Servant servant,
-			     const CORBA_char *text,
-			     CORBA_Environment *ev)
+impl_Composer_set_body (PortableServer_Servant servant,
+			const CORBA_char *body,
+			const CORBA_char *mime_type,
+			CORBA_Environment *ev)
 {
 	BonoboObject *bonobo_object;
 	EvolutionComposer *composer;
@@ -125,7 +127,14 @@ impl_Composer_set_body_text (PortableServer_Servant servant,
 	bonobo_object = bonobo_object_from_servant (servant);
 	composer = EVOLUTION_COMPOSER (bonobo_object);
 
-	e_msg_composer_set_body_text (composer->composer, text);
+	if (!g_strcasecmp (mime_type, "text/plain")) {
+		char *htmlbody = e_text_to_html (body, E_TEXT_TO_HTML_PRE);
+		e_msg_composer_set_body_text (composer->composer, htmlbody);
+		g_free (htmlbody);
+	} else if (!g_strcasecmp (mime_type, "text/html"))
+		e_msg_composer_set_body_text (composer->composer, body);
+	else
+		e_msg_composer_set_body (composer->composer, body, mime_type);
 }
 
 static void
@@ -224,7 +233,7 @@ evolution_composer_get_epv (void)
 	epv = g_new0 (POA_GNOME_Evolution_Composer__epv, 1);
 	epv->setHeaders       = impl_Composer_set_headers;
 	epv->setMultipartType = impl_Composer_set_multipart_type;
-	epv->setBodyText      = impl_Composer_set_body_text;
+	epv->setBody          = impl_Composer_set_body;
 	epv->attachMIME       = impl_Composer_attach_MIME;
 	epv->attachData       = impl_Composer_attach_data;
 	epv->show             = impl_Composer_show;
@@ -271,8 +280,8 @@ init (EvolutionComposer *composer)
 
 	gtk_signal_connect (GTK_OBJECT (composer->composer), "send",
 			    GTK_SIGNAL_FUNC (send_cb), NULL);
-	gtk_signal_connect (GTK_OBJECT (composer->composer), "postpone",
-			    GTK_SIGNAL_FUNC (postpone_cb), NULL);
+	gtk_signal_connect (GTK_OBJECT (composer->composer), "save-draft",
+			    GTK_SIGNAL_FUNC (save_draft_cb), NULL);
 }
 
 #if 0
@@ -374,7 +383,7 @@ factory_fn (BonoboGenericFactory *factory, void *closure)
 
 void
 evolution_composer_factory_init (void (*send) (EMsgComposer *, gpointer),
-				 void (*postpone) (EMsgComposer *, gpointer))
+				 void (*save_draft) (EMsgComposer *, int, gpointer))
 {
 	if (bonobo_generic_factory_new (GNOME_EVOLUTION_MAIL_COMPOSER_FACTORY_ID,
 					factory_fn, NULL) == NULL) {
@@ -382,7 +391,7 @@ evolution_composer_factory_init (void (*send) (EMsgComposer *, gpointer),
 			  _("Cannot initialize Evolution's composer."));
 		exit (1);
 	}
-
+	
 	send_cb = send;
-	postpone_cb = postpone;
+	save_draft_cb = save_draft;
 }
