@@ -31,20 +31,21 @@
 #ifdef __cplusplus
 extern "C" {
 #pragma }
-#endif /* __cplusplus }*/
+#endif /* __cplusplus */
 
 /* these are the possible modes for replying */
 enum {
 	REPLY_SENDER,
 	REPLY_LIST,
-	REPLY_ALL
+	REPLY_ALL,
+	REPLY_POST,
+	REPLY_NO_QUOTE = 0x80	/* dont quote reply */
 };
 
 void enumerate_msg (MessageList *ml, const char *uid, gpointer data);
 
 void fetch_mail            (GtkWidget *widget, gpointer user_data);
 void send_queued_mail      (GtkWidget *widget, gpointer user_data);
-void send_receive_mail     (GtkWidget *widget, gpointer user_data);
 
 void compose_msg           (GtkWidget *widget, gpointer user_data);
 void send_to_url           (const char *url);
@@ -54,10 +55,17 @@ void forward_quoted        (GtkWidget *widget, gpointer user_data);
 void forward_attached      (GtkWidget *widget, gpointer user_data);
 void forward               (GtkWidget *widget, gpointer user_data);
 
+void post_to_url           (const char *url);
+void post_message          (GtkWidget *widget, gpointer user_data);
+void post_reply            (GtkWidget *widget, gpointer user_data);
+
+void redirect              (GtkWidget *widget, gpointer user_data);
+
 void reply_to_sender       (GtkWidget *widget, gpointer user_data);
 void reply_to_list         (GtkWidget *widget, gpointer user_data);
 void reply_to_all          (GtkWidget *widget, gpointer user_data);
 
+void colour_msg            (GtkWidget *widget, gpointer user_data);
 void delete_msg            (GtkWidget *widget, gpointer user_data);
 void undelete_msg          (GtkWidget *widget, gpointer user_data);
 void move_msg_cb           (GtkWidget *widget, gpointer user_data);
@@ -70,10 +78,12 @@ void edit_msg              (GtkWidget *widget, gpointer user_data);
 void open_msg              (GtkWidget *widget, gpointer user_data);
 void save_msg              (GtkWidget *widget, gpointer user_data);
 void view_msg              (GtkWidget *widget, gpointer user_data);
+void view_digest           (GtkWidget *widget, gpointer user_data);
 void view_source           (GtkWidget *widget, gpointer user_data);
 void next_msg              (GtkWidget *widget, gpointer user_data);
 void next_unread_msg       (GtkWidget *widget, gpointer user_data);
 void next_flagged_msg      (GtkWidget *widget, gpointer user_data);
+void next_thread           (GtkWidget *widget, gpointer user_data);
 void previous_msg          (GtkWidget *widget, gpointer user_data);
 void previous_unread_msg   (GtkWidget *widget, gpointer user_data);
 void previous_flagged_msg  (GtkWidget *widget, gpointer user_data);
@@ -81,17 +91,21 @@ void resend_msg            (GtkWidget *widget, gpointer user_data);
 void search_msg            (GtkWidget *widget, gpointer user_data);
 void load_images           (GtkWidget *widget, gpointer user_data);
 
-void move_msg              (BonoboUIComponent *uih, void *user_data, const char *path);
-void copy_msg              (BonoboUIComponent *uih, void *user_data, const char *path);
-void select_all            (BonoboUIComponent *uih, void *user_data, const char *path);
-void select_thread         (BonoboUIComponent *uih, void *user_data, const char *path);
-void invert_selection      (BonoboUIComponent *uih, void *user_data, const char *path);
-void mark_as_seen          (BonoboUIComponent *uih, void *user_data, const char *path);
-void mark_all_as_seen      (BonoboUIComponent *uih, void *user_data, const char *path);
-void mark_as_unseen        (BonoboUIComponent *uih, void *user_data, const char *path);
-void mark_as_important     (BonoboUIComponent *uih, void *user_data, const char *path);
-void mark_as_unimportant   (BonoboUIComponent *uih, void *user_data, const char *path);
-void toggle_as_important   (BonoboUIComponent *uih, void *user_data, const char *path);
+void add_sender_to_addrbook    (BonoboUIComponent *uih, void *user_data, const char *path);
+void move_msg                  (BonoboUIComponent *uih, void *user_data, const char *path);
+void copy_msg                  (BonoboUIComponent *uih, void *user_data, const char *path);
+void select_all                (BonoboUIComponent *uih, void *user_data, const char *path);
+void select_thread             (BonoboUIComponent *uih, void *user_data, const char *path);
+void invert_selection          (BonoboUIComponent *uih, void *user_data, const char *path);
+void mark_as_seen              (BonoboUIComponent *uih, void *user_data, const char *path);
+void mark_all_as_seen          (BonoboUIComponent *uih, void *user_data, const char *path);
+void mark_as_unseen            (BonoboUIComponent *uih, void *user_data, const char *path);
+void mark_as_important         (BonoboUIComponent *uih, void *user_data, const char *path);
+void mark_as_unimportant       (BonoboUIComponent *uih, void *user_data, const char *path);
+void toggle_as_important       (BonoboUIComponent *uih, void *user_data, const char *path);
+void flag_for_followup         (BonoboUIComponent *uih, void *user_data, const char *path);
+void flag_followup_completed   (BonoboUIComponent *uih, void *user_data, const char *path);
+void flag_followup_clear       (BonoboUIComponent *uih, void *user_data, const char *path);
 
 void zoom_in               (BonoboUIComponent *uih, void *user_data, const char *path);
 void zoom_out              (BonoboUIComponent *uih, void *user_data, const char *path);
@@ -102,7 +116,6 @@ void open_message          (BonoboUIComponent *uih, void *user_data, const char
 void expunge_folder        (BonoboUIComponent *uih, void *user_data, const char *path);
 void filter_edit           (BonoboUIComponent *uih, void *user_data, const char *path);
 void vfolder_edit_vfolders (BonoboUIComponent *uih, void *user_data, const char *path);
-void providers_config      (BonoboUIComponent *uih, void *user_data, const char *path);
 void manage_subscriptions  (BonoboUIComponent *uih, void *user_data, const char *path);
 
 void configure_folder      (BonoboUIComponent *uih, void *user_data, const char *path);
@@ -114,7 +127,7 @@ void empty_trash           (BonoboUIComponent *uih, void *user_data, const char
 void mail_reply            (CamelFolder *folder, CamelMimeMessage *msg, const char *uid, int mode);
 
 void composer_send_cb      (EMsgComposer *composer, gpointer data);
-void composer_postpone_cb  (EMsgComposer *composer, gpointer data);
+void composer_save_draft_cb (EMsgComposer *composer, int quit, gpointer data);
 
 void forward_messages	   (CamelFolder *folder, GPtrArray *uids, gboolean inline);
 
