@@ -24,6 +24,12 @@
 #include <config.h>
 #endif
 
+#include <stdio.h>
+#include <errno.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+
 #include <glib.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
@@ -34,8 +40,6 @@
 #include <gtkhtml/htmlengine.h>
 #include <gtkhtml/htmlselection.h>
 
-#include <libgnomevfs/gnome-vfs.h>
-
 #include <gal/util/e-util.h>
 #include <gal/widgets/e-gui-utils.h>
 #include <gal/widgets/e-unicode.h>
@@ -68,6 +72,8 @@
 
 extern char *evolution_dir;
 
+static GList *all_summaries = NULL;
+
 static GtkObjectClass *e_summary_parent_class;
 
 struct _ESummaryMailFolderInfo {
@@ -77,15 +83,7 @@ struct _ESummaryMailFolderInfo {
 	int unread;
 };
 
-typedef struct _DownloadInfo {
-	GtkHTMLStream *stream;
-	char *uri;
-	char *buffer, *ptr;
-	guint32 bufsize;
-} DownloadInfo;
-
 struct _ESummaryPrivate {
-	GNOME_Evolution_Shell shell;
 	GNOME_Evolution_ShellView shell_view_interface;
 
 	GtkWidget *html_scroller;
@@ -130,6 +128,8 @@ destroy (GtkObject *object)
 		return;
 	}
 
+	all_summaries = g_list_remove (all_summaries, summary);
+	
 	if (priv->pending_reload_tag) {
 		gtk_timeout_remove (priv->pending_reload_tag);
 		priv->pending_reload_tag = 0;
@@ -239,6 +239,16 @@ e_summary_draw (ESummary *summary)
 	g_string_free (string, TRUE);
 }
 
+void
+e_summary_redraw_all (void)
+{
+	GList *p;
+
+	for (p = all_summaries; p; p = p->next) {
+		e_summary_draw (E_SUMMARY (p->data));
+	}
+}
+
 static char *
 e_pixmap_file (const char *filename)
 {
@@ -289,74 +299,6 @@ struct _imgcache {
 	int bufsize;
 };
 
-static void
-close_callback (GnomeVFSAsyncHandle *handle,
-		GnomeVFSResult result,
-		gpointer data)
-{
-	DownloadInfo *info = data;
-	struct _imgcache *img;
-
-	if (images_cache == NULL) {
-		images_cache = g_hash_table_new (g_str_hash, g_str_equal);
-	}
-
-	img = g_new (struct _imgcache, 1);
-	img->buffer = info->buffer;
-	img->bufsize = info->bufsize;
-
-	g_hash_table_insert (images_cache, info->uri, img);
-	g_free (info);
-}
-
-/* The way this behaves is a workaround for ximian bug 10235: loading
- * the image into gtkhtml progressively will result in garbage being
- * drawn, so we wait until we've read the whole thing and then write
- * it all at once.
- */
-static void
-read_callback (GnomeVFSAsyncHandle *handle,
-	       GnomeVFSResult result,
-	       gpointer buffer,
-	       GnomeVFSFileSize bytes_requested,
-	       GnomeVFSFileSize bytes_read,
-	       gpointer data)
-{
-	DownloadInfo *info = data;
-
-	if (result != GNOME_VFS_OK && result != GNOME_VFS_ERROR_EOF) {
-		gtk_html_stream_close (info->stream, GTK_HTML_STREAM_ERROR);
-		gnome_vfs_async_close (handle, close_callback, info);
-	} else if (bytes_read == 0) {
-		gtk_html_stream_write (info->stream, info->buffer, info->bufsize);
-		gtk_html_stream_close (info->stream, GTK_HTML_STREAM_OK);
-		gnome_vfs_async_close (handle, close_callback, info);
-	} else {
-		bytes_read += info->ptr - info->buffer;
-		info->bufsize += 4096;
-		info->buffer = g_realloc (info->buffer, info->bufsize);
-		info->ptr = info->buffer + bytes_read;
-		gnome_vfs_async_read (handle, info->ptr, 4095, read_callback, info);
-	}
-}
-
-static void
-open_callback (GnomeVFSAsyncHandle *handle,
-	       GnomeVFSResult result,
-	       DownloadInfo *info)
-{
-	if (result != GNOME_VFS_OK) {
-		gtk_html_stream_close (info->stream, GTK_HTML_STREAM_ERROR);
-		g_free (info->uri);
-		g_free (info);
-		return;
-	}
-
-	info->bufsize = 4096;
-	info->buffer = info->ptr = g_new (char, info->bufsize);
-	gnome_vfs_async_read (handle, info->buffer, 4095, read_callback, info);
-}
-
 static void
 e_summary_url_clicked (GtkHTML *html,
 		       const char *url,
@@ -387,6 +329,52 @@ e_summary_url_clicked (GtkHTML *html,
 	protocol_listener->listener (summary, url, protocol_listener->closure);
 }
 
+static char *
+e_read_file_with_length (const char *filename,
+			 size_t *length)
+{
+	int fd;
+	struct stat stat_buf;
+	char *buf;
+	size_t bytes_read, size;
+
+	g_return_val_if_fail (filename != NULL, NULL);
+
+	fd = open (filename, O_RDONLY);
+	g_return_val_if_fail (fd != -1, NULL);
+
+	fstat (fd, &stat_buf);
+	size = stat_buf.st_size;
+	buf = g_new (char, size + 1);
+
+	bytes_read = 0;
+	while (bytes_read < size) {
+		ssize_t rc;
+
+		rc = read (fd, buf + bytes_read, size - bytes_read);
+		if (rc < 0) {
+			if (errno != EINTR) {
+				close (fd);
+				g_free (buf);
+				
+				return NULL;
+			}
+		} else if (rc == 0) {
+			break;
+		} else {
+			bytes_read += rc;
+		}
+	}
+
+	buf[bytes_read] = '\0';
+
+	if (length) {
+		*length = bytes_read;
+	}
+
+	return buf;
+}
+
 static void
 e_summary_url_requested (GtkHTML *html,
 			 const char *url,
@@ -394,8 +382,6 @@ e_summary_url_requested (GtkHTML *html,
 			 ESummary *summary)
 {
 	char *filename;
-	GnomeVFSAsyncHandle *handle;
-	DownloadInfo *info;
 	struct _imgcache *img = NULL;
 
 	if (strncasecmp (url, "file:", 5) == 0) {
@@ -414,19 +400,28 @@ e_summary_url_requested (GtkHTML *html,
 
 	if (images_cache != NULL) {
 		img = g_hash_table_lookup (images_cache, filename);
+	} else {
+		images_cache = g_hash_table_new (g_str_hash, g_str_equal);
 	}
 
 	if (img == NULL) {
-		info = g_new (DownloadInfo, 1);
-		info->stream = stream;
-		info->uri = filename;
+		size_t length;
+		char *contents;
 
-		gnome_vfs_async_open (&handle, filename, GNOME_VFS_OPEN_READ,
-				      (GnomeVFSAsyncOpenCallback) open_callback, info);
-	} else {
-		gtk_html_stream_write (stream, img->buffer, img->bufsize);
-		gtk_html_stream_close (stream, GTK_HTML_STREAM_OK);
+		contents = e_read_file_with_length (filename, &length);
+		if (contents == NULL) {
+			return;
+		}
+
+		img = g_new (struct _imgcache, 1);
+		img->buffer = contents;
+		img->bufsize = length;
+
+		g_hash_table_insert (images_cache, g_strdup (filename), img);
 	}
+
+	gtk_html_stream_write (stream, img->buffer, img->bufsize);
+	gtk_html_stream_close (stream, GTK_HTML_STREAM_OK);
 }
 
 static void
@@ -461,7 +456,7 @@ alarm_fn (gpointer alarm_id,
 	e_summary_reconfigure (summary);
 }
 	
-#define DEFAULT_HTML "<html><head><title>Summary</title></head><body bgcolor=\"#ffffff\">hello</body></html>" 
+#define DEFAULT_HTML "<html><head><title>Summary</title></head><body bgcolor=\"#ffffff\">%s</body></html>" 
 
 static void
 e_summary_init (ESummary *summary)
@@ -471,12 +466,13 @@ e_summary_init (ESummary *summary)
 	ESummaryPrivate *priv;
 	GdkColor bgcolor = {0, 0xffff, 0xffff, 0xffff};
 	time_t t, day_end;
+	char *def, *default_utf;
 
 	summary->priv = g_new (ESummaryPrivate, 1);
 
 	priv = summary->priv;
 
-	priv->frozen = FALSE;
+	priv->frozen = TRUE;
 	priv->redraw_pending = FALSE;
 	priv->pending_reload_tag = 0;
 
@@ -489,8 +485,11 @@ e_summary_init (ESummary *summary)
 	gtk_html_set_default_content_type (GTK_HTML (priv->html),
 					   "text/html; charset=utf-8");
 	gtk_html_set_default_background_color (GTK_HTML (priv->html), &bgcolor);
-	gtk_html_load_from_string (GTK_HTML (priv->html), DEFAULT_HTML,
-				   strlen (DEFAULT_HTML));
+	def = g_strdup_printf (DEFAULT_HTML, _("Please wait..."));
+	default_utf = e_utf8_from_locale_string (def);
+	gtk_html_load_from_string (GTK_HTML (priv->html), default_utf, strlen (default_utf));
+	g_free (def);
+	g_free (default_utf);
 
 	gtk_signal_connect (GTK_OBJECT (priv->html), "url-requested",
 			    GTK_SIGNAL_FUNC (e_summary_url_requested), summary);
@@ -509,9 +508,6 @@ e_summary_init (ESummary *summary)
 	priv->protocol_hash = NULL;
 	priv->connections = NULL;
 
-	summary->prefs_window = NULL;
-	e_summary_preferences_init (summary);
-
 	CORBA_exception_init (&ev);
 	db = bonobo_get_object ("wombat:", "Bonobo/ConfigDatabase", &ev);
 	if (BONOBO_EX (&ev) || db == CORBA_OBJECT_NIL) {
@@ -545,16 +541,19 @@ E_MAKE_TYPE (e_summary, "ESummary", ESummary, e_summary_class_init,
 	     e_summary_init, PARENT_TYPE);
 
 GtkWidget *
-e_summary_new (const GNOME_Evolution_Shell shell)
+e_summary_new (const GNOME_Evolution_Shell shell,
+	       ESummaryPrefs *prefs)
 {
 	ESummary *summary;
 
 	summary = gtk_type_new (e_summary_get_type ());
-	summary->priv->shell = shell;
-
+	summary->shell = shell;
+	/* Just get a pointer to the global preferences */
+	summary->preferences = prefs;
+	
 	e_summary_add_protocol_listener (summary, "evolution", e_summary_evolution_protocol_listener, summary);
 
-	e_summary_mail_init (summary, shell);
+	e_summary_mail_init (summary);
 	e_summary_calendar_init (summary);
 	e_summary_tasks_init (summary);
 	e_summary_rdf_init (summary);
@@ -562,6 +561,7 @@ e_summary_new (const GNOME_Evolution_Shell shell)
 
 /*  	e_summary_draw (summary); */
 
+	all_summaries = g_list_prepend (all_summaries, summary);
 	return GTK_WIDGET (summary);
 }
 
@@ -735,10 +735,6 @@ e_summary_unset_message (ESummary *summary)
 void
 e_summary_reconfigure (ESummary *summary)
 {
-	if (summary->mail != NULL) {
-		e_summary_mail_reconfigure (summary);
-	}
-
 	if (summary->rdf != NULL) {
 		e_summary_rdf_reconfigure (summary);
 	}
@@ -754,6 +750,21 @@ e_summary_reconfigure (ESummary *summary)
 	if (summary->tasks != NULL) {
 		e_summary_tasks_reconfigure (summary);
 	}
+
+	e_summary_draw (summary);
+}
+
+void
+e_summary_reconfigure_all (void)
+{
+	GList *p;
+
+	/* This is here, because it only needs to be done once for all summaries */
+	e_summary_mail_reconfigure ();
+
+	for (p = all_summaries; p; p = p->next) {
+		e_summary_reconfigure (E_SUMMARY (p->data));
+	}
 }
 
 static gint
@@ -808,9 +819,7 @@ e_summary_count_connections (ESummary *summary)
 	GList *p;
 	int count = 0;
 
-	if (summary == NULL) {
-		return 0;
-	}
+	g_return_val_if_fail (IS_E_SUMMARY (summary), 0);
 
 	for (p = summary->priv->connections; p; p = p->next) {
 		ESummaryConnection *c;
@@ -829,9 +838,7 @@ e_summary_add_connections (ESummary *summary)
 	GList *p;
 	GList *connections = NULL;
 
-	if (summary == NULL) {
-		return NULL;
-	}
+	g_return_val_if_fail (IS_E_SUMMARY (summary), NULL);
 
 	for (p = summary->priv->connections; p; p = p->next) {
 		ESummaryConnection *c;
@@ -855,9 +862,7 @@ e_summary_set_online (ESummary *summary,
 {
 	GList *p;
 
-	if (summary == NULL) {
-		return;
-	}
+	g_return_if_fail (IS_E_SUMMARY (summary));
 
 	for (p = summary->priv->connections; p; p = p->next) {
 		ESummaryConnection *c;
@@ -908,6 +913,7 @@ void
 e_summary_freeze (ESummary *summary)
 {
 	g_return_if_fail (IS_E_SUMMARY (summary));
+	g_return_if_fail (summary->priv != NULL);
 
 	if (summary->priv->frozen == TRUE) {
 		return;
@@ -920,6 +926,8 @@ void
 e_summary_thaw (ESummary *summary)
 {
 	g_return_if_fail (IS_E_SUMMARY (summary));
+	g_return_if_fail (summary->priv != NULL);
+
 	if (summary->priv->frozen == FALSE) {
 		return;
 	}
