@@ -4,9 +4,8 @@
  * Copyright (C) 2000, 2001 Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,12 +26,18 @@
 
 #include "e-corba-storage.h"
 
-#include <glib.h>
-#include <bonobo/bonobo-main.h>
-#include <gal/util/e-util.h>
+#include "e-shell-constants.h"
 
 #include "Evolution.h"
 
+#include <glib.h>
+#include <gal/util/e-util.h>
+
+#include <bonobo/bonobo-main.h>
+#include <bonobo/bonobo-exception.h>
+
+#include <gdk/gdkx.h>
+
 
 #define PARENT_TYPE E_TYPE_STORAGE
 static EStorageClass *parent_class = NULL;
@@ -40,8 +45,6 @@ static EStorageClass *parent_class = NULL;
 typedef struct _StorageListenerServant StorageListenerServant;
 
 struct _ECorbaStoragePrivate {
-	char *name;
-
 	GNOME_Evolution_Storage storage_interface;
 
 	/* The Evolution::StorageListener interface we expose.  */
@@ -105,12 +108,15 @@ impl_StorageListener_notifyFolderCreated (PortableServer_Servant servant,
 	storage_listener_servant = (StorageListenerServant *) servant;
 	storage = storage_listener_servant->storage;
 
-	e_folder = e_folder_new (folder->displayName,
-				 folder->type,
-				 folder->description);
+	e_folder = e_folder_new (folder->displayName, folder->type, folder->description);
 
-	e_folder_set_physical_uri (e_folder, folder->physicalUri);
-	e_folder_set_unread_count (e_folder, folder->unreadCount);
+	e_folder_set_physical_uri     (e_folder, folder->physicalUri);
+	e_folder_set_unread_count     (e_folder, folder->unreadCount);
+	e_folder_set_can_sync_offline (e_folder, folder->canSyncOffline);
+	e_folder_set_sorting_priority (e_folder, folder->sortingPriority);
+
+	if (folder->customIconName[0] != '\0')
+		e_folder_set_custom_icon (e_folder, folder->customIconName);
 
 	if (! e_storage_new_folder (storage, path, e_folder)) {
 		g_warning ("Cannot register folder -- %s %s\n", path, folder->displayName);
@@ -126,7 +132,6 @@ impl_StorageListener_notifyFolderCreated (PortableServer_Servant servant,
 static void
 impl_StorageListener_notifyFolderUpdated (PortableServer_Servant servant,
 					  const CORBA_char *path,
-					  const CORBA_char *display_name,
 					  CORBA_long unread_count,
 					  CORBA_Environment *ev)
 {
@@ -146,7 +151,6 @@ impl_StorageListener_notifyFolderUpdated (PortableServer_Servant servant,
 		return;
 	}
 
-	e_folder_set_name (e_folder, display_name);
 	e_folder_set_unread_count (e_folder, unread_count);
 }
 
@@ -168,6 +172,27 @@ impl_StorageListener_notifyFolderRemoved (PortableServer_Servant servant,
 				     NULL);
 }
 
+static void
+impl_StorageListener_notifyHasSubfolders (PortableServer_Servant servant,
+					  const CORBA_char *path,
+					  const CORBA_char *message,
+					  CORBA_Environment *ev)
+{
+	StorageListenerServant *storage_listener_servant;
+	EStorage *storage;
+
+	storage_listener_servant = (StorageListenerServant *) servant;
+	storage = storage_listener_servant->storage;
+
+	if (! e_storage_has_subfolders (storage, path, message)) {
+		g_warning ("Cannot register subfolder tree -- %s\n", path);
+		CORBA_exception_set (ev,
+				     CORBA_USER_EXCEPTION,
+				     ex_GNOME_Evolution_StorageListener_Exists,
+				     NULL);
+	}
+}
+
 
 static gboolean
 setup_storage_listener (ECorbaStorage *corba_storage)
@@ -220,8 +245,6 @@ destroy (GtkObject *object)
 	corba_storage = E_CORBA_STORAGE (object);
 	priv = corba_storage->priv;
 
-	g_free (priv->name);
-
 	CORBA_exception_init (&ev);
 
 	if (priv->storage_interface != CORBA_OBJECT_NIL) {
@@ -247,37 +270,14 @@ destroy (GtkObject *object)
 
 	g_free (priv);
 
+	corba_storage->priv = NULL;
+
 	(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
 
 /* EStorage methods.  */
 
-static const char *
-get_name (EStorage *storage)
-{
-	ECorbaStorage *corba_storage;
-	ECorbaStoragePrivate *priv;
-
-	corba_storage = E_CORBA_STORAGE (storage);
-	priv = corba_storage->priv;
-
-	return priv->name;
-}
-static const char *
-get_display_name (EStorage *storage)
-{
-	ECorbaStorage *corba_storage;
-	ECorbaStoragePrivate *priv;
-
-	corba_storage = E_CORBA_STORAGE (storage);
-	priv = corba_storage->priv;
-
-	/* FIXME: Abstract a display_name, return it.  Necessary
-           changes to the IDL and EvolutionStorage required. */
-	return priv->name;
-}
-
 struct async_folder_closure {
 	EStorageResultCallback callback;
 	EStorage *storage;
@@ -294,37 +294,9 @@ async_folder_cb (BonoboListener *listener, char *event_name,
 	EStorageResult result;
 
 	corba_result = any->_value;
-	switch (*corba_result) {
-	case GNOME_Evolution_Storage_OK:
-		result = E_STORAGE_OK;
-		break;
-	case GNOME_Evolution_Storage_UNSUPPORTED_OPERATION:
-		result = E_STORAGE_UNSUPPORTEDOPERATION;
-		break;
-	case GNOME_Evolution_Storage_UNSUPPORTED_TYPE:
-		result = E_STORAGE_UNSUPPORTEDTYPE;
-		break;
-	case GNOME_Evolution_Storage_ALREADY_EXISTS:
-		result = E_STORAGE_EXISTS;
-		break;
-	case GNOME_Evolution_Storage_DOES_NOT_EXIST:
-		result = E_STORAGE_NOTFOUND;
-		break;
-	case GNOME_Evolution_Storage_PERMISSION_DENIED:
-		result = E_STORAGE_PERMISSIONDENIED;
-		break;
-	case GNOME_Evolution_Storage_NO_SPACE:
-		result = E_STORAGE_NOSPACE;
-		break;
-	case GNOME_Evolution_Storage_INVALID_URI:
-	case GNOME_Evolution_Storage_NOT_EMPTY:
-	case GNOME_Evolution_Storage_GENERIC_ERROR:
-	default:
-		result = E_STORAGE_GENERICERROR;
-		break;
-	}
+	result = e_corba_storage_corba_result_to_storage_result (*corba_result);
 
-	closure->callback (closure->storage, result, closure->data);
+	(* closure->callback) (closure->storage, result, closure->data);
 	bonobo_object_unref (BONOBO_OBJECT (listener));
 	g_free (closure);
 }
@@ -371,7 +343,7 @@ async_create_folder (EStorage *storage, const char *path,
 						   corba_listener, &ev);
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
-		callback (storage, E_STORAGE_GENERICERROR, data);
+		(* callback) (storage, E_STORAGE_GENERICERROR, data);
 		bonobo_object_unref (BONOBO_OBJECT (listener));
 		g_free (closure);
 	}
@@ -411,13 +383,95 @@ async_remove_folder (EStorage *storage, const char *path,
 						   corba_listener, &ev);
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
-		callback (storage, E_STORAGE_GENERICERROR, data);
+		(* callback) (storage, E_STORAGE_GENERICERROR, data);
 		bonobo_object_unref (BONOBO_OBJECT (listener));
 		g_free (closure);
 	}
 	CORBA_exception_free (&ev);
 }
 
+static void
+async_xfer_folder (EStorage *storage,
+		   const char *source_path,
+		   const char *destination_path,
+		   gboolean remove_source,
+		   EStorageResultCallback callback,
+		   void *data)
+{
+	ECorbaStorage *corba_storage;
+	ECorbaStoragePrivate *priv;
+	EFolder *folder;
+	BonoboListener *listener;
+	Bonobo_Listener corba_listener;
+	CORBA_Environment ev;
+	struct async_folder_closure *closure;
+
+	corba_storage = E_CORBA_STORAGE (storage);
+	priv = corba_storage->priv;
+
+	folder = e_storage_get_folder (storage, source_path);
+	if (e_folder_get_is_stock (folder) && remove_source)
+		(* callback) (storage, E_STORAGE_CANTCHANGESTOCKFOLDER, data);
+
+	closure = g_new (struct async_folder_closure, 1);
+	closure->callback = callback;
+	closure->storage = storage;
+	closure->data = data;
+	listener = bonobo_listener_new (async_folder_cb, closure);
+	corba_listener = bonobo_object_corba_objref (BONOBO_OBJECT (listener));
+
+	CORBA_exception_init (&ev);
+	GNOME_Evolution_Storage_asyncXferFolder (priv->storage_interface,
+						 source_path, destination_path,
+						 remove_source, corba_listener, &ev);
+
+	if (ev._major != CORBA_NO_EXCEPTION) {
+		(* callback) (storage, E_STORAGE_GENERICERROR, data);
+		bonobo_object_unref (BONOBO_OBJECT (listener));
+		g_free (closure);
+	}
+	CORBA_exception_free (&ev);
+}
+
+static gboolean
+async_open_folder_idle (gpointer data)
+{
+	gpointer *pair = data;
+
+	EStorage *storage = pair[0];
+	char *path = pair[1];
+
+	ECorbaStorage *corba_storage;
+	CORBA_Environment ev;
+
+	corba_storage = E_CORBA_STORAGE (storage);
+
+	if (corba_storage->priv != NULL) {
+
+		CORBA_exception_init (&ev);
+		GNOME_Evolution_Storage_asyncOpenFolder (corba_storage->priv->storage_interface,
+							 path, &ev);
+		CORBA_exception_free (&ev);
+	}
+
+	gtk_object_unref (GTK_OBJECT (storage));
+	g_free (path);
+	g_free (pair);
+
+	return FALSE;
+}
+
+static void
+async_open_folder (EStorage *storage,
+		   const char *path)
+{
+	gpointer *pair = g_new (gpointer, 2);
+	pair[0] = storage;
+	gtk_object_ref (GTK_OBJECT (storage));
+	pair[1] = g_strdup (path);
+
+	g_idle_add (async_open_folder_idle, pair);
+}
 
 
 static void
@@ -436,6 +490,7 @@ corba_class_init (void)
 	epv->notifyFolderCreated = impl_StorageListener_notifyFolderCreated;
 	epv->notifyFolderUpdated = impl_StorageListener_notifyFolderUpdated;
 	epv->notifyFolderRemoved = impl_StorageListener_notifyFolderRemoved;
+	epv->notifyHasSubfolders = impl_StorageListener_notifyHasSubfolders;
 
 	vepv = &storage_listener_vepv;
 	vepv->_base_epv                     = base_epv;
@@ -452,10 +507,10 @@ class_init (ECorbaStorageClass *klass)
 	object_class->destroy = destroy;
 
 	storage_class = E_STORAGE_CLASS (klass);
-	storage_class->get_name = get_name;
-	storage_class->get_display_name = get_display_name;
 	storage_class->async_create_folder = async_create_folder;
 	storage_class->async_remove_folder = async_remove_folder;
+	storage_class->async_xfer_folder = async_xfer_folder;
+	storage_class->async_open_folder = async_open_folder;
 
 	corba_class_init ();
 
@@ -468,7 +523,6 @@ init (ECorbaStorage *corba_storage)
 	ECorbaStoragePrivate *priv;
 
 	priv = g_new (ECorbaStoragePrivate, 1);
-	priv->name              = NULL;
 	priv->storage_interface = CORBA_OBJECT_NIL;
 
 	corba_storage->priv = priv;
@@ -478,25 +532,24 @@ init (ECorbaStorage *corba_storage)
 /* FIXME: OK to have a boolean construct function?  */
 void
 e_corba_storage_construct (ECorbaStorage *corba_storage,
-			   const char *toplevel_node_uri,
-			   const char *toplevel_node_type,
 			   const GNOME_Evolution_Storage storage_interface,
 			   const char *name)
 {
 	ECorbaStoragePrivate *priv;
 	CORBA_Environment ev;
+	EFolder *root_folder;
 
 	g_return_if_fail (corba_storage != NULL);
 	g_return_if_fail (E_IS_CORBA_STORAGE (corba_storage));
 	g_return_if_fail (storage_interface != CORBA_OBJECT_NIL);
 	g_return_if_fail (name != NULL);
 
-	e_storage_construct (E_STORAGE (corba_storage), toplevel_node_uri, toplevel_node_type);
+	/* FIXME: Need separate name and display name. */
+	root_folder = e_folder_new (name, "noselect", "");
+	e_storage_construct (E_STORAGE (corba_storage), name, root_folder);
 
 	priv = corba_storage->priv;
 
-	priv->name = g_strdup (name);
-
 	CORBA_exception_init (&ev);
 
 	Bonobo_Unknown_ref (storage_interface, &ev);
@@ -511,9 +564,7 @@ e_corba_storage_construct (ECorbaStorage *corba_storage,
 }
 
 EStorage *
-e_corba_storage_new (const char *toplevel_node_uri,
-		     const char *toplevel_node_type,
-		     const GNOME_Evolution_Storage storage_interface,
+e_corba_storage_new (const GNOME_Evolution_Storage storage_interface,
 		     const char *name)
 {
 	EStorage *new;
@@ -524,8 +575,6 @@ e_corba_storage_new (const char *toplevel_node_uri,
 	new = gtk_type_new (e_corba_storage_get_type ());
 
 	e_corba_storage_construct (E_CORBA_STORAGE (new),
-				   toplevel_node_uri,
-				   toplevel_node_type,
 				   storage_interface, name);
 
 	return new;
@@ -541,5 +590,125 @@ e_corba_storage_get_StorageListener (ECorbaStorage *corba_storage)
 	return corba_storage->priv->storage_listener_interface;
 }
 
+GNOME_Evolution_Storage
+e_corba_storage_get_corba_objref (ECorbaStorage *corba_storage)
+{
+	g_return_val_if_fail (corba_storage != NULL, CORBA_OBJECT_NIL);
+	g_return_val_if_fail (E_IS_CORBA_STORAGE (corba_storage), CORBA_OBJECT_NIL);
+
+	return corba_storage->priv->storage_interface;
+}
+
+
+GSList *
+e_corba_storage_get_folder_property_items (ECorbaStorage *corba_storage)
+{
+	GNOME_Evolution_Storage_FolderPropertyItemList *corba_items;
+	CORBA_Environment ev;
+	GSList *list;
+	int i;
+
+	g_return_val_if_fail (E_IS_CORBA_STORAGE (corba_storage), NULL);
+
+	CORBA_exception_init (&ev);
+
+	corba_items = GNOME_Evolution_Storage__get_folderPropertyItems (corba_storage->priv->storage_interface,
+									&ev);
+
+	if (BONOBO_EX (&ev)) {
+		CORBA_exception_free (&ev);
+		return NULL;
+	}
+
+	list = NULL;
+	for (i = 0; i < corba_items->_length; i ++) {
+		ECorbaStoragePropertyItem *item;
+
+		item = g_new (ECorbaStoragePropertyItem, 1);
+		item->label   = g_strdup (corba_items->_buffer[i].label);
+		item->tooltip = g_strdup (corba_items->_buffer[i].tooltip);
+		item->icon    = NULL; /* We don't care for now -- FIXME */
+
+		list = g_slist_prepend (list, item);
+	}
+	list = g_slist_reverse (list);
+
+	CORBA_free (corba_items);
+	CORBA_exception_free (&ev);
+
+	return list;
+}
+
+void
+e_corba_storage_free_property_items_list (GSList *list)
+{
+	GSList *p;
+
+	for (p = list; p != NULL; p = p->next) {
+		ECorbaStoragePropertyItem *item;
+
+		item = (ECorbaStoragePropertyItem *) p->data;
+
+		if (item->icon != NULL)
+			gdk_pixbuf_unref (item->icon);
+		g_free (item->label);
+		g_free (item->tooltip);
+		g_free (item);
+	}
+
+	g_slist_free (list);
+}
+
+void
+e_corba_storage_show_folder_properties (ECorbaStorage *corba_storage,
+					const char *path,
+					int property_item_id,
+					GdkWindow *parent_window)
+{
+	CORBA_Environment ev;
+
+	g_return_if_fail (E_IS_CORBA_STORAGE (corba_storage));
+	g_return_if_fail (path != NULL && path[0] == E_PATH_SEPARATOR);
+
+	CORBA_exception_init (&ev);
+
+	GNOME_Evolution_Storage_showFolderProperties (corba_storage->priv->storage_interface,
+						      path, property_item_id,
+						      GDK_WINDOW_XWINDOW (parent_window),
+						      &ev);
+	if (BONOBO_EX (&ev))
+		g_warning ("Error in Storage::showFolderProperties -- %s", BONOBO_EX_ID (&ev));
+
+	CORBA_exception_free (&ev);
+}
+
+EStorageResult
+e_corba_storage_corba_result_to_storage_result (GNOME_Evolution_Storage_Result corba_result)
+{
+	switch (corba_result) {
+	case GNOME_Evolution_Storage_OK:
+		return E_STORAGE_OK;
+	case GNOME_Evolution_Storage_UNSUPPORTED_OPERATION:
+		return E_STORAGE_UNSUPPORTEDOPERATION;
+	case GNOME_Evolution_Storage_UNSUPPORTED_TYPE:
+		return E_STORAGE_UNSUPPORTEDTYPE;
+	case GNOME_Evolution_Storage_INVALID_URI:
+		return E_STORAGE_INVALIDNAME;
+	case GNOME_Evolution_Storage_ALREADY_EXISTS:
+		return E_STORAGE_EXISTS;
+	case GNOME_Evolution_Storage_DOES_NOT_EXIST:
+		return E_STORAGE_NOTFOUND;
+	case GNOME_Evolution_Storage_PERMISSION_DENIED:
+		return E_STORAGE_PERMISSIONDENIED;
+	case GNOME_Evolution_Storage_NO_SPACE:
+		return E_STORAGE_NOSPACE;
+	case GNOME_Evolution_Storage_NOT_EMPTY:
+		return E_STORAGE_NOTEMPTY;
+	case GNOME_Evolution_Storage_GENERIC_ERROR:
+	default:
+		return E_STORAGE_GENERICERROR;
+	}
+}
+
 
 E_MAKE_TYPE (e_corba_storage, "ECorbaStorage", ECorbaStorage, class_init, init, PARENT_TYPE)
