@@ -1,7 +1,7 @@
- /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* e-folder.h
  *
- * Copyright (C) 2000  Ximian, Inc.
+ * Copyright (C) 2000, 2001, 2002  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -23,6 +23,8 @@
 #ifndef _E_FOLDER_H_
 #define _E_FOLDER_H_
 
+#include "Evolution.h"
+
 #include <gtk/gtkobject.h>
 
 #ifdef __cplusplus
@@ -70,21 +72,31 @@ EFolder *e_folder_new        (const char *name,
 			      const char *type,
 			      const char *description);
 
-const char *e_folder_get_name         (EFolder *folder);
-const char *e_folder_get_type_string  (EFolder *folder);
-const char *e_folder_get_description  (EFolder *folder);
-const char *e_folder_get_physical_uri (EFolder *folder);
-int         e_folder_get_unread_count (EFolder *folder);
-gboolean    e_folder_get_highlighted  (EFolder *folder);
-gboolean    e_folder_get_is_stock     (EFolder *folder);
+const char *e_folder_get_name         	  (EFolder *folder);
+const char *e_folder_get_type_string  	  (EFolder *folder);
+const char *e_folder_get_description  	  (EFolder *folder);
+const char *e_folder_get_physical_uri 	  (EFolder *folder);
+int         e_folder_get_unread_count 	  (EFolder *folder);
+gboolean    e_folder_get_highlighted  	  (EFolder *folder);
+gboolean    e_folder_get_is_stock     	  (EFolder *folder);
+gboolean    e_folder_get_can_sync_offline (EFolder *folder);
+const char *e_folder_get_custom_icon_name (EFolder *folder);
+int         e_folder_get_sorting_priority (EFolder *folder);
 
-void e_folder_set_name            (EFolder *folder, const char *name);
-void e_folder_set_type_string     (EFolder *folder, const char *type);
-void e_folder_set_description     (EFolder *folder, const char *description);
-void e_folder_set_physical_uri    (EFolder *folder, const char *physical_uri);
-void e_folder_set_unread_count    (EFolder *folder, int unread_count);
-void e_folder_set_child_highlight (EFolder *folder, gboolean highlighted);
-void e_folder_set_is_stock        (EFolder *folder, gboolean is_stock);
+void  e_folder_set_name              (EFolder *folder, const char *name);
+void  e_folder_set_type_string       (EFolder *folder, const char *type);
+void  e_folder_set_description       (EFolder *folder, const char *description);
+void  e_folder_set_physical_uri      (EFolder *folder, const char *physical_uri);
+void  e_folder_set_unread_count      (EFolder *folder, int unread_count);
+void  e_folder_set_child_highlight   (EFolder *folder, gboolean highlighted);
+void  e_folder_set_is_stock          (EFolder *folder, gboolean is_stock);
+void  e_folder_set_can_sync_offline  (EFolder *folder, gboolean can_sync_offline);
+void  e_folder_set_custom_icon       (EFolder *folder, const char *icon_name);
+void  e_folder_set_sorting_priority  (EFolder *folder, int sorting_priority);
+
+void e_folder_to_corba  (EFolder    *folder,
+			 const char *evolution_uri,
+			 GNOME_Evolution_Folder *folder_return);
 
 #ifdef __cplusplus
 }
