@@ -45,6 +45,7 @@
 
 #include <gal/widgets/e-gui-utils.h>
 #include <gal/util/e-util.h>
+#include <gal/util/e-unicode-i18n.h>
 
 #include "Evolution.h"
 
@@ -56,17 +57,21 @@
 #include "e-corba-storage-registry.h"
 #include "e-folder-type-registry.h"
 #include "e-local-storage.h"
+#include "e-setup.h"
 #include "e-shell-constants.h"
+#include "e-shell-corba-icon-utils.h"
 #include "e-shell-folder-selection-dialog.h"
 #include "e-shell-offline-handler.h"
+#include "e-shell-settings-dialog.h"
 #include "e-shell-startup-wizard.h"
 #include "e-shell-view.h"
 #include "e-shortcuts.h"
 #include "e-storage-set.h"
 #include "e-splash.h"
-#include "e-summary-storage.h"
 #include "e-uri-schema-registry.h"
 
+#include "evolution-shell-component-client.h"
+#include "evolution-shell-component-utils.h"
 #include "evolution-storage-set-view-factory.h"
 
 #include "e-shell.h"
@@ -87,7 +92,7 @@ struct _EShellPrivate {
 
 	EStorageSet *storage_set;
 	ELocalStorage *local_storage;
-	ESummaryStorage *summary_storage;
+	EStorage *summary_storage;
 
 	EShortcuts *shortcuts;
 	EFolderTypeRegistry *folder_type_registry;
@@ -116,6 +121,9 @@ struct _EShellPrivate {
 	/* Line status.  */
 	EShellLineStatus line_status;
 
+	/* Settings Dialog */
+	GtkWidget *settings_dialog;
+	
 	/* Configuration Database */
 	Bonobo_ConfigDatabase db;
 
@@ -128,6 +136,10 @@ struct _EShellPrivate {
 	   (Currently, it's interactive IIF there is at least one active
 	   view.)  */
 	unsigned int is_interactive : 1;
+
+	/* Whether quit has been requested, and the shell is now waiting for
+	   permissions from all the components to quit.  */
+	unsigned int preparing_to_quit : 1;
 };
 
 
@@ -142,11 +154,31 @@ struct _EShellPrivate {
 enum {
 	NO_VIEWS_LEFT,
 	LINE_STATUS_CHANGED,
+	NEW_VIEW_CREATED,
 	LAST_SIGNAL
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
+
+/* Utility functions.  */
+
+static void
+pop_up_activation_error_dialog (ESplash *splash,
+				const char *id,
+				CORBA_Environment *ev)
+{
+	char *error_message;
+
+	error_message = e_get_activation_failure_msg (ev);
+	e_notice (GTK_WINDOW (splash), GNOME_MESSAGE_BOX_ERROR,
+		  _("Cannot activate component %s :\n"
+		    "The error from the activation system is:\n"
+		    "%s"),
+		  id, error_message);
+	g_free (error_message);
+}
+
 
 /* Interactivity handling.  */
 
@@ -219,8 +251,7 @@ folder_selection_dialog_folder_selected_cb (EShellFolderSelectionDialog *folder_
 	GNOME_Evolution_FolderSelectionListener listener;
 	EStorageSet *storage_set;
 	EFolder *folder;
-	char *uri;
-	const char *physical_uri;
+	GNOME_Evolution_Folder corba_folder;
 
 	shell = E_SHELL (data);
 	listener = gtk_object_get_data (GTK_OBJECT (folder_selection_dialog), "corba_listener");
@@ -230,15 +261,32 @@ folder_selection_dialog_folder_selected_cb (EShellFolderSelectionDialog *folder_
 	storage_set = e_shell_get_storage_set (shell);
 	folder = e_storage_set_get_folder (storage_set, path);
 
-	uri = g_strconcat (E_SHELL_URI_PREFIX, path, NULL);
+	if (folder == NULL) {
+		corba_folder.type = "";
+		corba_folder.description = "";
+		corba_folder.displayName = "";
+		corba_folder.physicalUri = "";
+		corba_folder.evolutionUri = "";
+		corba_folder.unreadCount = -1;
+		corba_folder.customIconName = "";
+	} else {
+		corba_folder.type = (CORBA_char *)e_folder_get_type_string (folder);
+		corba_folder.description = (CORBA_char *)e_folder_get_description (folder);
+		if (corba_folder.description == NULL)
+			corba_folder.description = "";
+		corba_folder.displayName = (CORBA_char *)e_folder_get_name (folder);
+		corba_folder.physicalUri = (CORBA_char *)e_folder_get_physical_uri (folder);
+		if (corba_folder.physicalUri == NULL)
+			corba_folder.physicalUri = "";
+		corba_folder.evolutionUri = (CORBA_char *)g_strconcat (E_SHELL_URI_PREFIX, path, NULL);
+		corba_folder.unreadCount = e_folder_get_unread_count (folder);
+		corba_folder.customIconName = (CORBA_char *)e_folder_get_custom_icon_name (folder);
+		if (corba_folder.customIconName == NULL)
+			corba_folder.customIconName = "";
+	}
 
-	if (folder == NULL)
-		physical_uri = "";
-	else
-		physical_uri = e_folder_get_physical_uri (folder);
-
-	GNOME_Evolution_FolderSelectionListener_notifySelected (listener, uri, physical_uri, &ev);
-	g_free (uri);
+	GNOME_Evolution_FolderSelectionListener_notifySelected (listener, &corba_folder, &ev);
+	g_free (corba_folder.evolutionUri);
 
 	CORBA_exception_free (&ev);
 
@@ -316,6 +364,36 @@ impl_Shell_getComponentByType (PortableServer_Servant servant,
 	return CORBA_Object_duplicate (corba_component, ev);
 }
 
+static GNOME_Evolution_Icon *
+impl_Shell_getIconByType (PortableServer_Servant servant,
+			  const CORBA_char *type,
+			  const CORBA_boolean mini,
+			  CORBA_Environment *ev)
+{
+	BonoboObject *bonobo_object;
+	EFolderTypeRegistry *folder_type_registry;
+	GdkPixbuf *pixbuf;
+	GNOME_Evolution_Icon *icon;
+	EShell *shell;
+
+	if (raise_exception_if_not_ready (servant, ev))
+		return CORBA_OBJECT_NIL;
+
+	bonobo_object = bonobo_object_from_servant (servant);
+	shell = E_SHELL (bonobo_object);
+	folder_type_registry = shell->priv->folder_type_registry;
+
+	pixbuf = e_folder_type_registry_get_icon_for_type (folder_type_registry, type, mini);
+
+	if (pixbuf == NULL) {
+		CORBA_exception_set (ev, CORBA_USER_EXCEPTION, ex_GNOME_Evolution_Shell_NotFound, NULL);
+		return CORBA_OBJECT_NIL;
+	}
+
+	icon = e_new_corba_icon_from_pixbuf (pixbuf);
+	return icon;
+}
+
 static GNOME_Evolution_ShellView
 impl_Shell_createNewView (PortableServer_Servant servant,
 			  const CORBA_char *uri,
@@ -339,7 +417,7 @@ impl_Shell_createNewView (PortableServer_Servant servant,
 		return CORBA_OBJECT_NIL;
 	}
 
-	shell_view = e_shell_create_view (shell, uri, NULL);
+	shell_view = e_shell_create_view_from_uri_and_settings (shell, uri, 0);
 	if (shell_view == NULL) {
 		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
 				     ex_GNOME_Evolution_Shell_NotFound, NULL);
@@ -374,8 +452,9 @@ impl_Shell_handleURI (PortableServer_Servant servant,
 	shell = E_SHELL (bonobo_object_from_servant (servant));
 	priv = shell->priv;
 
-	if (strncmp (uri, E_SHELL_URI_PREFIX, E_SHELL_URI_PREFIX_LEN) == 0) {
-		GNOME_Evolution_Shell_createNewView (bonobo_object_corba_objref (BONOBO_OBJECT (shell)), uri, ev);
+	if (strncmp (uri, E_SHELL_URI_PREFIX, E_SHELL_URI_PREFIX_LEN) == 0
+	    || strncmp (uri, E_SHELL_DEFAULTURI_PREFIX, E_SHELL_DEFAULTURI_PREFIX_LEN) == 0) {
+		e_shell_create_view_from_uri_and_settings (shell, uri, 0);
 		return;
 	}
 
@@ -427,7 +506,6 @@ impl_Shell_selectUserFolder (PortableServer_Servant servant,
 			     const CORBA_char *title,
 			     const CORBA_char *default_folder,
 			     const GNOME_Evolution_Shell_FolderTypeNameList *corba_allowed_type_names,
-			     const CORBA_char *default_type,
 			     CORBA_Environment *ev)
 {
 	GtkWidget *folder_selection_dialog;
@@ -455,8 +533,7 @@ impl_Shell_selectUserFolder (PortableServer_Servant servant,
 								       title,
 								       NULL,
 								       default_folder,
-								       allowed_type_names,
-								       default_type);
+								       allowed_type_names);
 
 
 	listener_duplicate = CORBA_Object_duplicate (listener, ev);
@@ -538,8 +615,9 @@ impl_Shell_createStorageSetView (PortableServer_Servant servant,
 				 CORBA_Environment *ev)
 {
 	BonoboObject *bonobo_object;
-	EShell *shell;
 	BonoboControl *control;
+	CORBA_Object dup_objref;
+	EShell *shell;
 
 	if (raise_exception_if_not_ready (servant, ev))
 		return CORBA_OBJECT_NIL;
@@ -548,8 +626,9 @@ impl_Shell_createStorageSetView (PortableServer_Servant servant,
 	shell = E_SHELL (bonobo_object);
 
 	control = evolution_storage_set_view_factory_new_view (shell);
+	dup_objref = CORBA_Object_duplicate (bonobo_object_corba_objref (BONOBO_OBJECT (control)), ev);
 
-	return bonobo_object_corba_objref (BONOBO_OBJECT (control));
+	return dup_objref;
 }
 
 static void
@@ -639,6 +718,7 @@ setup_local_storage (EShell *shell)
 	EStorage *local_storage;
 	EShellPrivate *priv;
 	gchar *local_storage_path;
+	EFolder *summary_folder;
 
 	priv = shell->priv;
 
@@ -657,8 +737,11 @@ setup_local_storage (EShell *shell)
 	e_storage_set_add_storage (priv->storage_set, local_storage);
 	priv->local_storage = E_LOCAL_STORAGE (local_storage);
 
-	priv->summary_storage = E_SUMMARY_STORAGE (e_summary_storage_new ());
-	e_storage_set_add_storage (priv->storage_set, E_STORAGE (priv->summary_storage));
+	summary_folder = e_folder_new (U_("Summary"), "summary", "");
+	e_folder_set_physical_uri (summary_folder, "/");
+	priv->summary_storage = e_storage_new (E_SUMMARY_STORAGE_NAME,
+					       summary_folder);
+	e_storage_set_add_storage (priv->storage_set, priv->summary_storage);
 
 	return TRUE;
 }
@@ -674,7 +757,7 @@ get_icon_path_for_component_info (const OAF_ServerInfo *info)
 
 	/* FIXME: liboaf is not const-safe.  */
 	property = oaf_server_info_prop_find ((OAF_ServerInfo *) info,
-					      "evolution:shell-component-icon");
+					      "evolution:shell_component_icon");
 
 	if (property == NULL || property->v._d != OAF_P_STRING)
 		return gnome_pixmap_file ("gnome-question.png");
@@ -693,6 +776,7 @@ setup_components (EShell *shell,
 		  ESplash *splash)
 {
 	EShellPrivate *priv;
+	char *const selection_order[] = { "0-evolution:shell_component_launch_order", NULL };
 	OAF_ServerInfoList *info_list;
 	CORBA_Environment ev;
 	int i;
@@ -702,7 +786,7 @@ setup_components (EShell *shell,
 	priv = shell->priv;
 	priv->component_registry = e_component_registry_new (shell);
 
-	info_list = oaf_query ("repo_ids.has ('IDL:GNOME/Evolution/ShellComponent:1.0')", NULL, &ev);
+	info_list = oaf_query ("repo_ids.has ('IDL:GNOME/Evolution/ShellComponent:1.0')", selection_order, &ev);
 
 	if (ev._major != CORBA_NO_EXCEPTION)
 		g_error ("Eeek!  Cannot perform OAF query for Evolution components.");
@@ -734,17 +818,23 @@ setup_components (EShell *shell,
 
 	for (i = 0; i < info_list->_length; i++) {
 		const OAF_ServerInfo *info;
+		CORBA_Environment ev;
 
 		info = info_list->_buffer + i;
 
-		if (! e_component_registry_register_component (priv->component_registry, info->iid)) {
-			g_warning ("Cannot activate Evolution component -- %s", info->iid);
+		CORBA_exception_init (&ev);
+
+		if (! e_component_registry_register_component (priv->component_registry, info->iid, &ev)) {
+			pop_up_activation_error_dialog (splash, info->iid, &ev);
 		} else {
 			e_shell_user_creatable_items_handler_add_component
 				(priv->user_creatable_items_handler,
+				 info->iid,
 				 e_component_registry_get_component_by_id (priv->component_registry, info->iid));
 		}
 
+		CORBA_exception_free (&ev);
+
 		if (splash != NULL)
 			e_splash_set_icon_highlight (splash, i, TRUE);
 
@@ -759,7 +849,8 @@ setup_components (EShell *shell,
 
 /* FIXME what if anything fails here?  */
 static void
-set_owner_on_components (EShell *shell)
+set_owner_on_components (EShell *shell,
+			 ESplash *splash)
 {
 	GNOME_Evolution_Shell corba_shell;
 	EShellPrivate *priv;
@@ -787,14 +878,26 @@ set_owner_on_components (EShell *shell)
 				   id, evolution_shell_component_result_to_string (result));
 
 			if (result == EVOLUTION_SHELL_COMPONENT_OLDOWNERHASDIED) {
-				component_client = e_component_registry_restart_component (priv->component_registry, id);
-				result = evolution_shell_component_client_set_owner (component_client, corba_shell,
-										     local_directory);
-				if (result != EVOLUTION_SHELL_COMPONENT_OK) {
-					g_warning ("Error re-setting owner on component %s -- %s",
-						   id, evolution_shell_component_result_to_string (result));
-					/* (At this point, we give up.)  */
+				CORBA_Environment ev;
+
+				CORBA_exception_init (&ev);
+
+				component_client = e_component_registry_restart_component (priv->component_registry,
+											   id, &ev);
+
+				if (component_client == NULL) {
+					pop_up_activation_error_dialog (splash, id, &ev);
+				} else {
+					result = evolution_shell_component_client_set_owner (component_client, corba_shell,
+											     local_directory);
+					if (result != EVOLUTION_SHELL_COMPONENT_OK) {
+						g_warning ("Error re-setting owner on component %s -- %s",
+							   id, evolution_shell_component_result_to_string (result));
+						/* (At this point, we give up.)  */
+					}
 				}
+
+				CORBA_exception_free (&ev);
 			}
 		}
 	}
@@ -817,8 +920,20 @@ view_delete_event_cb (GtkWidget *widget,
 	shell = E_SHELL (data);
 	e_shell_save_settings (shell);
 
-	/* Destroy it */
-	return FALSE;
+	if (g_list_length (shell->priv->views) != 1) {
+		/* If it's not the last view, just destroy it.  */
+		return FALSE;
+	}
+
+	if (shell->priv->preparing_to_quit)
+		return TRUE;
+
+	/* If it's the last view, ask for confirm before actually destroying
+	   this view.  */
+	if (e_shell_prepare_for_quit (shell))
+		return FALSE;
+	else
+		return TRUE;
 }
 
 static void
@@ -834,9 +949,9 @@ view_destroy_cb (GtkObject *object,
 
 	num_views = g_list_length (shell->priv->views);
 
-	/* If this is our last view, save settings now because in the
-	   callback for no_views_left shell->priv->views will be NULL
-	   and settings won't be saved because of that */
+	/* If this is our last view, save settings now because in the callback
+	   for no_views_left shell->priv->views will be NULL and settings won't
+	   be saved because of that.  */
 	if (num_views - 1 == 0)
 		e_shell_save_settings (shell);
 
@@ -869,13 +984,8 @@ create_view (EShell *shell,
 	gtk_signal_connect (GTK_OBJECT (view), "destroy",
 			    GTK_SIGNAL_FUNC (view_destroy_cb), shell);
 
-	if (uri != NULL) {
-		if (!e_shell_view_display_uri (E_SHELL_VIEW (view), uri)) {
-			/* FIXME: Consider popping a dialog box up about how the provided URI does not
-			   exist/could not be displayed.  */
-			e_shell_view_display_uri (E_SHELL_VIEW (view), E_SHELL_VIEW_DEFAULT_URI);
-		}
-	}
+	if (uri != NULL)
+		e_shell_view_display_uri (E_SHELL_VIEW (view), uri, TRUE);
 
 	shell->priv->views = g_list_prepend (shell->priv->views, view);
 
@@ -887,6 +997,8 @@ create_view (EShell *shell,
 		e_shell_view_show_shortcut_bar (view, e_shell_view_shortcut_bar_shown (template_view));
 	}
 
+	gtk_signal_emit (GTK_OBJECT (shell), signals[NEW_VIEW_CREATED], view);
+
 	return view;
 }
 
@@ -1002,11 +1114,21 @@ class_init (EShellClass *klass)
 				GTK_TYPE_NONE, 1,
 				GTK_TYPE_ENUM);
 
+	signals[NEW_VIEW_CREATED] =
+		gtk_signal_new ("new_view_created",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (EShellClass, new_view_created),
+				gtk_marshal_NONE__POINTER,
+				GTK_TYPE_NONE, 1,
+				GTK_TYPE_POINTER);
+
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 
 	epv = & klass->epv;
 	epv->_get_displayName     = impl_Shell__get_displayName;
 	epv->getComponentByType   = impl_Shell_getComponentByType;
+	epv->getIconByType        = impl_Shell_getIconByType;
 	epv->createNewView        = impl_Shell_createNewView;
 	epv->handleURI            = impl_Shell_handleURI;
 	epv->selectUserFolder     = impl_Shell_selectUserFolder;
@@ -1039,10 +1161,12 @@ init (EShell *shell)
 	priv->corba_shortcuts              = NULL;
 	priv->offline_handler              = NULL;
 	priv->crash_type_names             = NULL;
-	priv->line_status                  = E_SHELL_LINE_STATUS_ONLINE;
+	priv->line_status                  = E_SHELL_LINE_STATUS_OFFLINE;
+	priv->settings_dialog              = NULL;
 	priv->db                           = CORBA_OBJECT_NIL;
 	priv->is_initialized               = FALSE;
 	priv->is_interactive               = FALSE;
+	priv->preparing_to_quit            = FALSE;
 
 	shell->priv = priv;
 }
@@ -1054,6 +1178,7 @@ init (EShell *shell)
  * @iid: OAFIID for registering the shell into the name server
  * @local_directory: Local directory for storing local information and folders
  * @show_splash: Whether to display a splash screen.
+ * @startup_line_mode: How to set up the line mode (online or offline) initally.
  * 
  * Construct @shell so that it uses the specified @local_directory and
  * @corba_object.
@@ -1064,26 +1189,42 @@ EShellConstructResult
 e_shell_construct (EShell *shell,
 		   const char *iid,
 		   const char *local_directory,
-		   gboolean show_splash)
+		   gboolean show_splash,
+		   EShellStartupLineMode startup_line_mode)
 {
 	GtkWidget *splash;
 	EShellPrivate *priv;
 	CORBA_Object corba_object;
 	CORBA_Environment ev;
 	gchar *shortcut_path;
+	gboolean start_online;
+
 	g_return_val_if_fail (shell != NULL, E_SHELL_CONSTRUCT_RESULT_INVALIDARG);
 	g_return_val_if_fail (E_IS_SHELL (shell), E_SHELL_CONSTRUCT_RESULT_INVALIDARG);
 	g_return_val_if_fail (local_directory != NULL, E_SHELL_CONSTRUCT_RESULT_INVALIDARG);
 	g_return_val_if_fail (g_path_is_absolute (local_directory), E_SHELL_CONSTRUCT_RESULT_INVALIDARG);
+	g_return_val_if_fail (startup_line_mode == E_SHELL_STARTUP_LINE_MODE_CONFIG
+			      || startup_line_mode == E_SHELL_STARTUP_LINE_MODE_ONLINE
+			      || startup_line_mode == E_SHELL_STARTUP_LINE_MODE_OFFLINE,
+			      E_SHELL_CONSTRUCT_RESULT_INVALIDARG);
 	
 	priv = shell->priv;
-	
+
 	priv->iid                  = g_strdup (iid);
 	priv->local_directory      = g_strdup (local_directory);
 	priv->folder_type_registry = e_folder_type_registry_new ();
 	priv->uri_schema_registry  = e_uri_schema_registry_new ();
 	priv->storage_set          = e_storage_set_new (priv->folder_type_registry);
 	
+	e_folder_type_registry_register_type (priv->folder_type_registry,
+					      "noselect", "empty.gif",
+					      "noselect", "", FALSE,
+					      0, NULL, 0, NULL);
+	e_folder_type_registry_register_type (priv->folder_type_registry,
+					      "working", "working-16.png",
+					      "working", "", FALSE,
+					      0, NULL, 0, NULL);
+
 	/* CORBA storages must be set up before the components, because otherwise components
            cannot register their own storages.  */
 	if (! setup_corba_storages (shell))
@@ -1103,6 +1244,7 @@ e_shell_construct (EShell *shell,
 		CORBA_exception_free (&ev);
 		return E_SHELL_CONSTRUCT_RESULT_NOCONFIGDB;
  	}
+	e_setup_check_db (priv->db, local_directory);
 	
 	CORBA_exception_free (&ev);
 
@@ -1138,9 +1280,7 @@ e_shell_construct (EShell *shell,
 	/* Set up the shortcuts.  */
 	
 	shortcut_path = g_concat_dir_and_file (local_directory, "shortcuts.xml");
-	priv->shortcuts = e_shortcuts_new (priv->storage_set,
-					   priv->folder_type_registry,
-					   shortcut_path);
+	priv->shortcuts = e_shortcuts_new_from_file (shell, shortcut_path);
 	g_assert (priv->shortcuts != NULL);
 	
 	if (e_shortcuts_get_num_groups (priv->shortcuts) == 0)
@@ -1162,11 +1302,10 @@ e_shell_construct (EShell *shell,
 	
 	/* Now that we have a local storage and all the interfaces set up, we
 	   can tell the components we are here.  */
-	set_owner_on_components (shell);
+	set_owner_on_components (shell, E_SPLASH (splash));
 	
-	if (show_splash) {
+	if (show_splash)
 		gtk_widget_destroy (splash);
-	}
 	
 	if (e_shell_startup_wizard_create () == FALSE) {
 		e_shell_unregister_all (shell);
@@ -1177,6 +1316,24 @@ e_shell_construct (EShell *shell,
 
 	priv->is_initialized = TRUE;
 
+	switch (startup_line_mode) {
+	case E_SHELL_STARTUP_LINE_MODE_CONFIG:
+		start_online = ! bonobo_config_get_boolean_with_default (priv->db, "/Shell/StartOffline", FALSE, NULL);
+		break;
+	case E_SHELL_STARTUP_LINE_MODE_ONLINE:
+		start_online = TRUE;
+		break;
+	case E_SHELL_STARTUP_LINE_MODE_OFFLINE:
+		start_online = FALSE;
+		break;
+	default:
+		start_online = FALSE; /* Make compiler happy.  */
+		g_assert_not_reached ();
+	}
+
+	if (start_online)
+		e_shell_go_online (shell, NULL);
+
 	return E_SHELL_CONSTRUCT_RESULT_OK;
 }
 
@@ -1184,6 +1341,7 @@ e_shell_construct (EShell *shell,
  * e_shell_new:
  * @local_directory: Local directory for storing local information and folders.
  * @show_splash: Whether to display a splash screen.
+ * @start_online: Whether to start in on-line mode or not.
  * @construct_result_return: A pointer to an EShellConstructResult variable into
  * which the result of the operation will be stored.
  * 
@@ -1194,6 +1352,7 @@ e_shell_construct (EShell *shell,
 EShell *
 e_shell_new (const char *local_directory,
 	     gboolean show_splash,
+	     EShellStartupLineMode startup_line_mode,
 	     EShellConstructResult *construct_result_return)
 {
 	EShell *new;
@@ -1205,7 +1364,11 @@ e_shell_new (const char *local_directory,
 
 	new = gtk_type_new (e_shell_get_type ());
 
-	construct_result = e_shell_construct (new, E_SHELL_OAFIID, local_directory, show_splash);
+	construct_result = e_shell_construct (new,
+					      E_SHELL_OAFIID,
+					      local_directory,
+					      show_splash,
+					      startup_line_mode);
 
 	if (construct_result != E_SHELL_CONSTRUCT_RESULT_OK) {
 		*construct_result_return = construct_result;
@@ -1260,20 +1423,17 @@ e_shell_create_view (EShell *shell,
 }
 
 EShellView *
-e_shell_create_view_from_settings (EShell *shell,
-				   const char *uri,
-				   EShellView *template_view,
-				   int view_num,
-				   gboolean *settings_found)
+e_shell_create_view_from_uri_and_settings (EShell *shell,
+					   const char *uri,
+					   int view_num)
 {
 	EShellView *view;
 
 	g_return_val_if_fail (shell != NULL, NULL);
 	g_return_val_if_fail (E_IS_SHELL (shell), NULL);
 
-	view = create_view (shell, uri, template_view);
-
-	*settings_found = e_shell_view_load_settings (view, view_num);
+	view = create_view (shell, uri, NULL);
+	e_shell_view_load_settings (view, view_num);
 
 	gtk_widget_show (GTK_WIDGET (view));
 	while (gtk_events_pending ())
@@ -1489,6 +1649,20 @@ save_settings_for_components (EShell *shell)
 	return retval;
 }
 
+static gboolean
+save_misc_settings (EShell *shell)
+{
+	EShellPrivate *priv;
+	gboolean is_offline;
+
+	priv = shell->priv;
+
+	is_offline = ( e_shell_get_line_status (shell) == E_SHELL_LINE_STATUS_OFFLINE );
+	bonobo_config_set_boolean (priv->db, "/Shell/StartOffline", is_offline, NULL);
+
+	return TRUE;
+}
+
 /**
  * e_shell_save_settings:
  * @shell: 
@@ -1504,31 +1678,35 @@ e_shell_save_settings (EShell *shell)
 {
 	gboolean views_saved;
 	gboolean components_saved;
+	gboolean misc_saved;
 
 	g_return_val_if_fail (shell != NULL, FALSE);
 	g_return_val_if_fail (E_IS_SHELL (shell), FALSE);
 
 	views_saved      = save_settings_for_views (shell);
 	components_saved = save_settings_for_components (shell);
+	misc_saved       = save_misc_settings (shell);
 
-	return views_saved && components_saved;
+	return views_saved && components_saved && misc_saved;
 }
 
 /**
  * e_shell_restore_from_settings:
  * @shell: An EShell object.
+ * @restore_all_views: whether to restore all the views
  * 
  * Restore the existing views from the saved configuration.  The shell must
- * have no views for this to work.
+ * have no views for this to work.  If @restore_all_views is TRUE, restore all
+ * the views; otherwise, just the first one.
  * 
  * Return value: %FALSE if the shell has some open views or there is no saved
  * configuration.  %TRUE if the configuration could be restored successfully.
  **/
 gboolean
-e_shell_restore_from_settings (EShell *shell)
+e_shell_restore_from_settings (EShell *shell,
+			       gboolean restore_all_views)
 {
 	EShellPrivate *priv;
-	gboolean retval;
 	int num_views;
 	int i;
 
@@ -1540,21 +1718,14 @@ e_shell_restore_from_settings (EShell *shell)
 
 	num_views = bonobo_config_get_long_with_default (priv->db, "/Shell/Views/NumberOfViews", 0, NULL);
 
-	if (num_views == 0)
-		return FALSE;
-	
-	retval = TRUE;
-
 	for (i = 0; i < num_views; i++) {
-		EShellView *view;
-		gboolean settings_found;
+		e_shell_create_view_from_uri_and_settings (shell, NULL, i);
 
-		view = e_shell_create_view_from_settings (shell, NULL, NULL, i, &settings_found);
-		if (! settings_found)
-			retval = FALSE;
+		if (! restore_all_views)
+			break;
 	}
 
-	return retval;
+	return (num_views > 0);
 }
 
 /**
@@ -1746,7 +1917,7 @@ e_shell_go_offline (EShell *shell,
 
 	g_assert (priv->offline_handler == NULL);
 
-	priv->offline_handler = e_shell_offline_handler_new (priv->component_registry);
+	priv->offline_handler = e_shell_offline_handler_new (shell);
 
 	gtk_signal_connect (GTK_OBJECT (priv->offline_handler), "offline_procedure_started",
 			    GTK_SIGNAL_FUNC (offline_procedure_started_cb), shell);
@@ -1810,11 +1981,85 @@ e_shell_go_online (EShell *shell,
 	gtk_signal_emit (GTK_OBJECT (shell), signals[LINE_STATUS_CHANGED], priv->line_status);
 }
 
+
+void
+e_shell_send_receive (EShell *shell)
+{
+	EShellPrivate *priv;
+	GList *id_list;
+	GList *p;
+
+	g_return_if_fail (E_IS_SHELL (shell));
+
+	priv = shell->priv;
+
+	id_list = e_component_registry_get_id_list (priv->component_registry);
+
+	for (p = id_list; p != NULL; p = p->next) {
+		EvolutionShellComponentClient *component_client;
+		CORBA_Environment ev;
+		const char *id;
+
+		id = (const char *) p->data;
+		component_client = e_component_registry_get_component_by_id (priv->component_registry, id);
+
+		CORBA_exception_init (&ev);
+
+		GNOME_Evolution_ShellComponent_sendReceive
+			(bonobo_object_corba_objref (BONOBO_OBJECT (component_client)), TRUE, &ev);
+
+		if (BONOBO_EX (&ev))
+			g_warning ("Error invoking Send/Receive on %s -- %s", id, BONOBO_EX_ID (&ev));
+
+		CORBA_exception_free (&ev);
+	}
+
+	e_free_string_list (id_list);
+}
+
+static void
+settings_dialog_destroy_cb (GtkWidget *widget, void *data)
+{
+	EShell *shell;
+	EShellPrivate *priv;
+	
+	shell = E_SHELL (data);
+	priv = shell->priv;
+	
+	priv->settings_dialog = NULL;
+}
+
+void
+e_shell_show_settings (EShell *shell, const char *type, EShellView *shell_view)
+{
+	EShellPrivate *priv;
+	
+	g_return_if_fail (shell != NULL);
+	g_return_if_fail (E_IS_SHELL (shell));
+	g_return_if_fail (type != NULL);
+
+	priv = shell->priv;
+	
+	if (priv->settings_dialog != NULL) {
+		gdk_window_show (priv->settings_dialog->window);
+		gtk_widget_grab_focus (priv->settings_dialog);
+		return;
+	}
+	
+	priv->settings_dialog = e_shell_settings_dialog_new ();
+	e_shell_settings_dialog_show_type (E_SHELL_SETTINGS_DIALOG (priv->settings_dialog), type);
+
+	gtk_signal_connect (GTK_OBJECT (priv->settings_dialog), "destroy",
+			    GTK_SIGNAL_FUNC (settings_dialog_destroy_cb), shell);
+
+	gtk_widget_show (priv->settings_dialog);
+}
+
 
 Bonobo_ConfigDatabase 
 e_shell_get_config_db (EShell *shell)
 {
-	g_return_val_if_fail (shell != NULL, CORBA_OBJECT_NIL);
+	g_return_val_if_fail (E_IS_SHELL (shell), CORBA_OBJECT_NIL);
 
 	return shell->priv->db;
 }
@@ -1893,6 +2138,170 @@ e_shell_construct_result_to_string (EShellConstructResult result)
 	}
 }
 
+
+static void
+prepare_for_quit_callback (EvolutionShellComponentClient *client,
+			   EvolutionShellComponentResult result,
+			   void *data)
+{
+	GNOME_Evolution_ShellComponentListener_Result *result_return;
+
+	result_return = (GNOME_Evolution_ShellComponentListener_Result *) data;
+	*result_return = result;
+}
+
+gboolean
+e_shell_prepare_for_quit (EShell *shell)
+{
+	EShellPrivate *priv;
+	GList *component_ids;
+	GList *p;
+	gboolean retval;
+
+	g_return_val_if_fail (E_IS_SHELL (shell), FALSE);
+
+	priv = shell->priv;
+	priv->preparing_to_quit = TRUE;
+
+	/* Make all the views insensitive so we have some modal-like
+	   behavior.  */
+	for (p = priv->views; p != NULL; p = p->next)
+		gtk_widget_set_sensitive (GTK_WIDGET (p->data), FALSE);
+
+	component_ids = e_component_registry_get_id_list (priv->component_registry);
+
+	for (p = component_ids; p != NULL; p = p->next) {
+		EvolutionShellComponentClient *client;
+		const char *id;
+		GNOME_Evolution_ShellComponentListener_Result result;
+
+		id = (const char *) p->data;
+		client = e_component_registry_get_component_by_id (priv->component_registry, id);
+
+		result = (GNOME_Evolution_ShellComponentListener_Result) -1;
+
+		evolution_shell_component_client_request_quit (client, prepare_for_quit_callback, &result);
+
+		while (result == (GNOME_Evolution_ShellComponentListener_Result) -1)
+			gtk_main_iteration ();
+
+		if (result == GNOME_Evolution_ShellComponentListener_CANCEL) {
+			retval = FALSE;
+			goto end;
+		}
+	}
+
+	retval = TRUE;
+
+ end:
+	/* Restore all the views to be sensitive.  */
+	for (p = priv->views; p != NULL; p = p->next)
+		gtk_widget_set_sensitive (GTK_WIDGET (p->data), TRUE);
+	priv->preparing_to_quit = FALSE;
+
+	e_free_string_list (component_ids);
+	return retval;
+}
+
+
+/* URI parsing.   */
+
+static gboolean
+parse_default_uri (EShell *shell,
+		   const char *uri,
+		   char **path_return,
+		   char **extra_return)
+{
+	const char *component_start;
+	const char *component;
+	const char *p;
+	char *db_path;
+	char *path;
+	gboolean is_default;
+
+	component_start = uri + E_SHELL_DEFAULTURI_PREFIX_LEN;
+	p = strchr (uri, '#');
+
+	if (p == NULL)
+		component = g_strdup (component_start);
+	else
+		component = g_strndup (component_start, p - component_start);
+
+	db_path = g_strdup_printf ("/DefaultFolders/%s_path", component);
+	path = bonobo_config_get_string_with_default (e_shell_get_config_db (shell),
+						      db_path, NULL, &is_default);
+
+	/* We expect an evolution: URI here, if we don't get it then something
+	   is messed up.  */
+	if (strncmp (path, E_SHELL_URI_PREFIX, E_SHELL_URI_PREFIX_LEN) != 0 || is_default) {
+		g_free (path);
+		if (path_return != NULL)
+			*path_return = NULL;
+		if (extra_return != NULL)
+			*extra_return = NULL;
+		return FALSE;
+	}
+
+	if (path_return != NULL)
+		*path_return = g_strdup (path + E_SHELL_URI_PREFIX_LEN);
+
+	if (extra_return != NULL) {
+		if (p == NULL)
+			*extra_return = NULL;
+		else
+			*extra_return = g_strdup (p + 1);
+	}
+
+	return TRUE;
+}
+
+static gboolean
+parse_evolution_uri (EShell *shell,
+		     const char *uri,
+		     char **path_return,
+		     char **extra_return)
+{
+	const char *path_start;
+	const char *p;
+
+	path_start = uri + E_SHELL_URI_PREFIX_LEN;
+	p = strchr (path_start, '#');
+
+	if (p != NULL && path_return != NULL)
+		*path_return = g_strndup (path_start, p - path_start);
+	else
+		*path_return = g_strdup (path_start);
+
+	if (extra_return != NULL) {
+		if (p == NULL)
+			*extra_return = NULL;
+		else
+			*extra_return = g_strdup (p + 1);
+	}
+
+	return TRUE;
+}
+
+gboolean
+e_shell_parse_uri (EShell *shell,
+		   const char *uri,
+		   char **path_return,
+		   char **extra_return)
+{
+	g_return_val_if_fail (uri != NULL, FALSE);
+
+	if (strncmp (uri, E_SHELL_DEFAULTURI_PREFIX, E_SHELL_DEFAULTURI_PREFIX_LEN) == 0)
+		return parse_default_uri (shell, uri, path_return, extra_return);
+
+	if (strncmp (uri, E_SHELL_URI_PREFIX, E_SHELL_URI_PREFIX_LEN) == 0)
+		return parse_evolution_uri (shell, uri, path_return, extra_return);
+
+	*path_return = NULL;
+	if (extra_return != NULL)
+		*extra_return =  NULL;
+	return FALSE;
+}
+
 
 E_MAKE_X_TYPE (e_shell, "EShell", EShell,
 	       class_init, init, PARENT_TYPE,
