@@ -4,9 +4,8 @@
  * Copyright (C) 2000  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +24,10 @@
 #include <config.h>
 #endif
 
-#include <string.h>
+#include "e-storage-set.h"
+
+#include "e-storage-set-view.h"
+#include "e-shell-constants.h"
 
 #include <glib.h>
 #include <gtk/gtkobject.h>
@@ -34,8 +36,7 @@
 
 #include <gal/util/e-util.h>
 
-#include "e-storage-set-view.h"
-#include "e-storage-set.h"
+#include <string.h>
 
 
 #define PARENT_TYPE GTK_TYPE_OBJECT
@@ -62,6 +63,7 @@ enum {
 	NEW_FOLDER,
 	UPDATED_FOLDER,
 	REMOVED_FOLDER,
+	CLOSE_FOLDER,
 	LAST_SIGNAL
 };
 
@@ -154,11 +156,14 @@ make_full_path (EStorage *storage,
 
 	storage_name = e_storage_get_name (storage);
 
-	if (! g_path_is_absolute (path))
-		full_path = g_strconcat (G_DIR_SEPARATOR_S, storage_name,
-					 G_DIR_SEPARATOR_S, path, NULL);
+	if (strcmp (path, E_PATH_SEPARATOR_S) == 0)
+		full_path = g_strconcat (E_PATH_SEPARATOR_S, storage_name,
+					 NULL);
+	else if (! g_path_is_absolute (path))
+		full_path = g_strconcat (E_PATH_SEPARATOR_S, storage_name,
+					 E_PATH_SEPARATOR_S, path, NULL);
 	else
-		full_path = g_strconcat (G_DIR_SEPARATOR_S, storage_name,
+		full_path = g_strconcat (E_PATH_SEPARATOR_S, storage_name,
 					 path, NULL);
 
 	return full_path;
@@ -209,6 +214,21 @@ storage_removed_folder_cb (EStorage *storage,
 	g_free (full_path);
 }
 
+static void
+storage_close_folder_cb (EStorage *storage,
+			 const char *path,
+			 void *data)
+{
+	EStorageSet *storage_set;
+	char *full_path;
+
+	storage_set = E_STORAGE_SET (data);
+
+	full_path = make_full_path (storage, path);
+	gtk_signal_emit (GTK_OBJECT (storage_set), signals[CLOSE_FOLDER], full_path);
+	g_free (full_path);
+}
+
 
 static EStorage *
 get_storage_for_path (EStorageSet *storage_set,
@@ -220,23 +240,24 @@ get_storage_for_path (EStorageSet *storage_set,
 	const char *first_separator;
 
 	g_return_val_if_fail (g_path_is_absolute (path), NULL);
+	g_return_val_if_fail (path[1] != E_PATH_SEPARATOR, NULL);
 
 	/* Skip initial separator.  */
 	path++;
 
-	first_separator = strchr (path, G_DIR_SEPARATOR);
+	first_separator = strchr (path, E_PATH_SEPARATOR);
 
-	if (first_separator == NULL || first_separator == path || first_separator[1] == 0) {
-		*subpath_return = NULL;
-		return NULL;
+	if (first_separator == NULL || first_separator[1] == 0) {
+		storage = e_storage_set_get_storage (storage_set, path);
+		*subpath_return = E_PATH_SEPARATOR_S;
+	} else {
+		storage_name = g_strndup (path, first_separator - path);
+		storage = e_storage_set_get_storage (storage_set, storage_name);
+		g_free (storage_name);
+
+		*subpath_return = first_separator;
 	}
 
-	storage_name = g_strndup (path, first_separator - path);
-	storage = e_storage_set_get_storage (storage_set, storage_name);
-	g_free (storage_name);
-
-	*subpath_return = first_separator;
-
 	return storage;
 }
 
@@ -254,7 +275,7 @@ signal_new_folder_for_all_folders_under_paths (EStorageSet *storage_set,
 
 		path = (const char *) p->data;
 
-		path_with_storage = g_strconcat (G_DIR_SEPARATOR_S, e_storage_get_name (storage), path, NULL);
+		path_with_storage = g_strconcat (E_PATH_SEPARATOR_S, e_storage_get_name (storage), path, NULL);
 		gtk_signal_emit (GTK_OBJECT (storage_set), signals[NEW_FOLDER], path_with_storage);
 		g_free (path_with_storage);
 
@@ -272,7 +293,7 @@ signal_new_folder_for_all_folders_in_storage (EStorageSet *storage_set,
 {
 	GList *path_list;
 
-	path_list = e_storage_get_subfolder_paths (storage, G_DIR_SEPARATOR_S);
+	path_list = e_storage_get_subfolder_paths (storage, E_PATH_SEPARATOR_S);
 
 	signal_new_folder_for_all_folders_under_paths (storage_set, storage, path_list);
 
@@ -355,6 +376,14 @@ class_init (EStorageSetClass *klass)
 				gtk_marshal_NONE__STRING,
 				GTK_TYPE_NONE, 1,
 				GTK_TYPE_STRING);
+	signals[CLOSE_FOLDER] = 
+		gtk_signal_new ("close_folder",
+				GTK_RUN_FIRST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (EStorageSetClass, close_folder),
+				gtk_marshal_NONE__STRING,
+				GTK_TYPE_NONE, 1,
+				GTK_TYPE_STRING);
 
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 }
@@ -456,6 +485,8 @@ e_storage_set_add_storage (EStorageSet *storage_set,
 			    GTK_SIGNAL_FUNC (storage_updated_folder_cb), storage_set);
 	gtk_signal_connect (GTK_OBJECT (storage), "removed_folder",
 			    GTK_SIGNAL_FUNC (storage_removed_folder_cb), storage_set);
+	gtk_signal_connect (GTK_OBJECT (storage), "close_folder",
+			    GTK_SIGNAL_FUNC (storage_close_folder_cb), storage_set);
 
 	priv->storages = g_list_append (priv->storages, storage);
 
@@ -568,15 +599,34 @@ e_storage_set_get_folder (EStorageSet *storage_set,
 }
 
 
+static void
+storage_set_view_folder_opened (EStorageSetView *storage_set_view,
+				const char *path,
+				EStorageSet *storage_set)
+{
+	EStorage *storage;
+	const char *subpath;
+
+	storage = get_storage_for_path (storage_set, path, &subpath);
+	if (storage == NULL)
+		return;
+
+	e_storage_async_open_folder (storage, subpath);
+}
+
 GtkWidget *
-e_storage_set_new_view (EStorageSet *storage_set, BonoboUIContainer *container)
+e_storage_set_create_new_view (EStorageSet *storage_set,
+			       BonoboUIContainer *ui_container)
 {
 	GtkWidget *storage_set_view;
 
 	g_return_val_if_fail (storage_set != NULL, NULL);
 	g_return_val_if_fail (E_IS_STORAGE_SET (storage_set), NULL);
 
-	storage_set_view = e_storage_set_view_new (storage_set, container);
+	storage_set_view = e_storage_set_view_new (storage_set, ui_container);
+	gtk_signal_connect (GTK_OBJECT (storage_set_view), "folder_opened",
+			    GTK_SIGNAL_FUNC (storage_set_view_folder_opened),
+			    storage_set);
 
 	return storage_set_view;
 }
@@ -716,7 +766,7 @@ e_storage_set_get_path_for_physical_uri (EStorageSet *storage_set,
 		if (storage_path != NULL) {
 			char *storage_set_path;
 
-			storage_set_path = g_strconcat (G_DIR_SEPARATOR_S,
+			storage_set_path = g_strconcat (E_PATH_SEPARATOR_S,
 							e_storage_get_name (storage),
 							storage_path,
 							NULL);
