@@ -4,9 +4,8 @@
  * Author: Michael Zucchi <notzed@ximian.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,7 +20,8 @@
 #ifndef __E_FILTER_BAR_H__
 #define __E_FILTER_BAR_H__
 
-#include <gtk/gtk.h>
+#include <gtk/gtkobject.h>
+#include <gtk/gtkwidget.h>
 #include "e-search-bar.h"
 
 #include "filter/rule-context.h"
@@ -52,22 +52,23 @@ typedef struct _EFilterBarClass  EFilterBarClass;
 
 typedef void (*EFilterBarConfigRule)(EFilterBar *, FilterRule *rule, int id, const char *query, void *data);
 
-struct _EFilterBar
-{
+struct _EFilterBar {
 	ESearchBar parent;
-
+	
 	int menu_base, option_base;
 	GPtrArray *menu_rules, *option_rules;
 
+	ESearchBarItem *default_items;
+	
 	GtkWidget *save_dialogue; /* current save dialogue (so we dont pop up multiple ones) */
-
+	
 	FilterRule *current_query; /* as it says */
 	int setquery;		/* true when we're setting a query directly to advanced, so dont popup the dialogue */
-
+	
 	RuleContext *context;
 	char *systemrules;
 	char *userrules;
-
+	
 	EFilterBarConfigRule config;
 	void *config_data;
 };
@@ -80,24 +81,33 @@ struct _EFilterBarClass
 /* "preset" items */
 enum {
 	/* preset menu options */
-	E_FILTERBAR_RESET_ID = -2,
-	E_FILTERBAR_SAVE_ID = -3,
-	E_FILTERBAR_EDIT_ID = -4,
+	E_FILTERBAR_SAVE_ID = -2,
+	E_FILTERBAR_EDIT_ID = -3,
 
 	/* preset option options */
-	E_FILTERBAR_ADVANCED_ID = -5,
-
-	E_FILTERBAR_LAST_ID = -6,
+	E_FILTERBAR_ADVANCED_ID = -4,
 };
 
-#define E_FILTERBAR_SAVE { N_("Save As ..."), E_FILTERBAR_SAVE_ID }
-#define E_FILTERBAR_RESET { N_("Show All"), E_FILTERBAR_RESET_ID }
-#define E_FILTERBAR_EDIT { N_("Edit ..."), E_FILTERBAR_EDIT_ID }
-#define E_FILTERBAR_ADVANCED { N_("Advanced ..."), E_FILTERBAR_ADVANCED_ID }
+#define E_FILTERBAR_SAVE      { N_("_Save Search..."), E_FILTERBAR_SAVE_ID, NULL }
+#define E_FILTERBAR_EDIT      { N_("_Edit Saved Searches..."), E_FILTERBAR_EDIT_ID, NULL }
+#define E_FILTERBAR_ADVANCED  { N_("_Advanced..."), E_FILTERBAR_ADVANCED_ID, NULL }
+#define E_FILTERBAR_SEPARATOR { NULL, 0, NULL }
 
-
-GtkType    e_filter_bar_get_type   (void);
-EFilterBar*e_filter_bar_new        (RuleContext *, const char *sys, const char *user, EFilterBarConfigRule config, void *data);
+#ifdef JUST_FOR_TRANSLATORS
+const char * strings[] = {
+	N_("_Save Search..."),
+	N_("_Edit Saved Searches..."),
+	N_("_Advanced...")
+};
+#endif
+
+
+GtkType     e_filter_bar_get_type (void);
+EFilterBar *e_filter_bar_new      (RuleContext *context,
+				   const char *systemrules,
+				   const char *userrules,
+				   EFilterBarConfigRule config,
+				   void *data);
 
 #ifdef __cplusplus
 }
